package de.heinerkuecker.asm_improved;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.StringUtil;

/**
 * Parser f&uuml;r Type-Informationen,
 * mit und ohne Generics,
 * anhand der von
 * <a href="asm.ow2.org">ASM</a> gelieferten
 * Strings.
 * 
 * @author Heiner K&uuml;cker
 */
final class AsmClassDescAndSignatureParser
{
	/**
	 * Konstruktor private,
	 * Instanziieren verboten,
	 * Util-Klasse.
	 */
	private AsmClassDescAndSignatureParser()
	{
		super();
	}

	public static ClassTypeparamsAndInheritTypeArgsTuple parseClassTypeparamsAndInheritTypeArgsTupel(
            final String signature ,
			final Map<String, AsmExtendsTypeParam> extendsTypeParamMap ,
			final ClasspathFiles classpathFiles ,
			final TreeMap<String, ? extends String> innerClassFileNameToDotSeparatedNameMap ,
			final FilteredForNonStaticAndSortedForOuterNameLengthReverseAsmDomInnerClasses innerClasses )
    {
    	if ( StringUtil.isNullOrEmpty( signature ) )
        {
	        return null;
        }

        final String typeParamParsedRestSignature;
    	final ImmutableArray<? extends AsmAbstractTypeParam> typeParams;
    	if ( signature.startsWith( "<" ) )
        {
    		final String typeParamStrToParse =
    				signature.substring(
    						//beginIndex
    						0 ,
    						//endIndex
    						AsmDescAndSignatureParserUtil.getAfterGtPosRecursiv(
    								signature ,
    								1 ) );

    		//System.out.println( typeParamStrToParse );

    		typeParams =
    				AsmTypeparamTypeParser.parseTypeParams(
    						typeParamStrToParse ,
    						extendsTypeParamMap ,
    						classpathFiles ,
    						innerClassFileNameToDotSeparatedNameMap ,
    						innerClasses );

    		typeParamParsedRestSignature =
    				signature.substring(
    						typeParamStrToParse.length() );
        }
    	else
    	{
    		typeParams = null;
    		typeParamParsedRestSignature = signature;
    	}

        final String superTypeArgsParsedRestSignature;
    	final ImmutableArray<? extends AsmType> superTypeArgs;
    	{
    		final int semicolonPos = typeParamParsedRestSignature.indexOf( ';' );
    		final int leftAnglePos = typeParamParsedRestSignature.indexOf( '<' );
    		
    		if ( leftAnglePos > - 1 &&
    				leftAnglePos < semicolonPos )
    			// es gibt ein '<'-Zeichen und dieses liegt vor dem Semikolon (ein Semikolon msste immer auftauchen)
    		{
    			final int afterGtPosRecursiv =
    					AsmDescAndSignatureParserUtil.getAfterGtPosRecursiv(
    							typeParamParsedRestSignature ,
    							leftAnglePos + 1 );

    			final String superTypeArgsStrToParse =
    					typeParamParsedRestSignature.substring(
    							//beginIndex
    							leftAnglePos ,
    							//endIndex
    							afterGtPosRecursiv );

    			superTypeArgs =
    					AsmTypeargsTypeParser.parseTypeArgs(
    							superTypeArgsStrToParse ,
    							extendsTypeParamMap ,
    							classpathFiles ,
    							innerClassFileNameToDotSeparatedNameMap ,
    							innerClasses );

    			superTypeArgsParsedRestSignature =
    					typeParamParsedRestSignature.substring(
    							afterGtPosRecursiv + 1 );
    		}
    		else
    		{
    			superTypeArgs = null;
    			superTypeArgsParsedRestSignature =
    					typeParamParsedRestSignature.substring(
    							semicolonPos + 1 );
    		}
    	}

    	//final Map<String, List<? extends AsmType>> interfaceTypeArgs;
    	final ImmutableArray<ImmutableArray<? extends AsmType>> interfaceTypeArgs;
    	if ( superTypeArgsParsedRestSignature.isEmpty() )
    	{
    		interfaceTypeArgs = null;
    	}
    	else
    	{
    		//final Map<String, List<? extends AsmType>> tmpInterfaceTypeArgs = new HashMap<>();
    		final List<ImmutableArray<? extends AsmType>> tmpInterfaceTypeArgs = new ArrayList<>();

    		String interfaceTypeArgsRestStr = superTypeArgsParsedRestSignature;

    		while ( ! interfaceTypeArgsRestStr.isEmpty() )
    		{
    			//System.out.println( interfaceTypeArgsRestStr );

    			final int semicolonPos = interfaceTypeArgsRestStr.indexOf( ';' );
        		final int leftAnglePos = interfaceTypeArgsRestStr.indexOf( '<' );

        		if ( leftAnglePos > - 1 &&
        				leftAnglePos < semicolonPos )
        			// es gibt ein '<'-Zeichen und dieses liegt vor dem Semikolon (ein Semikolon msste immer auftauchen)
        		{
        			final int afterGtPosRecursiv =
        					AsmDescAndSignatureParserUtil.getAfterGtPosRecursiv(
        							interfaceTypeArgsRestStr ,
        							leftAnglePos + 1 );

        			final String currInterfaceTypeArgsStrToParse =
        					interfaceTypeArgsRestStr.substring(
        							//beginIndex
        							leftAnglePos ,
        							//endIndex
        							afterGtPosRecursiv );

        			//final String interfaceClassNamePartStr =
        			//		interfaceTypeArgsRestStr.substring(
        			//				//beginIndex
        			//				1 ,
        			//				//endIndex
        			//				leftAnglePos );

        			final ImmutableArray<? extends AsmType> currInterfaceTypeArgs =
        					AsmTypeargsTypeParser.parseTypeArgs(
        							currInterfaceTypeArgsStrToParse ,
        							extendsTypeParamMap ,
        							classpathFiles ,
        							innerClassFileNameToDotSeparatedNameMap ,
        							innerClasses );

        			//final String interfaceClassFqName = interfaceClassNamePartStr.replace( '/' , '.' );
        			//tmpInterfaceTypeArgs.put( interfaceClassFqName , currInterfaceTypeArgs );
        			tmpInterfaceTypeArgs.add( currInterfaceTypeArgs );

        			interfaceTypeArgsRestStr =
        					interfaceTypeArgsRestStr.substring(
        							afterGtPosRecursiv + 1 );
        		}
        		else
        		{
        			// Einhaltung Position absichern
        			tmpInterfaceTypeArgs.add( null );

        			interfaceTypeArgsRestStr =
        					interfaceTypeArgsRestStr.substring(
        							semicolonPos + 1 );
        		}
    		}

    		interfaceTypeArgs =
    				//Collections./*unmodifiableMap*/unmodifiableList( tmpInterfaceTypeArgs )
    				ImmutableArray.<ImmutableArray<? extends AsmType>>create( tmpInterfaceTypeArgs );
    	}
    	
    	return new ClassTypeparamsAndInheritTypeArgsTuple(
    			typeParams ,
    			superTypeArgs ,
    			interfaceTypeArgs );
    }

}
