package de.heinerkuecker.asm_improved;

import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;

import de.heinerkuecker.asm_improved.type.AsmArray;
import de.heinerkuecker.asm_improved.type.AsmTopLevelOrInnerClassType;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.arg.AsmExtendsWildcard;
import de.heinerkuecker.asm_improved.type.generics.arg.AsmSuperWildcard;
import de.heinerkuecker.asm_improved.type.generics.arg.AsmUnboundWildcard;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.member.AsmInnerClassType;
import de.heinerkuecker.asm_improved.type.primitive.AsmBoolean;
import de.heinerkuecker.asm_improved.type.primitive.AsmByte;
import de.heinerkuecker.asm_improved.type.primitive.AsmChar;
import de.heinerkuecker.asm_improved.type.primitive.AsmDouble;
import de.heinerkuecker.asm_improved.type.primitive.AsmFloat;
import de.heinerkuecker.asm_improved.type.primitive.AsmInt;
import de.heinerkuecker.asm_improved.type.primitive.AsmLong;
import de.heinerkuecker.asm_improved.type.primitive.AsmShort;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * Util-Klasse.
 * 
 * @author Heiner K&uuml;cker
 */
final class AsmDescAndSignatureParserUtil
{

	/**
	 * Konstruktor private,
	 * Instanziieren verboten,
	 * Util-Klasse.
	 */
	private AsmDescAndSignatureParserUtil()
	{
		super();
	}

	/**
	 * 
	 * @param typeStr
	 * @param innerClassFileNameToDotSeparatedNameMap
	 * @return
	 */
	static AsmType parseTypeStr(
			final String typeStr ,
			final Map<String, ? extends AsmExtendsTypeParam> extendsTypeParamMap ,
			final ClasspathFiles classpathFiles ,
			final TreeMap<String, ? extends String> innerClassFileNameToDotSeparatedNameMap ,
			final FilteredForNonStaticAndSortedForOuterNameLengthReverseAsmDomInnerClasses innerClasses )
	{
		//System.out.println( new Exception().getStackTrace()[ 0 ] + " " + typeStr );

		// die interne Codierung desc ist zu erkennen im Quellcode von org.objectweb.asm.Type.getDescriptor(StringBuffer, Class<?>)
		switch ( typeStr.substring( 0 , 1 ) )
		{
		case "I":
			return AsmInt.INSTANCE;
		case "Z":
			return AsmBoolean.INSTANCE;
		case "B":
			return AsmByte.INSTANCE;
		case "C":
			return AsmChar.INSTANCE;
		case "S":
			return AsmShort.INSTANCE;
		case "J":
			return AsmLong.INSTANCE;
		case "D":
			return AsmDouble.INSTANCE;
		case "F":
			return AsmFloat.INSTANCE;
		case "V":
			// void
			return null;
		case "*":
			// Hinzugefgt weil aufgetaucht in Klasse com/sun/beans/decoder/MethodElementHandler Methode getValueObject
			return AsmUnboundWildcard.INSTANCE;
		default:
			if ( typeStr.startsWith( "L" ) )
				// eine Klasse
			{
				int parsePos = 
						// fuehrendes L ueberspringen
						1;

				AsmTopLevelOrInnerClassType tmpType = null;

				while ( true )
				{
					final int preParsePos = parsePos;

					final ImmutableArray<? extends AsmType> typeArgs;
					final int classSlashFqnEndIndexInclusive;
					if ( ! typeStr.substring( parsePos ).contains( "<" ) )
						// keine Generics
					{
						classSlashFqnEndIndexInclusive =
								//endIndex: hinten steht ein Semikolon
								typeStr.length() - 2;

						parsePos = classSlashFqnEndIndexInclusive + 2;

						typeArgs = null;
					}
					else
					{
						classSlashFqnEndIndexInclusive =
								//endIndex: hinten steht ein '<'-Zeichen
								typeStr.substring( parsePos ).indexOf( '<' ) + parsePos - 1;

						final ArrayList<AsmType> tmpTypeArgs = new ArrayList<>();

						// nach dem beginennden <
						int currTypeArgPos = classSlashFqnEndIndexInclusive + 2;

						while ( true )
						{
							final String currTypeArgStrToParse;
							if ( typeStr.charAt( currTypeArgPos ) == '*' )
								// Sonderfall unbound-wildcard
							{
								currTypeArgStrToParse = "*";
							}
							else
							{
								final int currTypeArgEndIndexInclusive =
										AsmDescAndSignatureParserUtil.getTypeEndPos(
												typeStr ,
												currTypeArgPos ,
												//primitivePossible
												//true TODO type-argumente koenne keine primitive sein, evtl ab java-value-types, valhalla, es sind aber primitive-arrays erlaubt
												false );
								//final int currTypeArgEndIndex =

								currTypeArgStrToParse =
										typeStr.substring(
												//beginIndex
												currTypeArgPos ,
												//endIndex
												currTypeArgEndIndexInclusive + 1
												//currTypeArgEndIndex
												);
							}

							final AsmType currTypeArg =
									parseTypeStr(
											currTypeArgStrToParse ,
											extendsTypeParamMap ,
											classpathFiles ,
											innerClassFileNameToDotSeparatedNameMap ,
											innerClasses );

							tmpTypeArgs.add( currTypeArg );

							currTypeArgPos +=
									currTypeArgStrToParse.length();

							if ( typeStr.charAt( currTypeArgPos ) == '>' )
							{
								break;
							}
						}

						parsePos =
								currTypeArgPos + 
								// eine Position fr das endende >
								1;

						tmpTypeArgs.trimToSize();
						typeArgs = /*Collections.unmodifiableList*/ImmutableArray.create( tmpTypeArgs );
					}

					final String rawName =
							typeStr.substring(
									//beginIndex: vorn steht L
									preParsePos ,
									classSlashFqnEndIndexInclusive + 1 );

					if ( tmpType == null )
					{
						final Fqn fqn =
								//new Fqn(
								//      new PackageName( packageName ) ,
								//      DottedName.valueOfDotSeparatedStr( simpleName ) ) ,
								InnerOrNestedClassNameResolver.resolve(
										//dollarSeparatedRawName
										rawName ,
										innerClassFileNameToDotSeparatedNameMap );

						tmpType =
								new AsmClassType(
										fqn ,
										typeArgs );
					}
					else
					{
						// der Name kann auch Punkte enthalten wegen zwischenklassen, die keine type-argumente haben
						final String[] rawNameArr = rawName.split( "\\." );

						for ( int i = 0 ; i < rawNameArr.length ; i++ )
						{
							final String subRawName = rawNameArr[ i ];

							tmpType =
									new AsmInnerClassType(
											tmpType ,
											//DottedName.valueOfNoDotSeparatedStr(
											//new JavaIdentifier(
													subRawName ,
											// nur der letzte sub-Name wird mit den Type-argumenten verbunden
											i == rawNameArr.length -1
											? typeArgs
													: null );
						}
					}

					if ( parsePos >= typeStr.length() -
							// endendes Semikolon beachten
							1 )
					{
						if ( (tmpType instanceof AsmTopLevelOrInnerClassType ) &&
								( ! innerClasses.isEmpty() ) )
						{
							tmpType =
									AsmInnerClassTypeCorrector.correctMemberClassType(
											(AsmTopLevelOrInnerClassType) tmpType ,
											innerClasses );
						}

						return tmpType;
					}

					if ( typeStr.charAt( parsePos ) == '.' )
						// Punkt fr innere/geschachtelte Klasse
					{
						parsePos++;
					}
					else
					{
						throw new IllegalStateException( "unexpected character at " + parsePos + ": " + typeStr );
					}
				}
			}
			else if ( typeStr.startsWith( "[" ) )
				// ein Array
			{
				return new AsmArray(
						parseTypeStr(
								// fuehrende Klammer ueberspringen
								typeStr.substring( 1 ) ,
								extendsTypeParamMap ,
								classpathFiles ,
								innerClassFileNameToDotSeparatedNameMap ,
								innerClasses ) );
			}
			else if ( typeStr.startsWith( "+" ) )
				// ein extends-Wildcard
			{
				return new AsmExtendsWildcard(
						parseTypeStr(
								// fuehrendes Plus-Zeichen ueberspringen
								typeStr.substring( 1 ) ,
								extendsTypeParamMap ,
								classpathFiles ,
								innerClassFileNameToDotSeparatedNameMap ,
								innerClasses ) );
			}
			else if ( typeStr.startsWith( "*" ) )
				// ein unbound-Wildcard
			{
				return AsmUnboundWildcard.INSTANCE;
			}
			else if ( typeStr.startsWith( "-" ) )
				// super-wildcard
				// signature: (Ljava/util/function/Consumer<-TT;>;)V
				// java.util.Optional<T>: public void ifPresent(Consumer<? super T> consumer)
			{
				return new AsmSuperWildcard(
						parseTypeStr(
								// fuehrendes Minus-Zeichen ueberspringen
								typeStr.substring( 1 ) ,
								extendsTypeParamMap ,
								classpathFiles ,
								innerClassFileNameToDotSeparatedNameMap ,
								innerClasses ) );
			}
			else if ( typeStr.startsWith( "T" ) )
				// eine Typ-Variable
			{
				return AsmTypevarTypeParser.parseTypvar(
						typeStr ,
						extendsTypeParamMap ,
						classpathFiles ,
						innerClassFileNameToDotSeparatedNameMap ,
						innerClasses );
			}
			else
			{
				throw new IllegalArgumentException( typeStr );
			}
		}
	}

	/**
	 * Entscheiden, ob desc oder signature geparst werden soll
	 * und Exception-String entfernen.
	 * 
	 * @param desc
	 * @param signature
	 * @return zu parsender Type-String
	 */
	static String determineDescOrSignatureAndRemoveExceptions(
			final String desc ,
			final String signature )
	{
		final String strToParseWithExceptions;
		if ( signature == null )
		{
			// alte Welt ohne Generics
			strToParseWithExceptions = desc;
		}
		else
		{
			// beim Auftreten von Generics sind in signature alle Informationen aus desc auch enthalten
			strToParseWithExceptions = signature;
		}

		return removeExceptions(
				strToParseWithExceptions );
	}

	/**
	 * @param strToParseWithExceptions
	 * @return zu parsender String ohne Exceptions
	 */
	static String removeExceptions(
			final String strToParseWithExceptions )
	{
		final String strToParse;
		if ( strToParseWithExceptions.contains( "^" ) )
			// checked Exceptions in der Signatur enthalten
		{
			strToParse =
					strToParseWithExceptions.substring(
							//beginIndex
							0 ,
							//endIndex
							strToParseWithExceptions.indexOf( '^' ) );
		}
		else
		{
			strToParse = strToParseWithExceptions;
		}

		return strToParse;
	}

	/**
	 * Typ-Ende, hauptsaechlich
	 * Semikolon suchen unter Beachtung
	 * Schachtelung Generics.
	 * 
	 * @param str
	 * @param startPos
	 * @return
	 */
	static int getTypeEndPos(
			final String str ,
			final int startPos ,
			// bei Typ-Variablen koennte der Name der Typ-Variablen dem Code eines primitive entsprechen
			final boolean primitivePossible )
	{
		if ( primitivePossible &&
				( str.charAt( startPos ) == 'I' ||
				str.charAt( startPos ) == 'I' ||
				str.charAt( startPos ) == 'Z' ||
				str.charAt( startPos ) == 'B' ||
				str.charAt( startPos ) == 'C' ||
				str.charAt( startPos ) == 'S' ||
				str.charAt( startPos ) == 'J' ||
				str.charAt( startPos ) == 'D' ||
				str.charAt( startPos ) == 'F' ||
				str.charAt( startPos ) == 'V' ) )
			// ein Primitive hat Semikolon
			// erstes Zeichen des Typs ist auch letztes Zeichen des Typs
		{
			return startPos;
		}

		if ( // extends bound
				str.charAt( startPos ) == '+' ||
				// super bound
				str.charAt( startPos ) == '-' )
		{
			return getTypeEndPos(
					str ,
					startPos + 1 ,
					primitivePossible );
		}

		if ( str.charAt( startPos ) == '[' )
			// Array
		{
			// rekursiver Aufruf, damit die Logik mit den Primitiven erneut wirken kann
			// java.util.Calendar, Field: cachedLocaleData
			// Ljava/util/concurrent/ConcurrentMap<Ljava/util/Locale;[I>;
			return getTypeEndPos(
					str ,
					startPos + 1 ,
					//primitivePossible: Primitiv-Array erlaubt
					true );
		}

		int currPos = startPos;

		while ( str.charAt( currPos ) != ';' )
		{
			if ( str.charAt( currPos ) == '<' )
				// spitze Klammer Generics
			{
				currPos =
						getAfterGtPosRecursiv(
								str ,
								// damit die Methode getAfterGtPosRecursiv funktioniert, muss das beginnende kleiner-als-Zeichen verbraucht worden sein TODO evtl umbauen
								currPos + 1 );
			}
			else
			{
				currPos++;
			}
		}

		return currPos;
	}

	/**
	 * Groesser-Als-Zeichen suchen
	 * mit moeglicher Schachtelung
	 * Kleiner-Als-Zeichen zu
	 * Groesser-Als-Zeichen.
	 * 
	 * @param str
	 * @param startPos
	 * @return
	 */
	static int getAfterGtPosRecursiv(
			final String str ,
			final int startPos )
	{
		int currPos = startPos;

		while ( str.charAt( currPos ) != '>' )
		{
			if ( str.charAt( currPos ) == '<' )
				// spitze Klammer Generics
			{
				currPos =
						getAfterGtPosRecursiv(
								str ,
								currPos + 1 );
			}
			else
			{
				currPos++;
			}
		}

		// '>' nicht noch einmal pruefen
		return currPos + 1;
	}

	static String takeParamTypeStrFromParamsTypeStr(
			final String paramsTypeStr )
	{
		int startPosForArrayPrefix = 0;

		while ( paramsTypeStr.charAt( startPosForArrayPrefix ) == '[' )
			// bei mehrdimensionalen Arrays stehen vor dem Element-Type oeffnende eckige Klammern in der Anzahl der Array-Dimansionen
		{
			startPosForArrayPrefix++;
		}

		if ( paramsTypeStr.charAt( startPosForArrayPrefix ) == 'L' ||
				// der Parameter ist eine Klasse
				paramsTypeStr.charAt( startPosForArrayPrefix ) == 'T' )
			// der Parameter ist eine Typ-Variable
		{
			final int endIndex =
					getTypeEndPos(
							paramsTypeStr ,
							startPosForArrayPrefix + 1 ,
							//primitivePossible
							false ) + 1;

			return paramsTypeStr.substring(
					//beginIndex
					0 ,
					endIndex );
		}

		return paramsTypeStr.substring(
				//beginIndex
				0 ,
				//endIndex
				startPosForArrayPrefix + 1 );
	}

}
