package de.heinerkuecker.asm_improved;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import de.heinerkuecker.asm_improved.asm_dom.AsmDomInnerClass;
import de.heinerkuecker.asm_improved.type.AsmTopLevelOrInnerClassType;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.member.AsmInnerClassType;
import de.heinerkuecker.asm_improved.util.DottedName;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * Siehe {@link #correctMemberClassType}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmInnerClassTypeCorrector
{
	/**
	 * TODO
	 */
	private AsmInnerClassTypeCorrector()
	{
		super();
	}

	/**
	 * Beim Parsen kann es passieren,
	 * dass eine innere Klasse (non static)
	 * als geschachtelte(nested) Klasse (static)
	 * erkannt wird.
	 * Hier erfolgt die Korrektur, falls es sich
	 * um eine innere Klasse (non static)
	 * handelt.
	 * 
	 * @param typeToCorrect
	 * @param innerClasses
	 * @return
	 */
	static AsmTopLevelOrInnerClassType correctMemberClassType(
			final AsmTopLevelOrInnerClassType typeToCorrect ,
			final FilteredForNonStaticAndSortedForOuterNameLengthReverseAsmDomInnerClasses innerClasses )
	{
		if ( typeToCorrect instanceof AsmInnerClassType )
		{
			/*
			 * wenn schon eine innere Klasse vorliegt, die aeusere Klasse eventuell korrigieren.
			 * Beruht darauf, dass in einer nicht statischen inneren Klasse keine statische innere Klasse mehr deklariert werden darf.
			 */
			final AsmInnerClassType tmpMemberClassType = (AsmInnerClassType) typeToCorrect;

			final AsmTopLevelOrInnerClassType outerCorrectedType =
					correctMemberClassType(
							tmpMemberClassType.outerOrEnclosingType ,
							innerClasses );

			return new AsmInnerClassType(
					outerCorrectedType ,
					//relativName
					//DottedName.valueOfDotSeparatedStr(
					//new JavaIdentifier(
							tmpMemberClassType.getEntireFqn().dottedName.toJavaName().substring(
									outerCorrectedType.getEntireFqn().dottedName.toJavaName().length() + 1 ) ,
					// typ argumente werden an letzte klasse gehaengt
					tmpMemberClassType.typeArgs );
			//throw new RuntimeException( "nicht fertig" );
		}

		// TODO diese variable kann weg
		AsmTopLevelOrInnerClassType tmpType = typeToCorrect;

		final Fqn entireFqn = tmpType.getEntireFqn();

		for ( final AsmDomInnerClass innerClass : innerClasses )
		{
			//if ( ( innerClass.access & org.objectweb.asm.Opcodes.ACC_STATIC ) == 0 )
				// not static
			{
				//System.out.println( " " + new Exception().getStackTrace()[ 0 ] + " " + entireFqn.toClassFileFqnStr() + " " + innerClass.outerName + " " + innerClass.innerName );

				if ( ( innerClass.outerName == null &&
						// es handelt sich um eine anonyme Klasse
						entireFqn.toClassFileFqnStr().equals( innerClass.name ) ) ||
						( innerClass.outerName != null &&
						( entireFqn.toClassFileFqnStr().equals(
								innerClass.outerName
								// null darf hier nicht durchkommen
								.toString() +
								"$" +
								innerClass.innerName.toString() ) ||
								entireFqn.toClassFileFqnStr().startsWith(
										innerClass.outerName
										// null darf hier nicht durchkommen
										.toString() +
										"$" +
										innerClass.innerName
										// null darf hier nicht durchkommen
										.toString() +
										"$" ) ) ) )
				{
					//System.out.println( "*" + new Exception().getStackTrace()[ 0 ] + " " + entireFqn.toClassFileFqnStr() + " " + innerClass.outerName + " " + innerClass.innerName );

					// noch mal Pruefen, ob es an der Trenn-Stelle outerName + '$' + innerName eine Trennung im dottedName gibt, weil deas $-Zeichen kein eindeutiger Trenner ist
					final DottedName dottedName = entireFqn.dottedName;

					final String outerNameWithoutPackage;
					if ( innerClass.outerName == null )
						// es handelt sich um eine anonyme Klasse
					{
						final String nameWithoutPackage;
						final int lastIndexOfSlashInName = innerClass.name.lastIndexOf( '/' );
						if ( lastIndexOfSlashInName >= 0 )
						{
							nameWithoutPackage =
									innerClass.name.substring(
											lastIndexOfSlashInName + 1 );
						}
						else
						{
							nameWithoutPackage = innerClass.name;
						}
						outerNameWithoutPackage =
								nameWithoutPackage.substring(
										//beginIndex
										0 ,
										//endIndex
										nameWithoutPackage.lastIndexOf( "$" ) );
					}
					else
					{
						final int lastIndexOfSlashInOuterName = innerClass.outerName.lastIndexOf( '/' );
						if ( lastIndexOfSlashInOuterName >= 0 )
						{
							outerNameWithoutPackage =
									innerClass.outerName.substring(
											lastIndexOfSlashInOuterName + 1 );
						}
						else
						{
							outerNameWithoutPackage = innerClass.outerName;
						}
					}

					final String innerName;
					if ( innerClass.innerName == null )
						// es handelt sich um eine anonyme Klasse
					{
						innerName =
								innerClass.name.substring(
										//beginIndex
										innerClass.name.lastIndexOf( "$" ) + 1 );
					}
					else
					{
						innerName =
								innerClass.innerName
								// null darf hier nicht durchkommen
								.toString();
					}
					
					if ( dottedName.arePartsOfJavaClassFileName(
							outerNameWithoutPackage ,
							innerName ) )
					{
						final AsmClassType tmpClassType = (AsmClassType) tmpType;

						final AsmClassType outerType =
								new AsmClassType(
										new Fqn(
												tmpClassType.fqn.packageName ,
												tmpClassType.fqn.dottedName.instanceFromLeftPartsJavaClassName(
														outerNameWithoutPackage ) ) );

						final AsmTopLevelOrInnerClassType outerCorrectedType =
								// recursive call
								correctMemberClassType(
										outerType ,
										innerClasses );

						tmpType =
								new AsmInnerClassType(
										outerCorrectedType ,
										//relativName
										//DottedName.valueOfDotSeparatedStr(
										//new JavaIdentifier( 
										// TODO kann ich hier auch innerClass.innerName verwenden ?
										tmpClassType.fqn.dottedName.toJavaName().substring(
												outerType.fqn.dottedName.toJavaName().length() + 1 ) ,
										// typ argumente werden an letzte klasse gehaengt
										tmpClassType.typeArgs );

						return tmpType;
					}
				}
			}
		}

		return tmpType;
		//throw new RuntimeException( "nicht fertig" );
	}

	/**
	 * Sortieren nach Laenge der linken Namens-Teile, damit von links beginnend eventuell mehrfach innere Klassen geteilt werden
	 * 
	 * @param innerClasses
	 * @return gefilterte und sortierte {@link AsmDomInnerClass}-Objekte 
	 */
	public static FilteredForNonStaticAndSortedForOuterNameLengthReverseAsmDomInnerClasses filterForNonStaticAndSortInnerClassesForOuterNameLengthReverse(
			final List<AsmDomInnerClass> innerClasses )
	{
		final List<AsmDomInnerClass> result = new ArrayList<>();
		
		for ( final AsmDomInnerClass asmDomInnerClass : innerClasses )
		{
			if ( 
					// asmDomInnerClass.outerName und asmDomInnerClass.innerName sind bei anonymen Klassen null, siehe AsmInnerClassTypeCorrectorTest
					//asmDomInnerClass.outerName != null &&
					//asmDomInnerClass.innerName != null &&
					( asmDomInnerClass.access & org.objectweb.asm.Opcodes.ACC_STATIC ) == 0 )
				// not static
			{
				result.add( asmDomInnerClass );
			}
		}
		
		Collections.sort(
				result ,
				new Comparator<AsmDomInnerClass>() {
					@Override
					public int compare(
							final AsmDomInnerClass innerClass1 ,
							final AsmDomInnerClass innerClass2 )
					{
						// TODO outerName ist bei anonymen Klassen null
						//return -innerClass1.outerName.compareTo(
						//		innerClass2.outerName );
						return -innerClass1.name.compareTo(
								innerClass2.name );
					}
				} );
		
		return 
				new FilteredForNonStaticAndSortedForOuterNameLengthReverseAsmDomInnerClasses(
				result );
	}

}
