package de.heinerkuecker.asm_improved;

import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;

import de.heinerkuecker.asm_improved.type.AsmTopLevelOrInnerClassType;
import de.heinerkuecker.asm_improved.type.AsmTopLevelOrInnerClassTypeOrTypevar;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * Parser f&uuml;r Typ-Informationen von Methoden.
 * 
 * @author Heiner K&uuml;cker
 */
final class AsmMethodDescAndSignatureParser
{
	/**
	 * Konstruktor private,
	 * Instanziieren verboten,
	 * Util-Klasse.
	 */
	private AsmMethodDescAndSignatureParser()
	{
		super();
	}

	public static MethodTypeparamsAndReturnAndParamsAndExceptionsTuple parseMethodDescriptionAndSignature(
	        final String desc ,
	        final String signature ,
	        final String[] exceptionsStrArr ,
			final Map<String, AsmExtendsTypeParam> extendsTypeParamMap ,
			final ClasspathFiles classpathFiles ,
			final TreeMap<String, ? extends String> innerClassFileNameToDotSeparatedNameMap ,
			final FilteredForNonStaticAndSortedForOuterNameLengthReverseAsmDomInnerClasses innerClasses )
	{
	    final ImmutableArray<? extends AsmTopLevelOrInnerClassTypeOrTypevar> exceptions;
	    if ( exceptionsStrArr == null )
	    {
	        exceptions = null;
	    }
	    else
	    {
	        if ( signature == null )
	            // keine Generics
	        {
	            final ArrayList<AsmTopLevelOrInnerClassType> tmpExceptionList =
	                    parseNonGenericsExceptions(
	                            exceptionsStrArr ,
	                            innerClassFileNameToDotSeparatedNameMap );

	            exceptions = ImmutableArray.<AsmTopLevelOrInnerClassType>create( tmpExceptionList );
	        }
	        else
	        {
	            if ( signature.contains( "^" ) )
	            {
	                final ArrayList<AsmTopLevelOrInnerClassTypeOrTypevar> tmpExceptionList = new ArrayList<>();

	                // vor den Exceptions steht ein ^-Zeichen(Circumflex)
	                final String exceptionTypeStrToParse = signature.substring( signature.indexOf( '^' ) + 1 );

	                //System.out.println( exceptionTypeStrToParse );

	                String exceptionTypeRestStrToParse = exceptionTypeStrToParse;

	                while ( tmpExceptionList.size() < exceptionsStrArr.length )
	                {
	                    final String exceptionTypePartStrToParse;
	                    if ( tmpExceptionList.size() < exceptionsStrArr.length - 1 )
	                    {
	                        // mehrere Exceptions sind durch ^-Zeichen(Circumflex) abgetrennt
	                        exceptionTypePartStrToParse = exceptionTypeRestStrToParse.substring( 0 , exceptionTypeRestStrToParse.indexOf( '^' ) );

	                        exceptionTypeRestStrToParse = exceptionTypeRestStrToParse.substring( exceptionTypeRestStrToParse.indexOf( '^' ) + 1 );
	                    }
	                    else
	                    {
	                        exceptionTypePartStrToParse = exceptionTypeRestStrToParse;
	                    }

	                    //System.out.println( "exceptionTypePartStrToParse: " + exceptionTypePartStrToParse );

	                    final AsmTopLevelOrInnerClassTypeOrTypevar exceptionType =
	                    		(AsmTopLevelOrInnerClassTypeOrTypevar) AsmDescAndSignatureParserUtil.parseTypeStr(
	                    				exceptionTypePartStrToParse ,
	                    				extendsTypeParamMap ,
	                    				classpathFiles ,
	                    				innerClassFileNameToDotSeparatedNameMap ,
	                    				innerClasses );

	                    tmpExceptionList.add( exceptionType );
	                }

	                exceptions = ImmutableArray.create( tmpExceptionList );
	            }
	            else
	                // keine Generics bei den Exceptions
	            {
	                final ArrayList<AsmTopLevelOrInnerClassType> tmpExceptionList =
	                        parseNonGenericsExceptions(
	                                exceptionsStrArr ,
	                                innerClassFileNameToDotSeparatedNameMap );

	                exceptions = ImmutableArray.<AsmTopLevelOrInnerClassType>create( tmpExceptionList );
	            }
	        }
	    }

	    final String strToParse =
	            AsmDescAndSignatureParserUtil.determineDescOrSignatureAndRemoveExceptions(
	                    desc ,
	                    signature );

	    final ImmutableArray<? extends AsmAbstractTypeParam> typeParams;
	    final ImmutableArray<? extends AsmType> paramTypes;
	    final int returnTypeStartIndex;
	    if ( strToParse.startsWith( "()" ) )
	        // keine Parameter
	    {
	        typeParams = null;
	        paramTypes = null;
	        returnTypeStartIndex = "()".length();
	    }
	    else if ( strToParse.startsWith( "(" ) )
	    {
	        typeParams = null;

	        final ArrayList<AsmType> tmpParamTypes = new ArrayList<>();

	        String paramsTypeStrToParse =
	                strToParse.substring(
	                        //beginIndex: fhrende Klammer ueberspringen
	                        1 ,
	                        //endIndex
	                        strToParse.lastIndexOf( ')' ) );

	        while ( ! paramsTypeStrToParse.isEmpty() )
	        {
	            final String paramTypeStrToParse =
	                    AsmDescAndSignatureParserUtil.takeParamTypeStrFromParamsTypeStr(
	                            paramsTypeStrToParse );

	            final AsmType paramType =
	                    AsmDescAndSignatureParserUtil.parseTypeStr(
	                            paramTypeStrToParse ,
	                            extendsTypeParamMap ,
	                            classpathFiles ,
	                            innerClassFileNameToDotSeparatedNameMap ,
	                            innerClasses );

	            tmpParamTypes.add( paramType );

	            paramsTypeStrToParse =
	                    paramsTypeStrToParse.substring(
	                            //beginIndex
	                            paramTypeStrToParse.length() );
	        }

	        paramTypes = ImmutableArray.create( tmpParamTypes );

	        returnTypeStartIndex = strToParse.lastIndexOf( ')' ) + 1;
	    }
	    else if ( strToParse.startsWith( "<" ) )
	        // Typ-Parameter
	    {
	        final int typeParamsEndIndex =
	                AsmDescAndSignatureParserUtil.getAfterGtPosRecursiv(
	                        strToParse ,
	                        // damit die Methode getAfterGtPosRecursiv funktioniert, muss das beginnende kleiner-als-Zeichen verbraucht worden sein TODO evtl umbauen
	                        0 + 1 );

	        final String typeParamsStr =
	                strToParse.substring(
	                        //beginIndex
	                        0 ,
	                        typeParamsEndIndex );

	        typeParams =
	                AsmTypeparamTypeParser.parseTypeParams(
	                        typeParamsStr ,
	                        extendsTypeParamMap ,
	                        classpathFiles ,
	                        innerClassFileNameToDotSeparatedNameMap ,
	                        innerClasses );

	        final String strToParseWithoutTypeParams =
	                strToParse.substring(
	                        typeParamsEndIndex );

	        final MethodTypeparamsAndReturnAndParamsAndExceptionsTuple tplWithoutTypeparams =
	                parseMethodDescriptionAndSignature(
	                        //desc
	                        null ,
	                        //signature
	                        strToParseWithoutTypeParams +
	                        // Exception-Teil wieder anfgen
	                        ( exceptionsStrArr != null && exceptionsStrArr.length > 0 && signature != null && signature.contains( "^" )
	                            ? signature.substring( signature.indexOf( '^' ) )
	                            : "" ) ,
	                        exceptionsStrArr ,
	                        extendsTypeParamMap ,
	                        classpathFiles ,
	                        innerClassFileNameToDotSeparatedNameMap ,
	                        innerClasses );

	        return new MethodTypeparamsAndReturnAndParamsAndExceptionsTuple(
	                typeParams ,
	                tplWithoutTypeparams.returnType ,
	                tplWithoutTypeparams.paramTypes ,
	                tplWithoutTypeparams.exceptions );
	    }
	    else
	    {
	        throw new IllegalArgumentException( strToParse );
	    }

	    final String returnTypeStrToParse =
	            strToParse.substring(
	                    returnTypeStartIndex );

	    final AsmType returnType =
	            AsmDescAndSignatureParserUtil.parseTypeStr(
	                    returnTypeStrToParse ,
	                    extendsTypeParamMap ,
	                    classpathFiles ,
	                    innerClassFileNameToDotSeparatedNameMap ,
	                    innerClasses );

	    return new MethodTypeparamsAndReturnAndParamsAndExceptionsTuple(
	            typeParams ,
	            returnType ,
	            paramTypes ,
	            exceptions );
	}

    private static ArrayList<AsmTopLevelOrInnerClassType> parseNonGenericsExceptions(
            final String[] exceptionsStrArr ,
            final TreeMap<String, ? extends String> innerClassFileNameToDotSeparatedNameMap )
    {
        final ArrayList<AsmTopLevelOrInnerClassType> tmpExceptionList = new ArrayList<>();

        for ( final String exceptionsStr : exceptionsStrArr )
        {
            final Fqn fqn =
                    InnerOrNestedClassNameResolver.resolve(
                            //dollarSeparatedRawName
                            exceptionsStr ,
                            innerClassFileNameToDotSeparatedNameMap );

            final AsmClassType exceptionType =
            		// TODO hier knnte auch eine innere/geschachtelte Klasse auftauchen
                    new AsmClassType(
                            fqn , 
                            //typeArgs sind bei Exceptions nicht erlaubt
                            (ImmutableArray<? extends AsmType>) null );

            tmpExceptionList.add( exceptionType );
        }

        return tmpExceptionList;
    }

}
