package de.heinerkuecker.asm_improved;

import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;

/**
 * Informationen zu einem Methoden-Parameter.
 */
public final class AsmMethodParamInfo
{
	public final AsmType type;
	
	/**
	 * Annotationen.
	 */
	public final ImmutableArray<? extends AsmAnnotationUse> annotations;

	/**
	 * @param type
	 * @param annotations
	 */
	AsmMethodParamInfo(
			final AsmType type ,
			final ImmutableArray<? extends AsmAnnotationUse> annotations )
	{
		if ( type == null )
		{
			throw new IllegalArgumentException( "type is null" );
		}
		
		this.type = type;
		this.annotations = annotations;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString()
	{
		return
				//"AsmMethodParamInfo [type=" + 
				//", annotations=" +
				( this.annotations != null ? this.annotations + " " : "" ) +
				this.type
				//+ "]"
				;
	}

    public AsmMethodParamInfo resolveBounds(
    		// Typ-Argumente der konkreten Instanz dieser Klasse
    		final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap )
    {
    	return new AsmMethodParamInfo(
    			type.resolveBound( typeargsNameToBoundMap ) ,
    			annotations );
    }

}
