package de.heinerkuecker.asm_improved;

import java.util.HashMap;

import de.heinerkuecker.asm_improved.util.GetTypesafeMap;
import de.heinerkuecker.asm_improved.util.PackageName;

/**
 * Zusammenfassung der {@link AsmClassInfo}-Objekte
 * eines Java-Package.
 */
public final class AsmPackageInfo
{
	/**
	 * {@link AsmClassInfo}-Objekte.
	 */
	final GetTypesafeMap<String, AsmClassInfo> classMap = GetTypesafeMap.create( new HashMap<String, AsmClassInfo>() );

	/**
	 * Vollqualifizierter Name des Package
	 * oder Leerstring beim default-Package.
	 */
	public final PackageName name;

	/**
	 * Konstruktor.
	 *
	 * @param name vollqualifizierter Name
	 */
	AsmPackageInfo(
			final PackageName name )
	{
		if ( name == null 
				// empty (default) package erlaubt
				//|| name.trim().isEmpty()
				)
		{
			throw new IllegalArgumentException(
					"name is null" );
		}
		this.name = name;
	}

	/**
	 * @return ob dies das default (leere) Package ist
	 */
	public boolean isDefaultPackage()
	{
		return this.name.value.isEmpty();
	}
	
	public Iterable<? extends AsmClassInfo> getAllClass()
	{
		// TODO read only, immutable
		return this.classMap.values();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString()
	{
		return this.getClass().getSimpleName() + "[" +
		/*"qualifiedName=" +*/ name +
		"]";
	}

	/**
	 * {@inheritDoc}
	 */
    @Override
    public int hashCode()
    {
        return name.hashCode();
    }

	/**
	 * {@inheritDoc}
	 */
    @Override
    public boolean equals(
    		final Object obj )
    {
        if (this == obj)
	        return true;
        if (obj == null)
	        return false;
        if (getClass() != obj.getClass())
	        return false;
        final AsmPackageInfo other = (AsmPackageInfo) obj;
        /*if (qualifiedName == null)
        {
	        if (other.qualifiedName != null)
		        return false;
        } else*/ if (!name.equals( other.name ) )
	        return false;
        return true;
    }
}