package de.heinerkuecker.asm_improved;

import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;

import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * TODO
 * 
 * @author Heiner K&uuml;cker
 */
final class AsmTypeargsTypeParser
{

	/**
	 * Konstruktor private,
	 * Instanziieren verboten,
	 * Util-Klasse.
	 */
	private AsmTypeargsTypeParser()
	{
		super();
	}

	static ImmutableArray<? extends AsmType> parseTypeArgs(
			final String typeArgsStr ,
			final Map<String, ? extends AsmExtendsTypeParam> extendsTypeParamMap ,
			final ClasspathFiles classpathFiles ,
			final TreeMap<String, ? extends String> innerClassFileNameToDotSeparatedNameMap ,
			final FilteredForNonStaticAndSortedForOuterNameLengthReverseAsmDomInnerClasses innerClasses )
	{
		if ( ! typeArgsStr.startsWith( "<" ) )
		{
			throw new IllegalArgumentException( typeArgsStr );
		}

		String restTypeArgsStr =
				typeArgsStr.substring(
						//beginIndex: spitze Klammer (kleiner-als) am Anfang ueberspringen
						1 ,
						//endIndex: endende spitze Klammer (groesser-als) ueberspringen
						typeArgsStr.length() - 1 );

		final ArrayList<AsmType> result = new ArrayList<>();

		while ( ! restTypeArgsStr.isEmpty() )
		{
			final AsmType currTypArg;
			final int currTypArgEndIndex;
			final int primitiveArrayEndIndex;
			if ( ( primitiveArrayEndIndex =
					AsmTypeargsTypeParser.getPrimitiveArrayEndIndex(
							restTypeArgsStr ) ) > 0 )
				// als Type-Argument kann ein Primitiv-Array auftauchen, siehe java.util.stream.Node.OfDouble
			{
				currTypArgEndIndex = primitiveArrayEndIndex;
			}
			else
			{
				currTypArgEndIndex =
						AsmDescAndSignatureParserUtil.getTypeEndPos(
								restTypeArgsStr ,
								//startPos
								0 ,
								//primitivePossible 
								false ) + 1;
			}

			final String currTypArgStrToParse =
					restTypeArgsStr.substring(
							//beginIndex
							0 ,
							//endIndex
							currTypArgEndIndex );

			currTypArg =
					AsmDescAndSignatureParserUtil.parseTypeStr(
							currTypArgStrToParse ,
							extendsTypeParamMap ,
							classpathFiles ,
							innerClassFileNameToDotSeparatedNameMap ,
							innerClasses );

			result.add( currTypArg );

			restTypeArgsStr =
					restTypeArgsStr.substring(
							//beginIndex
							currTypArgEndIndex );
		}

		return ImmutableArray.create( result );
	}

	/**
	 * Feststellen, ob ein Primitiv-Array codiert wurde und End-Index zurckgeben
	 * @param typeStrWithPossiblePrimitiveArrTypeAtStart Type-String mit eventuellem codiertem Primitiv-Array am Anfang
	 * @return der End-Index (Basis 0, rechts aussen liegen) des Primitiv-Arrays oder -1, wenn Primitiv-Array codiert wurde
	 */
	private static int getPrimitiveArrayEndIndex(
			final String typeStrWithPossiblePrimitiveArrTypeAtStart )
	{
		//if ( ! typeStrWithPossiblePrimitiveArrTypeAtStart.startsWith( "[" ) )
		//{
		//	return -1;
		//}

		int currPos = 0;

		while ( true )
		{
			final char currChar =
					typeStrWithPossiblePrimitiveArrTypeAtStart.charAt(
							currPos );

			if ( currChar == '[' )
				// nchste Array-Dimension
			{
				currPos++;
			}
			else if (
					currChar == 'I' ||
					currChar == 'Z' ||
					currChar == 'B' ||
					currChar == 'C' ||
					currChar == 'S' ||
					currChar == 'J' ||
					currChar == 'D' ||
					currChar == 'F' )
				// Kenn-Buchstabe fr Primitive
			{
				return currPos + 1;
			}
			else
			{
				return -1;
			}
		}
	}

}
