package de.heinerkuecker.asm_improved;

import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;

import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmIntersectionBound;
import de.heinerkuecker.asm_improved.type.generics.param.AsmUnboundedTypeParam;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * TODO
 * 
 * @author Heiner K&uuml;cker
 */
final class AsmTypeparamTypeParser
{
	/**
	 * Konstruktor private,
	 * Instanziieren verboten,
	 * Util-Klasse.
	 */
	private AsmTypeparamTypeParser()
	{
		super();
	}

	static ImmutableArray<? extends AsmAbstractTypeParam> parseTypeParams(
			final String typeParamsStr ,
			final Map<String, AsmExtendsTypeParam> extendsTypeParamMap ,
			final ClasspathFiles classpathFiles ,
			final TreeMap<String, ? extends String> innerClassFileNameToDotSeparatedNameMap ,
			final FilteredForNonStaticAndSortedForOuterNameLengthReverseAsmDomInnerClasses innerClasses )
	{
		if ( ! typeParamsStr.startsWith( "<" ) )
		{
			throw new IllegalArgumentException( typeParamsStr );
		}

		//System.out.println( typeParamsStr );

		final String restTypeParamsStr =
				typeParamsStr.substring(
						//beginIndex: beginnende spitze Klammer (kleiner-als) am Anfang ueberspringen
						1 ,
						//endIndex: endende spitze Klammer (groesser-als) ueberspringen
						typeParamsStr.length() - 1 );

		final ArrayList<AsmAbstractTypeParam> result = new ArrayList<>();

		int currPos = 0;

		while ( currPos < restTypeParamsStr.length() )
		{
			final int colonPos =
					restTypeParamsStr.indexOf(
							':' ,
							currPos );

			final String typeParamName =
					restTypeParamsStr.substring(
							//beginIndex
							currPos ,
							//endIndex
							colonPos );

			currPos = colonPos;

			final ArrayList<AsmType> tmpBoundTypes = new ArrayList<>();

			while ( restTypeParamsStr.startsWith( ":" , currPos ) )
			{
				if ( restTypeParamsStr.startsWith( "::" , currPos ) )
					// manchmal stehen nach dem Typ-Parameter-Namen zwei Doppelpunkte '::'
				{
					currPos++;

					continue;
				}
				else if ( restTypeParamsStr.startsWith( ":Ljava/lang/Object;" , currPos ) )
					// entweder unbounded Typ-Parameter oder anderweitig ignorierbar
				{
					currPos +=
							":Ljava/lang/Object;".length();
				}
				else
				{
					final int currTypParamBoundEndIndex =
							//AsmDescAndSignatureParserUtil.getSemicolonPosUnderConsiderationGenerics(
							//		restTypeParamsStr ,
							//		//startPos
							//		colonPos + 1 ,
							//		//primitivePossible 
							//		false );
							getCurrTypParamBoundEndIndex(
									restTypeParamsStr.substring(
											//beginIndex
											currPos ) );

					final String currTypParamBoundStrToParse =
							restTypeParamsStr.substring(
									//beginIndex: fhrenden Doppelpunkt berspringen
									currPos + 1 ,
									//endIndex
									currPos + currTypParamBoundEndIndex );

					currPos += currTypParamBoundEndIndex;

					final AsmType currTypParamBoundType =
							AsmDescAndSignatureParserUtil.parseTypeStr(
									currTypParamBoundStrToParse ,
									extendsTypeParamMap ,
									classpathFiles ,
									innerClassFileNameToDotSeparatedNameMap ,
									innerClasses );

					tmpBoundTypes.add( currTypParamBoundType );
				}
			}

			final AsmAbstractTypeParam currTypParam;
			if ( tmpBoundTypes.isEmpty() )
			{
				currTypParam =
						new AsmUnboundedTypeParam(
								typeParamName );
			}
			else if ( tmpBoundTypes.size() == 1 )
			{
				currTypParam =
						new AsmExtendsTypeParam(
								typeParamName ,
								tmpBoundTypes.get( 0 ) );
			}
			else if ( tmpBoundTypes.size() > 1 )
			{
				final AsmIntersectionBound boundType =
						new AsmIntersectionBound(
								ImmutableArray.create( tmpBoundTypes ) );

				currTypParam =
						new AsmExtendsTypeParam(
								typeParamName ,
								boundType );
			}
			else
			{
				throw new IllegalStateException();
			}

			result.add( currTypParam );
			
			if ( currTypParam instanceof AsmExtendsTypeParam )
				// Testfall InnerClassOfTypevarWithTypeparamFieldUsing0AsTypevarBound, Type-Parameter werden als Type-Variable fr relative Angabe von inneren/geschachtelten Klassen benutzt
			{
				extendsTypeParamMap.put(
						currTypParam.name ,
						(AsmExtendsTypeParam) currTypParam );
			}
		}

		return ImmutableArray.create( result );
	}

	// siehe ClassWithTypeparams
	// T0 extends Comparable<T0> & Serializable, T1 extends Comparable<T1> & Serializable
	// T0::Ljava/lang/Comparable<TT0;>;:Ljava/io/Serializable;T1::Ljava/lang/Comparable<TT1;>;:Ljava/io/Serializable;
	// der aktuelle Typ-Parameter endet am Ende oder
	// wenn ein weiterer Typ-Parameter-Name gefolgt von zwei Doppelpunkten auftaucht
	private static final int getCurrTypParamBoundEndIndex(
			final String restTypeParamsStr )
	{
		int afterLeadingColonsIndex = 0;
		
		while ( restTypeParamsStr.startsWith(
				":" ,
				afterLeadingColonsIndex ) )
		{
			afterLeadingColonsIndex++;
		}

		final int secondColonIndex =
				restTypeParamsStr.indexOf(
						":" ,
						//fromIndex
						afterLeadingColonsIndex );

		if ( secondColonIndex < 0 )
			// ":" nicht vorhanden
		{
			return restTypeParamsStr.length();
		}

		for ( int currPos = secondColonIndex - 1 ; ; currPos-- )
			// Semikolon(Strichpunkt) links von der ":"-Position suchen
		{
			if ( restTypeParamsStr.charAt( currPos ) == ';' )
			{
				return currPos + 1;
			}
		}
	}
	
}
