package de.heinerkuecker.asm_improved;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * Dateien des zu scannenden Classpath,
 * Verzeichnisse, Class-Files oder Jar-Files.
 *
 * @author Heiner K&uuml;cker
 */
final class ClasspathFiles
{
	public final ImmutableArray<? extends File> fileArr;

	/**
	 * Konstruktor.
	 * 
	 * @param fileArr
	 */
	ClasspathFiles(
			final ImmutableArray<? extends File> fileArr )
	{
		this.fileArr = fileArr;
	}

	/**
	 * Konstruktor.
	 * 
	 * @param fileArr
	 */
	ClasspathFiles(
			final File... fileArr )
	{
		for ( final File file : fileArr )
		{
			if ( ! file.exists() )
			{
				throw new RuntimeException( "file not found " + file.getPath() );
			}
		}
		
		this.fileArr = ImmutableArray.<File>create( fileArr );
	}

	boolean containsClass(
			final Fqn fqn )
	{
		for ( final File classpathFile : this.fileArr )
		{
			if ( classpathFile.isDirectory() )
			{
				final File classFile =
						new File(
								classpathFile.getPath() +
								File.separator +
								fqn.toClassFileFqnStr() +
								".class" );

				if ( classFile.exists() )
				{
					return true;
				}
			}
			else if ( classpathFile.isFile() )
			{
				try ( JarFile jarFile = new JarFile( classpathFile ) )
				{
					final String jarEntryName =
							fqn.toClassFileFqnStr() +
							".class";

					final JarEntry jarEntry =
							jarFile.getJarEntry(
									jarEntryName );

					if ( jarEntry != null )
					{
						try ( InputStream jarInputStream = jarFile.getInputStream( jarEntry ) )
						{
							return true;
						}
					}
				}
				catch ( IOException iox )
				{
					//System.err.println( iox.getMessage() );
					//iox.printStackTrace();
					throw new RuntimeException( iox );
				}
			}
			else
			{
				throw new IllegalStateException(
						String.valueOf( classpathFile ) );
			}
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "ClasspathFiles [fileArr=" + this.fileArr + "]";
	}

}
