package de.heinerkuecker.asm_improved.annotation_use;

import de.heinerkuecker.asm_improved.type.AsmClassType;

/**
 * Benutzung eines Enum-Wertes
 * als Parameter oder Default-Wert
 * einer Annotation
 * im Java-Byte-Code.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmEnumValueUse
{
	/**
	 * Klasse des Enum-Wertes.
	 */
	public final AsmClassType enumType;

	/**
	 * Name des Enum-Wertes als String.
	 */
	public final String enumValueName;

	/**
	 * Konstruktor.
	 * 
	 * @param enumType Klasse des Enum
     * @param enumValueName String-Darstellung des Enum-Wertes
	 */
    public AsmEnumValueUse(
    		final AsmClassType enumType ,
    		final String enumValueName )
    {
	    this.enumType = enumType;
	    this.enumValueName = enumValueName;
    }

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
    @Override
    public int hashCode()
    {
	    final int prime = 31;
	    int result = 1;
	    result = prime * result
	            + ((enumType == null) ? 0 : enumType.hashCode());
	    result = prime * result
	            + ((enumValueName == null) ? 0 : enumValueName.hashCode());
	    return result;
    }

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
    @Override
    public boolean equals(Object obj)
    {
	    if (this == obj)
		    return true;
	    if (obj == null)
		    return false;
	    if (getClass() != obj.getClass())
		    return false;
	    AsmEnumValueUse other = (AsmEnumValueUse) obj;
	    if (enumType == null)
	    {
		    if (other.enumType != null)
			    return false;
	    } else if (!enumType.equals(other.enumType))
		    return false;
	    if (enumValueName == null)
	    {
		    if (other.enumValueName != null)
			    return false;
	    } else if (!enumValueName.equals(other.enumValueName))
		    return false;
	    return true;
    }

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
    @Override
    public String toString()
    {
	    return "EnumValueUse [enumType=" + enumType + ", enumValueName="
	            + enumValueName + "]";
    }

}
