package de.heinerkuecker.asm_improved.asm_dom;

import org.objectweb.asm.AnnotationVisitor;

/**
 * Sammeln der Informationen einer Annotation,
 * welche einer Annotation als Wert/Parameter
 * &uuml;bergeben wird.<br/>
 * {@link AnnotationVisitor}-Implementierung zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmDomAnnotationAnnotationValueVisitor
extends AnnotationVisitor
{
	/**
	 * Objekt zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
	 */
	public final AsmDomAnnotationAnnotationValue domAnnotationAnnotationValue;

	/**
	 * Konstruktor.
	 * 
	 * @param preAnnotationVisitor
	 */
	AsmDomAnnotationAnnotationValueVisitor(
			final AnnotationVisitor preAnnotationVisitor ,
			final String name ,
			final String desc )
	{
		super(
				org.objectweb.asm.Opcodes.ASM5 ,
				preAnnotationVisitor );

		this.domAnnotationAnnotationValue = new AsmDomAnnotationAnnotationValue( name , desc );

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + desc );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines primitiven
	 * Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visit(java.lang.String, java.lang.Object)
	 */
	@Override
	public void visit(
			final String name ,
			final Object value )
	{
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + value );

		super.visit( name , value );
		
		this.domAnnotationAnnotationValue.values.add( new AsmDomAnnotationPrimitiveValue( name , value ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Enum-Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitEnum(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void visitEnum(
			final String name ,
			final String desc ,
			final String value )
	{
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " +  name + " " + desc + " " +  value );

		super.visitEnum(name, desc, value);

		this.domAnnotationAnnotationValue.values.add( new AsmDomAnnotationEnumValue( name , desc , value ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitAnnotation(java.lang.String, java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitAnnotation(
			final String name ,
			final String desc )
	{
		// TODO Auto-generated method stub
		//return super.visitAnnotation(name, desc);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " +  name + " " + desc );

		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotation( name , desc );
		
		final AsmDomAnnotationAnnotationValueVisitor nextAnnotationVisitor = new AsmDomAnnotationAnnotationValueVisitor( preAnnotationVisitor , name , desc );

		this.domAnnotationAnnotationValue.values.add( nextAnnotationVisitor.domAnnotationAnnotationValue );
		
		return nextAnnotationVisitor;
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitArray(java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitArray(
			final String name )
	{
		// TODO Auto-generated method stub
		//return super.visitArray( name );

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name );

		final AnnotationVisitor preAnnotationVisitor = super.visitArray( name );
		
		final AsmDomArrayValuesAnnotationVisitor nextAnnotationVisitor = new AsmDomArrayValuesAnnotationVisitor( preAnnotationVisitor , name , null );

		this.domAnnotationAnnotationValue.values.add( nextAnnotationVisitor.domArray );
		
		return nextAnnotationVisitor;
	}

}
