package de.heinerkuecker.asm_improved.asm_dom;

import org.objectweb.asm.AnnotationVisitor;

/**
 * {@link AnnotationVisitor}-Implementierung zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmDomAnnotationAnnotationVisitor
extends AnnotationVisitor
{
	/**
	 * Objekt zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
	 */
	public final AsmDomAnnotationAnnotationValue domAnnotation;

	/**
	 * Konstruktor.
	 * 
	 * @param preAnnotationVisitor
	 */
	public AsmDomAnnotationAnnotationVisitor(
			final AnnotationVisitor preAnnotationVisitor ,
			final String name ,
			final String desc )
	{
		super(
				org.objectweb.asm.Opcodes.ASM5 ,
				preAnnotationVisitor );

		this.domAnnotation = new AsmDomAnnotationAnnotationValue( name , desc );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines primitiven
	 * Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visit(java.lang.String, java.lang.Object)
	 */
	@Override
	public void visit(
			final String name ,
			final Object value )
	{
		super.visit( name , value );
		
		this.domAnnotation.values.add( new AsmDomAnnotationPrimitiveValue( name , value ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Enum-Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitEnum(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void visitEnum(
			final String name ,
			final String desc ,
			final String value )
	{
		super.visitEnum(name, desc, value);

		this.domAnnotation.values.add( new AsmDomAnnotationEnumValue( name , desc , value ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitAnnotation(java.lang.String, java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitAnnotation(
			final String name ,
			final String desc )
	{
		// TODO Auto-generated method stub
		//return super.visitAnnotation(name, desc);
		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotation( name , desc );
		
		final AsmDomAnnotationAnnotationVisitor nextAnnotationVisitor = new AsmDomAnnotationAnnotationVisitor( preAnnotationVisitor , name , desc );

		this.domAnnotation.values.add( nextAnnotationVisitor.domAnnotation );
		
		return nextAnnotationVisitor;
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitArray(java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitArray(
			final String name )
	{
		// TODO Auto-generated method stub
		//return super.visitArray( name );
		
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name );

		final AnnotationVisitor preAnnotationVisitor = super.visitArray( name );
		
		final AsmDomArrayValuesAnnotationVisitor nextAnnotationVisitor = new AsmDomArrayValuesAnnotationVisitor( preAnnotationVisitor , name , null );

		this.domAnnotation.values.add( nextAnnotationVisitor.domArray );
		
		return nextAnnotationVisitor;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.AnnotationVisitor#visitEnd()
	 */
	//@Override
	//public void visitEnd() {
	//	// TODO Auto-generated method stub
	//	super.visitEnd();
	//}

}
