package de.heinerkuecker.asm_improved.asm_dom;

import org.objectweb.asm.AnnotationVisitor;

/**
 * {@link AnnotationVisitor}-Implementierung zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmDomAnnotationDefaultAnnotationVisitor
extends AnnotationVisitor
{
	/**
	 * Informationen die beim Durchlaufen der Methode {@link de.heinerkuecker.asm_improved.asm_dom.AsmDomMethodVisitor#visitAnnotationDefault()} aufgetreten sind.
	 */
	public final AsmDomAnnotationDefault domAnnotationDefault = new AsmDomAnnotationDefault();

	/**
	 * Konstruktor.
	 * 
	 * @param preAnnotationVisitor
	 */
	public AsmDomAnnotationDefaultAnnotationVisitor(
			final AnnotationVisitor preAnnotationVisitor )
	{
		super(
				org.objectweb.asm.Opcodes.ASM5 ,
				preAnnotationVisitor );
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.AnnotationVisitor#visit(java.lang.String, java.lang.Object)
	 */
	@Override
	public void visit(
			final String name ,
			final Object value )
	{
		// TODO Auto-generated method stub
		super.visit(name, value);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + value );

		this.domAnnotationDefault.domPrimitiveValue = new AsmDomAnnotationPrimitiveValue( name , value );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Enum-Annotations-Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitEnum(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void visitEnum(
			final String name ,
			final String desc ,
			final String value )
	{
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + desc + " " + value );

		this.domAnnotationDefault.domEnumValue = new AsmDomAnnotationEnumValue( name , desc , value );

		// TODO Auto-generated method stub
		super.visitEnum( name , desc , value );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitAnnotation(java.lang.String, java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitAnnotation(
			final String name ,
			final String desc )
	{
		// TODO Auto-generated method stub
		//return super.visitAnnotation(name, desc);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + desc );

		if ( desc == null )
		{
			throw new IllegalArgumentException( "annotation has no class" );
		}
		//if ( desc != null )
		//{
		//	throw new IllegalArgumentException( "annotation has class" );
		//}

		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotation( name , desc );

		final AsmDomAnnotationAnnotationValueVisitor nextAnnotationVisitor = new AsmDomAnnotationAnnotationValueVisitor( preAnnotationVisitor , name , desc );

		this.domAnnotationDefault.domAnnotationAnnotationValue = nextAnnotationVisitor.domAnnotationAnnotationValue;

		return nextAnnotationVisitor;
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitArray(java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitArray(
			final String name )
	{
		// TODO Auto-generated method stub
		//return super.visitArray(name);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name );

		final AnnotationVisitor preAnnotationVisitor = super.visitArray( name );
		
		final AsmDomArrayValuesAnnotationVisitor nextAnnotationVisitor = new AsmDomArrayValuesAnnotationVisitor( preAnnotationVisitor , name , null );

		this.domAnnotationDefault.domArray = nextAnnotationVisitor.domArray;
		
		return nextAnnotationVisitor;
	}

}
