package de.heinerkuecker.asm_improved.asm_dom;

/**
 * Abstrakte Oberklasse f&uuml;r Annotationswerte (Parameter).
 * 
 * @author Heiner K&uuml;cker
 */
public abstract class AsmDomAnnotationValue
{
	/**
	 * Name des Annotationswertes/-parameters.
	 */
	public final String name;

	/**
	 * Konstruktor.
	 * 
	 * @param name
	 */
	protected AsmDomAnnotationValue(
			final String name )
	{
		// TODO avoid null, aber nicht, wenn hier auch Array-Werte erlaubt sind
		this.name = name;
	}

	/**
	 * &Uuml;berschreiben der geerbten {@link #toString()}-Methode
	 * zum erzwungenen Aufruf der {@link #toString(String)}-Methode.
	 * 
	 * @return Debug-Format-Ausgabe
	 * @see java.lang.Object#toString()
	 */
	@Override
	final public String toString()
	{
		// empty start indent
		return toString( "" );
	}

	/**
	 * Methode zum Erzeugen einer
	 * Baum-artigen Debug-Anzeige.
	 * 
	 * @param indent Einrückung für Baum-artige-Anzeige
	 * @return Debug-Format-Ausgabe
	 */
	abstract public String toString(
			final String indent );

	/**
	 * Hilfsmethode zum Ausgeben {@link AsmDomAnnotationValue}s.
	 * 
	 * @param indent Einr&uuml;ckung f&uuml;r Baum-artige-Anzeige
	 * @param values auszugebende Objekte
	 * @return Debug-Format-Ausgabe
	 */
	protected static String annotationValuesToString(
			final String indent ,
			final Iterable<? extends AsmDomAnnotationValue> values )
	{
		final StringBuilder buff = new StringBuilder();

		for ( final AsmDomAnnotationValue value : values )
		{
			buff.append( value.toString( indent + "\t" ) );
			buff.append( "\n" );
		}
		
		return buff.toString();
	}

	/**
	 * Hilfsmethode zum Ausgeben {@link AsmDomAnnotationValue}s.
	 * 
	 * @param indent Ein&uuml;ckung f&uuml;r Baum-artige-Anzeige
	 * @param values auszugebende Objekte
	 * @return Debug-Format-Ausgabe
	 */
	protected static String objValuesToString(
			final String indent ,
			final Iterable<? extends Object> values )
	{
		final StringBuilder buff = new StringBuilder();

		for ( final Object value : values )
		{
			if ( value instanceof AsmDomAnnotationValue )
			{
				buff.append( ( (AsmDomAnnotationValue) value ).toString( indent + "\t" ) );
			}
			else
			{
				buff.append( String.valueOf( value ) );
			}
			buff.append( "\n" );
		}

		return buff.toString();
	}
}
