package de.heinerkuecker.asm_improved.asm_dom;

import org.objectweb.asm.AnnotationVisitor;

/**
 * {@link AnnotationVisitor}-Implementierung zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmDomArrayValuesAnnotationVisitor
extends AnnotationVisitor
{
	/**
	 * Objekt zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
	 */
	public final AsmDomArray domArray;

	/**
	 * Konstruktor.
	 * 
	 * @param preAnnotationVisitor
	 */
	public AsmDomArrayValuesAnnotationVisitor(
			final AnnotationVisitor preAnnotationVisitor ,
			final String name ,
			final String desc )
	{
		super(
				org.objectweb.asm.Opcodes.ASM5 ,
				preAnnotationVisitor );

		this.domArray = new AsmDomArray( name , desc );

		//System.out.println( ( new Exception() ).getStackTrace()[ 1 ] + " " + name );
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines primitiven
	 * Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visit(java.lang.String, java.lang.Object)
	 */
	@Override
	public void visit(
			final String name ,
			final Object value )
	{
		super.visit( name , value );

		// leider treten Aufrufe mit name == null auf, weil der AnnotationVisitor in ASM in verschiedenen Kontexten verwendet wird
		//if ( name != null )
		//{
		//	throw new IllegalArgumentException( "array value has no name" );
		//}

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + value );

		this.domArray.values.add( new AsmDomAnnotationPrimitiveValue( name , value ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Enum-Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitEnum(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void visitEnum(
			final String name ,
			final String desc ,
			final String value )
	{
		super.visitEnum( name , desc , value );

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + desc + " " + value );

		this.domArray.values.add( new AsmDomAnnotationEnumValue( name , desc , value ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitAnnotation(java.lang.String, java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitAnnotation(
			final String name ,
			final String desc )
	{
		// TODO Auto-generated method stub
		//return super.visitAnnotation(name, desc);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + desc );

		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotation( name , desc );

		if ( desc == null )
		{
			throw new IllegalArgumentException( "array has no class" );
		}
		//if ( desc != null )
		//{
		//	throw new IllegalArgumentException( "array has class" );
		//}

		//final AsmDomArrayValuesAnnotationVisitor nextAnnotationVisitor = new AsmDomArrayValuesAnnotationVisitor( preAnnotationVisitor , name , desc );
		//this.domArray.values.add( nextAnnotationVisitor.domArray );
		final AsmDomAnnotationAnnotationValueVisitor nextAnnotationVisitor = new AsmDomAnnotationAnnotationValueVisitor( preAnnotationVisitor , name , desc );
		this.domArray.values.add( nextAnnotationVisitor.domAnnotationAnnotationValue );

		return nextAnnotationVisitor;
	}

	/**
	 * Diese Methode sollte bei Array-Werten
	 * niemals aufgerufen werden, weil
	 * Annotations-Werte nur eindimensionale
	 * Arrays sein d&uuml;rfen.
	 * 
	 * @throws IllegalStateException
	 * @see org.objectweb.asm.AnnotationVisitor#visitArray(java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitArray(
			final String name )
	{
		// TODO Auto-generated method stub
		//return super.visitArray( name );
		// TODO unglaublicherweise tritt dies in Sch...-ASM doch auf: throw new IllegalStateException( "only one dimensional arrays possible" );

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name );

		final AnnotationVisitor preAnnotationVisitor = super.visitArray( name );

		//final AsmDomArrayValuesAnnotationVisitor nextAnnotationVisitor = new AsmDomArrayValuesAnnotationVisitor( preAnnotationVisitor , name , null );
		//this.domArray.values.add( nextAnnotationVisitor.domArray );

		// ich versuche mal, ob hier eine untergeordnete Annotation mit einem Array-Value gemeint ist
		//final AsmDomAnnotationVisitor nextAnnotationVisitor = new AsmDomAnnotationVisitor( preAnnotationVisitor , new AsmDomAnnotation( this.domArray.desc , /*this.domArray.visibleAtRuntime*/false ) );
		//this.domArray.values.add( nextAnnotationVisitor.domAnnotation );

		final AsmDomAnnotationAnnotationValueVisitor nextAnnotationVisitor = new AsmDomAnnotationAnnotationValueVisitor( preAnnotationVisitor , name , this.domArray.desc );
		this.domArray.values.add( nextAnnotationVisitor.domAnnotationAnnotationValue );

		return nextAnnotationVisitor;
		
		
	}

}
