package de.heinerkuecker.asm_improved.asm_dom;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.asm.ClassVisitor;

/**
 * Klasse zum Vermerken der beim Durchlaufen eines
 * {@link ClassVisitor} aufgetretenen Informationen.
 *  
 * @author Heiner K&uuml;cker
 */
public final class AsmDomClass
{
	public int version;
	public int access;
	public String classRawName;
	public String signature;
	public String superName;
	public String[] interfaceRawNames;
	public String sourceFileName;
	public String additionalDebugInformation;

	/**
	 * Annotationen der Klasse.
	 */
	public final List<AsmDomAnnotation> annotations = new ArrayList<>();

	/**
	 * Felder der Klasse.
	 */
	public final List<AsmDomField> fields = new ArrayList<>();

	/**
	 * Methoden der Klasse.
	 */
	public final List<AsmDomMethod> methods = new ArrayList<>();

	/**
	 * Vermerken der beim Durchlaufen der Methode {@link ClassVisitor#visitOuterClass} aufgetretenen Informationen.
	 */
	public final List<AsmDomOuterClass> outerClasses = new ArrayList<>();

	/**
	 * Vermerken der beim Durchlaufen der Methode {@link ClassVisitor#visitInnerClass} aufgetretenen Informationen.
	 */
	public final List<AsmDomInnerClass> innerClasses = new ArrayList<>();

	/**
	 * Typ-Annotationen der Typ-Parameter dieser Klasse.
	 */
	public final List<List<AsmDomTypeAnnotation>> typeParamTypeAnnotations = new ArrayList<>();

	/**
	 * Typ-Annotationen der Typ-Parameter dieser Klasse.
	 */
	public final List<List<List<AsmDomTypeAnnotation>>> typeParamBoundTypeAnnotations = new ArrayList<>();

	/**
	 * Typ-Annotationen der Super-Typen (extends) dieser Klasse.
	 */
	public final List<AsmDomTypeAnnotation> superTypeAnnotations = new ArrayList<>();

	/**
	 * Typ-Annotationen der Interface-Typen (interface extends/implements) dieser Klasse.
	 */
	public final List<List<AsmDomTypeAnnotation>> interfaceTypeAnnotations = new ArrayList<>();

}
