package de.heinerkuecker.asm_improved.asm_dom;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.TypePath;
import org.objectweb.asm.TypeReference;

import com.sun.xml.internal.ws.org.objectweb.asm.Opcodes;

/**
 * {@link ClassVisitor}-Implementierung zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmDomClassVisitor
extends ClassVisitor
{
	/**
	 * Objekt zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
	 */
	public final AsmDomClass domClass = new AsmDomClass();

	/**
	 * Konstruktor.
	 */
	public AsmDomClassVisitor(
			//final ClassVisitor preClassVisitor
			)
	{
		super(
				org.objectweb.asm.Opcodes.ASM5 
				//preClassVisitor
				);
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken der Klassen-Informationen.
	 * 
	 * @see org.objectweb.asm.ClassVisitor#visit(int, int, java.lang.String, java.lang.String, java.lang.String, java.lang.String[])
	 */
	@Override
	public void visit(
			final int version ,
			final int access ,
			final String name ,
			final String signature ,
			final String superName ,
			final String[] interfaces )
	{
		super.visit( version , access , name , signature , superName , interfaces );

		this.domClass.version = version;
		this.domClass.access = access;
		this.domClass.classRawName = name;
		this.domClass.signature = signature;
		this.domClass.superName = superName;
		this.domClass.interfaceRawNames = interfaces;
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken der Klassen-Informationen.
	 * 
	 * @see org.objectweb.asm.ClassVisitor#visitSource(java.lang.String, java.lang.String)
	 */
	@Override
	public void visitSource(
			final String sourceFileName ,
			final String additionalDebugInformation )
	{
		super.visitSource( sourceFileName , additionalDebugInformation );

		this.domClass.sourceFileName = sourceFileName;
		this.domClass.additionalDebugInformation = additionalDebugInformation;
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken der Klassen-Informationen.
	 * 
	 * @see org.objectweb.asm.ClassVisitor#visitOuterClass(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void visitOuterClass(
			final String owner ,
			final String name ,
			final String desc )
	{
		//throw new RuntimeException( "not implemented" );
		super.visitOuterClass(owner, name, desc);
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + owner + " " + name + " " + desc );
		this.domClass.outerClasses.add( new AsmDomOuterClass( owner , name , desc ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.ClassVisitor#visitAnnotation(java.lang.String, boolean)
	 */
	@Override
	public AnnotationVisitor visitAnnotation(
			final String desc ,
			final boolean visibleAtRuntime )
	{
		// TODO Auto-generated method stub
		//return super.visitAnnotation(desc, visible);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + desc );

		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotation( desc , visibleAtRuntime );

		final AsmDomAnnotationVisitor nextAnnotationVisitor = new AsmDomAnnotationVisitor( preAnnotationVisitor , /*desc , visibleAtRuntime*/ new AsmDomAnnotation( desc , visibleAtRuntime ) );

		this.domClass.annotations.add( nextAnnotationVisitor.domAnnotation );

		return nextAnnotationVisitor;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.ClassVisitor#visitTypeAnnotation(int, org.objectweb.asm.TypePath, java.lang.String, boolean)
	 */
	@Override
	public AnnotationVisitor visitTypeAnnotation(
			final int typeRef ,
			final TypePath typePath ,
			final String desc ,
			final boolean visibleAtRuntime )
	{
		// TODO Auto-generated method stub
		//return super.visitTypeAnnotation(typeRef, typePath, desc, visible);
		//throw new RuntimeException( "not implemented" );

		System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " typeRef: " + typeRef + " typePath: " + typePath + " desc: " + desc + " visibleAtRuntime: " + visibleAtRuntime );

		System.out.println( "typeRef CLASS_TYPE_PARAMETER        " + ( ( typeRef >>> 24 ) == TypeReference.CLASS_TYPE_PARAMETER ) );
		System.out.println( "typeRef CLASS_EXTENDS               " + ( ( typeRef >>> 24 ) == TypeReference.CLASS_EXTENDS ) );
		System.out.println( "typeRef CLASS_TYPE_PARAMETER_BOUND  " + ( ( typeRef >>> 24 ) == TypeReference.CLASS_TYPE_PARAMETER_BOUND ) );

		if ( ( typeRef >>> 24 ) == TypeReference.CLASS_TYPE_PARAMETER )
		{
			final int typeParameterIndex = new TypeReference( typeRef ).getTypeParameterIndex();

			System.out.println( "typeParameterIndex " + typeParameterIndex );

			final AnnotationVisitor preAnnotationVisitor = super.visitTypeAnnotation( typeRef , typePath , desc , visibleAtRuntime );

			final AsmDomTypeAnnotationVisitor nextAnnotationVisitor =
					new AsmDomTypeAnnotationVisitor(
							preAnnotationVisitor ,
							new AsmDomTypeAnnotation(
									desc ,
									typePath ,
									visibleAtRuntime ) );

			while ( this.domClass.typeParamTypeAnnotations.size() <= typeParameterIndex )
				// Listengroesse anpassen
			{
				this.domClass.typeParamTypeAnnotations.add(
						new ArrayList<>() );

			}

			this.domClass.typeParamTypeAnnotations.get(
					typeParameterIndex ).add(
							nextAnnotationVisitor.domTypeAnnotation );

			return nextAnnotationVisitor;
		}

		if ( ( typeRef >>> 24 ) == TypeReference.CLASS_TYPE_PARAMETER_BOUND )
		{
			final int typeParameterIndex = new TypeReference( typeRef ).getTypeParameterIndex();

			System.out.println( "typeParameterIndex " + typeParameterIndex );

			final int typeParameterBoundIndex = new TypeReference( typeRef ).getTypeParameterBoundIndex();

			System.out.println( "typeParameterBoundIndex " + typeParameterBoundIndex );

			final AnnotationVisitor preAnnotationVisitor = super.visitTypeAnnotation( typeRef , typePath , desc , visibleAtRuntime );

			final AsmDomTypeAnnotationVisitor nextAnnotationVisitor =
					new AsmDomTypeAnnotationVisitor(
							preAnnotationVisitor ,
							new AsmDomTypeAnnotation(
									desc ,
									typePath ,
									visibleAtRuntime ) );

			while ( this.domClass.typeParamBoundTypeAnnotations.size() <= typeParameterIndex )
				// Listengroesse anpassen
			{
				this.domClass.typeParamBoundTypeAnnotations.add(
						new ArrayList<>() );

			}

			final List<List<AsmDomTypeAnnotation>> currTypeParamBoundTypeAnnotations =
					this.domClass.typeParamBoundTypeAnnotations.get( typeParameterIndex );

			while ( currTypeParamBoundTypeAnnotations.size() <= typeParameterBoundIndex )
				// Listengroesse anpassen
			{
				currTypeParamBoundTypeAnnotations.add(
						new ArrayList<>() );

			}

			currTypeParamBoundTypeAnnotations.get(
					typeParameterBoundIndex ).add(
							nextAnnotationVisitor.domTypeAnnotation );

			return nextAnnotationVisitor;
		}

		if ( ( typeRef >>> 24 ) == TypeReference.CLASS_EXTENDS )
		{
			final int superTypeIndex = new TypeReference( typeRef ).getSuperTypeIndex();

			System.out.println( "superTypeIndex " + superTypeIndex );

			final AnnotationVisitor preAnnotationVisitor = super.visitTypeAnnotation( typeRef , typePath , desc , visibleAtRuntime );

			final AsmDomTypeAnnotationVisitor nextAnnotationVisitor =
					new AsmDomTypeAnnotationVisitor(
							preAnnotationVisitor ,
							new AsmDomTypeAnnotation(
									desc ,
									typePath ,
									visibleAtRuntime ) );

			if ( superTypeIndex < 0 )
				// super-Klasse
			{
				this.domClass.superTypeAnnotations.add(
						nextAnnotationVisitor.domTypeAnnotation );
			}
			else
				// Interface
			{
				while ( this.domClass.interfaceTypeAnnotations.size() <= superTypeIndex )
					// Listengroesse anpassen
				{
					this.domClass.interfaceTypeAnnotations.add(
							new ArrayList<>() );
				}

				this.domClass.interfaceTypeAnnotations.get(
						superTypeIndex ).add(
								nextAnnotationVisitor.domTypeAnnotation );
			}

			return nextAnnotationVisitor;
		}

		//throw new RuntimeException( "not implemented" );
		// TODO wieder rein:
		throw new IllegalArgumentException( String.valueOf( typeRef ) );
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " not implemented " );
		//return null;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.ClassVisitor#visitAttribute(org.objectweb.asm.Attribute)
	 */
	@Override
	public void visitAttribute(Attribute attr) {
		// TODO Auto-generated method stub
		super.visitAttribute(attr);
		throw new RuntimeException( "not implemented" );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken der Klassen-Informationen.
	 * 
	 * @see org.objectweb.asm.ClassVisitor#visitInnerClass(java.lang.String, java.lang.String, java.lang.String, int)
	 */
	@Override
	public void visitInnerClass(
			final String name ,
			final String outerName ,
			final String innerName ,
			final int access )
	{
		// TODO Auto-generated method stub
		//super.visitInnerClass(name, outerName, innerName, access);
		//throw new RuntimeException( "not implemented" );

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " name: " + name + " outerName: " + outerName + " innerName: " + innerName + " access: " + access + " " + ( ( access & Opcodes.ACC_STATIC ) != 0 ? "static" : ""  ) );
		this.domClass.innerClasses.add( new AsmDomInnerClass( name , outerName , innerName , access ) );
		super.visitInnerClass(name, outerName, innerName, access);
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Felds
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.ClassVisitor#visitField(int, java.lang.String, java.lang.String, java.lang.String, java.lang.Object)
	 */
	@Override
	public FieldVisitor visitField(
			final int access ,
			final String name ,
			final String desc ,
			final String signature ,
			final Object value )
	{
		// TODO Auto-generated method stub
		//return super.visitField(access, name, desc, signature, value);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " +  desc + " " + signature );

		final FieldVisitor preFieldVisitor = super.visitField( access , name , desc , signature , value );

		final AsmDomFieldVisitor nextFieldVisitor = new AsmDomFieldVisitor( preFieldVisitor , access , name , desc , signature , value );

		this.domClass.fields.add( nextFieldVisitor.domField );

		return nextFieldVisitor;
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken einer Methode
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.ClassVisitor#visitMethod(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String[])
	 */
	@Override
	public MethodVisitor visitMethod(
			final int access ,
			final String name ,
			final String desc ,
			final String signature ,
			final String[] exceptions )
	{
		// TODO Auto-generated method stub
		//return super.visitMethod(access, name, desc, signature, exceptions);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " +  desc + " " + signature );

		final MethodVisitor preMethodVisitor = super.visitMethod(access, name, desc, signature, exceptions);

		final AsmDomMethodVisitor nextMethodVisitor = new AsmDomMethodVisitor( preMethodVisitor , access , name , desc , signature , exceptions );

		this.domClass.methods.add( nextMethodVisitor.domMethod );

		return nextMethodVisitor;
	}

}
