package de.heinerkuecker.asm_improved.asm_dom;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Attribute;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.TypePath;

/**
 * {@link FieldVisitor}-Implementierung zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmDomFieldVisitor
extends FieldVisitor
{
	/**
	 * Objekt zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
	 */
	public final AsmDomField domField;
	
	/**
	 * Konstruktor.
	 */
	public AsmDomFieldVisitor(
			final FieldVisitor preFieldVisitor ,
			final int access , 
			final String name ,
			final String desc ,
			final String signature ,
            final Object value )
	{
		super(
				org.objectweb.asm.Opcodes.ASM5 ,
				preFieldVisitor );
		
		domField = new AsmDomField( access , name , desc , signature , value );

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + desc + " " + signature + " " + value );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.FieldVisitor#visitAnnotation(java.lang.String, boolean)
	 */
	@Override
	public AnnotationVisitor visitAnnotation(
			final String desc ,
			final boolean visibleAtRuntime )
	{
		// TODO Auto-generated method stub
		//return super.visitAnnotation(desc, visible);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + desc );

		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotation( desc , visibleAtRuntime );

		final AsmDomAnnotationVisitor nextAnnotationVisitor =
				new AsmDomAnnotationVisitor(
						preAnnotationVisitor ,
						new AsmDomAnnotation(
								desc ,
								visibleAtRuntime ) );

		this.domField.annotations.add( nextAnnotationVisitor.domAnnotation );

		return nextAnnotationVisitor;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.FieldVisitor#visitTypeAnnotation(int, org.objectweb.asm.TypePath, java.lang.String, boolean)
	 */
	@Override
	public AnnotationVisitor visitTypeAnnotation(
			final int typeRef ,
			final TypePath typePath ,
			final String desc ,
			final boolean visibleAtRuntime )
	{
		// TODO Auto-generated method stub
		//return super.visitTypeAnnotation(typeRef, typePath, desc, visible);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " typeRef: " + typeRef + " typePath: " + typePath + " desc: " + desc + " visibleAtRuntime: " + visibleAtRuntime );

		final AnnotationVisitor preAnnotationVisitor = super.visitTypeAnnotation( typeRef , typePath , desc , visibleAtRuntime );

		final AsmDomTypeAnnotationVisitor nextAnnotationVisitor =
				new AsmDomTypeAnnotationVisitor(
						preAnnotationVisitor ,
						new AsmDomTypeAnnotation(
								desc ,
								typePath ,
								visibleAtRuntime ) );

		this.domField.typeAnnotations.add( nextAnnotationVisitor.domTypeAnnotation );

		return nextAnnotationVisitor;
		//throw new RuntimeException( "not implemented" );
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.FieldVisitor#visitAttribute(org.objectweb.asm.Attribute)
	 */
	@Override
	public void visitAttribute(Attribute attr)
	{
		// TODO Auto-generated method stub
		//super.visitAttribute(attr);
		throw new RuntimeException( "not implemented" );
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.FieldVisitor#visitEnd()
	 */
	//@Override
	//public void visitEnd() {
	//	// TODO Auto-generated method stub
	//	super.visitEnd();
	//}

	
}
