package de.heinerkuecker.asm_improved.asm_dom;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.asm.MethodVisitor;

/**
 * Klasse zum Vermerken der beim Durchlaufen eines
 * {@link MethodVisitor} aufgetretenen Informationen.
 *  
 * @author Heiner K&uuml;cker
 */
public final class AsmDomMethod
{
	public final int access;
	public final String name;
	public final String desc;
	public final String signature;
	public final String[] exceptions;

	/**
	 * Annotationen der Methode.
	 */
	public final List<AsmDomAnnotation> annotations = new ArrayList<>();

	/**
	 * Informationen die beim Durchlaufen der Methode {@link de.heinerkuecker.asm_improved.asm_dom.AsmDomMethodVisitor#visitAnnotationDefault()} aufgetreten sind,
	 * falls es sich um eine Methode in einer Annotatuions-Deklaration handelt.
	 */
	public AsmDomAnnotationDefault annotationDefault;
	
	/**
	 * Parameter-Annotationen der Methode.<br/>
	 * &Uuml;bergeordnete Liste fr die Annotationen je Methode mit
	 * untergeordenten Listen fr die Annotationen je Parameter.
	 */
	//public final List<List<AsmDomParameterAnnotation>> parameterAnnotations = new ArrayList<>();
	public final List<List<AsmDomAnnotation>> parameterAnnotationListList = new ArrayList<>();

	/**
	 * Annotationen des Method-Return-Typs.
	 */
	public final List<AsmDomTypeAnnotation> returnTypeAnnotations = new ArrayList<>();

	/**
	 * Annotationen der Methoden-Parameter-Typen.
	 */
	public final List<List<AsmDomTypeAnnotation>> formalParamTypeAnnotations = new ArrayList<>();

	/**
	 * Annotationen der throws-Exception-Deklaration.
	 */
	public final List<List<AsmDomTypeAnnotation>> throwsTypeAnnotations = new ArrayList<>();

	/**
	 * Annotationen der Typ-Parameter dieser Methode.
	 */
	public final List<List<AsmDomTypeAnnotation>> typeParamTypeAnnotations = new ArrayList<>();

	/**
	 * Konstruktor.
	 * 
	 * @param name
	 * @param desc
	 */
	AsmDomMethod(
			final int access ,
			final String name ,
			final String desc ,
			final String signature ,
			final String[] exceptions )
	{
		this.access = access;
		this.name = name;
		this.desc = desc;
		this.signature = signature;
		this.exceptions = exceptions;
	}

	/**
	 * {@inheritDoc}
	 */
	//@Override
	//public String toString(
	//		final String indent )
	//{
	//	return indent + this.getClass().getSimpleName() + this.name + " " + this.desc + "\n" + annotationValuesToString( indent , values );
	//}

}
