package de.heinerkuecker.asm_improved.asm_dom;

import java.util.ArrayList;
import java.util.Arrays;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Attribute;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.TypePath;
import org.objectweb.asm.TypeReference;

/**
 * {@link MethodVisitor}-Implementierung zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmDomMethodVisitor
extends MethodVisitor
{
	/**
	 * Objekt zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
	 */
	public final AsmDomMethod domMethod;

	/**
	 * Konstruktor.
	 */
	public AsmDomMethodVisitor(
			final MethodVisitor preMethodVisitor ,
			final int access ,
			final String name ,
			final String desc ,
			final String signature ,
			final String[] exceptions )
	{
		super(
				org.objectweb.asm.Opcodes.ASM5 ,
				preMethodVisitor );

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + access + " " + name + " " + desc + " " + signature + " " + Arrays.toString( exceptions ) );

		this.domMethod = new AsmDomMethod(access, name, desc, signature, exceptions);
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.MethodVisitor#visitParameter(java.lang.String, int)
	 */
	@Override
	public void visitParameter(String name, int access) {
		// TODO Auto-generated method stub
		//super.visitParameter(name, access);
		throw new RuntimeException( "not implemented" );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 */
	@Override
	public AnnotationVisitor visitAnnotationDefault()
	{
		// TODO Auto-generated method stub
		//return super.visitAnnotationDefault();

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] );

		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotationDefault();

		final AsmDomAnnotationDefaultAnnotationVisitor nextAnnotationVisitor = new AsmDomAnnotationDefaultAnnotationVisitor( preAnnotationVisitor );

		this.domMethod.annotationDefault = nextAnnotationVisitor.domAnnotationDefault;

		return nextAnnotationVisitor;
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 */
	@Override
	public AnnotationVisitor visitAnnotation(
			final String desc ,
			final boolean visibleAtRuntime )
	{
		// TODO Auto-generated method stub
		//return super.visitAnnotation(desc, visible);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + desc + " " +  visibleAtRuntime );

		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotation( desc , visibleAtRuntime );

		final AsmDomAnnotationVisitor nextAnnotationVisitor = new AsmDomAnnotationVisitor( preAnnotationVisitor , /*desc , visibleAtRuntime*/ new AsmDomAnnotation( desc , visibleAtRuntime ) );

		this.domMethod.annotations.add( nextAnnotationVisitor.domAnnotation );

		return nextAnnotationVisitor;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.MethodVisitor#visitTypeAnnotation(int, org.objectweb.asm.TypePath, java.lang.String, boolean)
	 */
	@Override
	public AnnotationVisitor visitTypeAnnotation(
			final int typeRef ,
			final TypePath typePath ,
			final String desc ,
			final boolean visibleAtRuntime )
	{
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + ( typeRef ) + " " + ( typeRef  >>> 24 ) + " " + typePath + " " + desc + " " + visibleAtRuntime );

		//System.out.println( "typeRef METHOD_TYPE_PARAMETER       " + ( ( typeRef >>> 24 ) == TypeReference.METHOD_TYPE_PARAMETER ) );
		//System.out.println( "typeRef METHOD_TYPE_PARAMETER_BOUND " + ( ( typeRef >>> 24 ) == TypeReference.METHOD_TYPE_PARAMETER_BOUND ) );
		//System.out.println( "typeRef METHOD_RETURN               " + ( ( typeRef >>> 24 ) == TypeReference.METHOD_RETURN ) );
		//System.out.println( "typeRef METHOD_RECEIVER             " + ( ( typeRef >>> 24 ) == TypeReference.METHOD_RECEIVER ) );
		//System.out.println( "typeRef METHOD_FORMAL_PARAMETER     " + ( ( typeRef >>> 24 ) == TypeReference.METHOD_FORMAL_PARAMETER ) );
		//System.out.println( "typeRef THROWS                      " + ( ( typeRef >>> 24 ) == TypeReference.THROWS ) );

		// TODO Auto-generated method stub
		//return super.visitTypeAnnotation(typeRef, typePath, desc, visibleAtRuntime);
		final AnnotationVisitor preAnnotationVisitor = super.visitTypeAnnotation( typeRef , typePath , desc , visibleAtRuntime );

		if ( /*( typeRef >>> 24 )*/ new TypeReference( typeRef ).getSort() == TypeReference.METHOD_RETURN )
		{
			//final AsmDomAnnotationVisitor nextAnnotationVisitor = new AsmDomAnnotationVisitor( preAnnotationVisitor , /*desc , visibleAtRuntime*/ new AsmDomAnnotation( desc , visibleAtRuntime ) );
			//this.domMethod.annotations.add( nextAnnotationVisitor.domAnnotation );
			//return nextAnnotationVisitor;

			// TODO 02.09.2016 veraltet, typ parameter implementiert: hier nichts tun, weil fr Parameter-Annotationen noch die Methode visitParameterAnnotation aufgerufen wird
			//return preAnnotationVisitor;

			final AsmDomTypeAnnotationVisitor nextAnnotationVisitor =
					new AsmDomTypeAnnotationVisitor(
							preAnnotationVisitor ,
							new AsmDomTypeAnnotation(
									desc ,
									typePath ,
									visibleAtRuntime ) );

			this.domMethod.returnTypeAnnotations.add( nextAnnotationVisitor.domTypeAnnotation );

			return nextAnnotationVisitor;
		}

		if ( ( typeRef >>> 24 ) == TypeReference.METHOD_FORMAL_PARAMETER )
		{
			// TODO 02.09.2016 veraltet, typ parameter implementiert: hier nichts tun, weil fr Parameter-Annotationen noch die Methode visitParameterAnnotation aufgerufen wird
			//return preAnnotationVisitor;

			final int formalParamIndex = new TypeReference( typeRef ).getFormalParameterIndex();

			//System.out.println( "typeRef formal param index " + formalParamIndex );

			final AsmDomTypeAnnotationVisitor nextAnnotationVisitor =
					new AsmDomTypeAnnotationVisitor(
							preAnnotationVisitor ,
							new AsmDomTypeAnnotation(
									desc ,
									typePath ,
									visibleAtRuntime ) );

			while ( this.domMethod.formalParamTypeAnnotations.size() <= formalParamIndex )
				// Listengroesse anpassen
			{
				this.domMethod.formalParamTypeAnnotations.add(
						new ArrayList<>() );

			}

			this.domMethod.formalParamTypeAnnotations.get(
					formalParamIndex ).add(
							nextAnnotationVisitor.domTypeAnnotation );

			return nextAnnotationVisitor;
		}

		if ( ( typeRef >>> 24 ) == TypeReference.THROWS )
		{
			//System.out.println( "throws index " + new TypeReference/*.newExceptionReference*/( typeRef ).getExceptionIndex() );

			final AsmDomTypeAnnotationVisitor nextAnnotationVisitor =
					new AsmDomTypeAnnotationVisitor(
							preAnnotationVisitor ,
							new AsmDomTypeAnnotation(
									desc ,
									typePath ,
									visibleAtRuntime ) );

			final int throwsIndex = new TypeReference/*.newExceptionReference*/( typeRef ).getExceptionIndex();

			while ( this.domMethod.throwsTypeAnnotations.size() <= throwsIndex )
				// Listengroesse anpassen
			{
				this.domMethod.throwsTypeAnnotations.add(
						new ArrayList<>() );

			}

			this.domMethod.throwsTypeAnnotations.get(
					throwsIndex ).add(
							nextAnnotationVisitor.domTypeAnnotation );

			return nextAnnotationVisitor;
		}

		if ( ( typeRef >>> 24 ) == TypeReference.METHOD_TYPE_PARAMETER )
		{
			//System.out.println( "type param index " + new TypeReference/*.newExceptionReference*/( typeRef ).getTypeParameterIndex() );

			final AsmDomTypeAnnotationVisitor nextAnnotationVisitor =
					new AsmDomTypeAnnotationVisitor(
							preAnnotationVisitor ,
							new AsmDomTypeAnnotation(
									desc ,
									typePath ,
									visibleAtRuntime ) );

			final int typeParamIndex = new TypeReference/*.newExceptionReference*/( typeRef ).getTypeParameterIndex();

			while ( this.domMethod.typeParamTypeAnnotations.size() <= typeParamIndex )
				// Listengroesse anpassen
			{
				this.domMethod.typeParamTypeAnnotations.add(
						new ArrayList<>() );

			}

			this.domMethod.typeParamTypeAnnotations.get(
					typeParamIndex ).add(
							nextAnnotationVisitor.domTypeAnnotation );

			return nextAnnotationVisitor;
		}
		
		//throw new RuntimeException( "not implemented" );
		throw new IllegalArgumentException( String.valueOf( typeRef ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Parameter-Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 */
	@Override
	public AnnotationVisitor visitParameterAnnotation(
			final int parameterIndex ,
			final String desc ,
			final boolean visibleAtRuntime )
	{
		// TODO Auto-generated method stub
		//return super.visitParameterAnnotation(parameter, desc, visible);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + parameterIndex + " " + desc + " " +  visibleAtRuntime );

		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotation( desc , visibleAtRuntime );

		//final AsmDomAnnotation asmDomParameterAnnotation = new AsmDomParameterAnnotation( parameterIndex , desc , visibleAtRuntime );
		final AsmDomAnnotation parameterDomAnnotation = new AsmDomAnnotation( desc, visibleAtRuntime );

		//final AsmDomAnnotationVisitor nextAnnotationVisitor = new AsmDomAnnotationVisitor( preAnnotationVisitor , /*desc , visibleAtRuntime*/ asmDomParameterAnnotation.domAnnotation );
		final AsmDomAnnotationVisitor nextAnnotationVisitor = new AsmDomAnnotationVisitor( preAnnotationVisitor , /*desc , visibleAtRuntime*/ parameterDomAnnotation );

		while ( this.domMethod.parameterAnnotationListList.size() <= parameterIndex )
			// Listengroesse anpassen
		{
			this.domMethod.parameterAnnotationListList.add(
					new ArrayList<>() );
		}

		this.domMethod.parameterAnnotationListList.get( parameterIndex ).add( parameterDomAnnotation );

		return nextAnnotationVisitor;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.MethodVisitor#visitAttribute(org.objectweb.asm.Attribute)
	 */
	@Override
	public void visitAttribute(Attribute attr) {
		// TODO Auto-generated method stub
		//super.visitAttribute(attr);
		throw new RuntimeException( "not implemented" );
	}

}
