package de.heinerkuecker.asm_improved.asm_dom;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.TypePath;

import de.heinerkuecker.asm_improved.type.AsmTypePath;

/**
 * Informationen einer Annotation.<br/>
 * 
 * Klasse zum Vermerken der beim Durchlaufen eines
 * {@link AnnotationVisitor} aufgetretenen Informationen.
 * 
 * @author Heiner K&uuml;cker
 */
public class AsmDomTypeAnnotation
{
	/**
	 * Klassen-Information der Annotation.
	 */
	public final String desc;

	/**
	 * Typ-Pfad.
	 */
	public final AsmTypePath typePath;

	/**
	 * Ob die Annotation zur Laufzeit sichtbar ist.
	 * Wird von ASM immer mitgeliefert und deshalb hier vermerkt.
	 */
	public final boolean visibleAtRuntime;

	/**
	 * Werte/Parameter der Annotation.
	 */
	public final List<AsmDomAnnotationValue> values = new ArrayList<>();

	/**
	 * Konstruktor.
	 * 
	 * @param desc
	 * @param nativeTypePath 
	 * @param visibleAtRuntime
	 */
	AsmDomTypeAnnotation(
			final String desc ,
			final TypePath nativeTypePath ,
			final boolean visibleAtRuntime )
	{
		this.desc = desc;

		this.typePath =
				AsmTypePath.fromNativeTypePath(
						nativeTypePath );

		this.visibleAtRuntime = visibleAtRuntime;
	}


}
