package de.heinerkuecker.asm_improved.asm_dom;

import org.objectweb.asm.AnnotationVisitor;

/**
 * Sammeln der Informationen einer Typ-Annotation,
 * welche einer Annotation als Wert/Parameter
 * &uuml;bergeben wird.<br/>
 * {@link AnnotationVisitor}-Implementierung zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmDomTypeAnnotationVisitor
extends AnnotationVisitor
{
	/**
	 * Objekt zum Vermerken der beim Durchlaufen/Besuchen aufgetretenen Informationen.
	 */
	public final AsmDomTypeAnnotation domTypeAnnotation;

	/**
	 * Konstruktor.
	 * 
	 * @param preAnnotationVisitor
	 */
	AsmDomTypeAnnotationVisitor(
			final AnnotationVisitor preAnnotationVisitor ,
			//final String name ,
			//final String desc ,
			//final boolean visibleAtRuntime ,
			final AsmDomTypeAnnotation domTypeAnnotation )
	{
		super(
				org.objectweb.asm.Opcodes.ASM5 ,
				preAnnotationVisitor );

		//this.domAnnotation = new AsmDomAnnotation( desc , visibleAtRuntime );
		this.domTypeAnnotation = domTypeAnnotation;

		//System.out.println( ( new Exception() ).getStackTrace()[ 1 ] );
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines primitiven
	 * Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visit(java.lang.String, java.lang.Object)
	 */
	@Override
	public void visit(
			final String name ,
			final Object value )
	{
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " +  value );

		super.visit( name , value );
		
		this.domTypeAnnotation.values.add( new AsmDomAnnotationPrimitiveValue( name , value ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Enum-Wertes.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitEnum(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void visitEnum(
			final String name ,
			final String desc ,
			final String value )
	{
		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " +  desc + " " +  value );

		super.visitEnum(name, desc, value);

		this.domTypeAnnotation.values.add( new AsmDomAnnotationEnumValue( name , desc , value ) );
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitAnnotation(java.lang.String, java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitAnnotation(
			final String name ,
			final String desc )
	{
		// TODO Auto-generated method stub
		//return super.visitAnnotation(name, desc);

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name + " " + desc );

		final AnnotationVisitor preAnnotationVisitor = super.visitAnnotation( name , desc );
		
		final AsmDomAnnotationAnnotationValueVisitor nextAnnotationVisitor =
				new AsmDomAnnotationAnnotationValueVisitor(
						preAnnotationVisitor ,
						name ,
						desc );

		this.domTypeAnnotation.values.add( nextAnnotationVisitor.domAnnotationAnnotationValue );
		
		return nextAnnotationVisitor;
	}

	/**
	 * berschreiben der geerbten Methode
	 * zum Vermerken eines Annotations-Wertes
	 * und zum Zur&uuml;ckgeben eines
	 * Visitors zum Sammeln der untergeordneten
	 * Informationen.
	 * 
	 * @see org.objectweb.asm.AnnotationVisitor#visitArray(java.lang.String)
	 */
	@Override
	public AnnotationVisitor visitArray(
			final String name )
	{
		// TODO Auto-generated method stub
		//return super.visitArray( name );

		//System.out.println( ( new Exception() ).getStackTrace()[ 0 ] + " " + name );

		final AnnotationVisitor preAnnotationVisitor = super.visitArray( name );

		final AsmDomArrayValuesAnnotationVisitor nextAnnotationVisitor =
				new AsmDomArrayValuesAnnotationVisitor(
						preAnnotationVisitor ,
						name ,
						null );

		this.domTypeAnnotation.values.add( nextAnnotationVisitor.domArray );

		return nextAnnotationVisitor;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.AnnotationVisitor#visitEnd()
	 */
	//@Override
	//public void visitEnd() {
	//	// TODO Auto-generated method stub
	//	super.visitEnd();
	//}

}
