package de.heinerkuecker.asm_improved.type;

import java.util.Map;
import java.util.Stack;

import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.util.CollectionUtil;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;
import de.heinerkuecker.asm_improved.util.SeparatedStrBuff;

/**
 * Wurzel-(Ober-)Interface aller ASM-Typen.
 * 
 * @author Heiner K&uuml;cker
 * 
 * TODO rename to AbstractAsmType
 */
public abstract class AsmType
{
	/**
	 * Typ-Annotationen.
	 */
	public final ImmutableArray<? extends AsmAnnotationUse> typeAnnotations;

	/**
	 * Konstruktor.
	 * 
	 * @param typeAnnotations
	 */
	protected AsmType(
			final ImmutableArray<? extends AsmAnnotationUse> typeAnnotations )
	{
		this.typeAnnotations = typeAnnotations;
	}

	/**
	 * @return primitive type or not
	 */
	abstract public boolean isPrimitive();

	/**
	 * Ob eine Variable oder ein Paramter
	 * von diesem Typ von einem Wert des
	 * bergebenen Typs zugewiesen werden kann.
	 * <br/>
	 * Also muss dieser Typ gleich oder
	 * ein Super-Typ des bergebenen Typs
	 * sein oder dieser Typ ist ein Primitive,
	 * in welchen der bergebene Wert ohne
	 * Cast umgewandelt werden kann.
	 * 
	 * @param typeToTest
     * @return the {@code boolean} value indicating whether objects of the
     * specified type {@code typeToTest} can be assigned to objects of this class
	 * @see Class#isAssignableFrom(Class)
	 */
	abstract public boolean isAssignableFrom(
    		final AsmType typeToTest ,
    		final AsmClasspathInfo asmClasspathInfo );

    /**
     * @return in Java-Quellcode verwendbarer String, vollqualifiziert (nicht auf <code>import</code> angewiesen)
     */
	abstract public String toJavaString();

    /**
     * @return in Java-Quellcode verwendbarer String, vollqualifiziert (nicht auf <code>import</code> angewiesen), ohne Typ-Annotationen
     * 
     * TODO kann weg
     */
	abstract public String toJavaStringWithoutTypeAnnotations();

	abstract public AsmType resolveBound(
    		// Typ-Argumente der konkreten Instanz dieser Klasse
    		final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap );

    /**
     * 
     * @param typeargsNameToBoundMap
     * @param typevarNameStack Stack zum Vermeiden {@link StackOverflowError} bei rekursiv verwendeten Typ-Variablen
     * @return
     */
	abstract public AsmType resolveBound(
    		// Typ-Argumente der konkreten Instanz dieser Klasse
    		final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap ,
    		final Stack<String> typevarNameStack );

    /**
     * Add specified type annotations to this type without side effects.
     * 
     * @param typeAnnotationsMap asm dom annotations to add
     * @return annotated type
     */
	abstract public AsmType cloneAddTypeAnnotations(
			final Map<AsmTypePath, ImmutableArray<AsmAnnotationUse>> typeAnnotationsMap ,
			final AsmTypePath typePath );

	/**
	 * @return type annotaions as string
	 */
	protected final String typeAnnotationsToStringWithDelimiterSpace() {
		return CollectionUtil.isNullOrEmpty( this.typeAnnotations ) ? "" : SeparatedStrBuff.join( this.typeAnnotations ,  " "  ) + " ";
	}
}
