package de.heinerkuecker.asm_improved.type.generics.arg;

import java.util.Map;
import java.util.Stack;

import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.AsmTypePath;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;

/**
 * Extends wildcard.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmExtendsWildcard
extends AsmWildcard
{
	public final AsmType boundType;

	/**
	 * Konstruktor.
	 */
	public AsmExtendsWildcard(
			final ImmutableArray<AsmAnnotationUse> typeAnnotations ,
			final AsmType boundType )
	{
		super ( typeAnnotations );
		this.boundType = boundType;
	}

	/**
	 * Konstruktor.
	 */
	public AsmExtendsWildcard(
			final AsmType boundType )
	{
		super ( null );
		this.boundType = boundType;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaString()
	{
    	return typeAnnotationsToStringWithDelimiterSpace() + "? extends " + this.boundType;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaStringWithoutTypeAnnotations()
	{
    	return "? extends " + this.boundType;
	}

	/**
	 * {@inheritDoc}
	 */
    @Override
    public String toString()
    {
	    //return "AsmExtendsWildcard [boundType=" + boundType + "]";
		return toJavaString();
    }

	/**
	 * {@inheritDoc}
	 */
    @Override
    public boolean isPrimitive()
    {
	    return false;
    }

	/**
	 * {@inheritDoc}
	 */
    @Override
    public boolean isAssignableFrom(
    		final AsmType typeToTest ,
    		final AsmClasspathInfo asmClasspathInfo )
    {
    	throw new RuntimeException( "nicht fertig" );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode()
    {
        final int prime = 31;
        int result = 1;
        result = prime * result + ( ( boundType == null ) ? 0 : boundType.hashCode() );
        return result;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals( Object obj )
    {
        if ( this == obj )
            return true;
        if ( obj == null )
            return false;
        if ( getClass() != obj.getClass() )
            return false;
        AsmExtendsWildcard other = (AsmExtendsWildcard) obj;
        if ( boundType == null )
        {
            if ( other.boundType != null )
                return false;
        }
        else if ( !boundType.equals( other.boundType ) )
            return false;
        return true;
    }

	@Override
	public AsmType resolveBound(
			ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap )
	{
		return this.boundType.resolveBound(
				typeargsNameToBoundMap ,
				new Stack<String>() );
	}

	@Override
	public AsmType resolveBound(
			ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap ,
			final Stack<String> typevarNameStack )
	{
		return this.boundType.resolveBound(
				typeargsNameToBoundMap ,
				typevarNameStack );
	}

	@Override
	public AsmExtendsWildcard cloneAddTypeAnnotations(
			final Map<AsmTypePath, ImmutableArray<AsmAnnotationUse>> typeAnnotationsMap ,
			final AsmTypePath typePath )
	{
		if ( this.typeAnnotations != null )
			// zweimal Typ-Annotationen hinzufgen sollte nicht vorkommen
		{
			throw new IllegalStateException();
		}

		return new AsmExtendsWildcard(
				typeAnnotationsMap.get( typePath ) ,
				this.boundType.cloneAddTypeAnnotations(
						typeAnnotationsMap ,
						typePath.cloneAdd(
								new AsmTypePath.WILDCARD_BOUND(
										null ) ) ) );
		//throw new RuntimeException( "nicht fertig" );
	}

}
