package de.heinerkuecker.asm_improved.type.generics.arg;

import java.util.Map;
import java.util.Stack;

import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.AsmTypePath;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;

/**
 * Wildcard ohne Bound &lt;?&gt;.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmUnboundWildcard
extends AsmWildcard
{
	/**
	 * Siingleton-Instanz.
	 */
	public static final AsmUnboundWildcard INSTANCE = new AsmUnboundWildcard( null );
	
	/**
	 * Singleton-Konstruktor, private.
	 */
    private AsmUnboundWildcard(
    		final ImmutableArray<AsmAnnotationUse> typeAnnotations )
    {
	    super( typeAnnotations );
    }

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaString()
	{
    	return typeAnnotationsToStringWithDelimiterSpace() + toJavaStringWithoutTypeAnnotations();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaStringWithoutTypeAnnotations()
	{
    	return "?";
	}

	/**
	 * {@inheritDoc}
	 */
    @Override
    public String toString()
    {
		return toJavaString();
    }

	/**
	 * {@inheritDoc}
	 */
    @Override
    public boolean isPrimitive()
    {
	    return false;
    }

	/**
	 * {@inheritDoc}
	 */
    @Override
    public boolean isAssignableFrom(
    		final AsmType typeToTest ,
    		final AsmClasspathInfo asmClasspathInfo )
    {
    	throw new RuntimeException( "nicht fertig" );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode()
    {
        return getClass().getCanonicalName().hashCode();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals( 
            final Object obj )
    {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        return getClass() == obj.getClass();
    }

	@Override
	public AsmClassType resolveBound(
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap )
	{
		return AsmClassType.JAVA_LANG_OBJECT;
	}

	@Override
	public AsmClassType resolveBound(
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap ,
			final Stack<String> typevarNameStack )
	{
		return AsmClassType.JAVA_LANG_OBJECT;
	}

	@Override
	public AsmUnboundWildcard cloneAddTypeAnnotations(
			final Map<AsmTypePath, ImmutableArray<AsmAnnotationUse>> typeAnnotationsMap ,
			final AsmTypePath typePath )
	{
		if ( this.typeAnnotations != null )
			// zweimal Typ-Annotationen hinzufgen sollte nicht vorkommen
		{
			throw new IllegalStateException();
		}

		return new AsmUnboundWildcard(
				typeAnnotationsMap.get( typePath ) );
		//throw new RuntimeException( "nicht fertig" );
	}

}
