package de.heinerkuecker.asm_improved.type.generics.param;

import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.util.CollectionUtil;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.JavaIdentifier;
import de.heinerkuecker.asm_improved.util.SeparatedStrBuff;

/**
 * Abstrakte Oberklasse der ASM-Type-Parameter.
 * 
 * @author Heiner K&uuml;cker
 */
public abstract class AsmAbstractTypeParam
{
	/**
	 * Typ-Annotationen.
	 */
	public final ImmutableArray<? extends AsmAnnotationUse> typeAnnotations;

	/**
	 * Name des Typ-Parameters.
	 */
	public final String name;

	/**
	 * Konstruktor.
	 * 
	 * @param name see {@link #name}
	 */
	protected AsmAbstractTypeParam(
			final ImmutableArray<? extends AsmAnnotationUse> typeAnnotations ,
			final String name )
	{
		this.typeAnnotations = typeAnnotations;
		
		if ( ! JavaIdentifier.is( name ) )
		{
			throw new IllegalArgumentException(
					"name: " + name );
		}
		this.name = name;
	}

	/**
	 * @return type annotaions as string
	 */
	protected final String typeAnnotationsToStringWithDelimiterSpace() {
		return CollectionUtil.isNullOrEmpty( this.typeAnnotations ) ? "" : SeparatedStrBuff.join( this.typeAnnotations ,  " "  ) + " ";
	}

}
