package de.heinerkuecker.asm_improved.type.generics.param;

import java.util.Map;
import java.util.Stack;

import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.AsmTypePath;
import de.heinerkuecker.asm_improved.type.util.AsmTypeUtil;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;
import de.heinerkuecker.asm_improved.util.SeparatedStrBuff;

/**
 * Mehrere UND-verkn&uuml;fte Klassen oder Interfaces
 * als Bound eines Typ-Parameters.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmIntersectionBound
extends AsmType
{
	/**
	 * Bound-Types.
	 */
	public final ImmutableArray<? extends AsmType> boundTypes;

	/**
	 * Konstruktor.
	 * 
	 * @param boundTypes
	 */
	public AsmIntersectionBound(
			final ImmutableArray<? extends AsmType> boundTypes )
	{
		super( null );
		this.boundTypes = boundTypes;
	}

	@Override
	public boolean isPrimitive()
	{
		return false;
	}

	@Override
	public boolean isAssignableFrom(
			final AsmType typeToTest,
			final AsmClasspathInfo asmClasspathInfo)
	{
		throw new RuntimeException( "nicht fertig" );
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaString()
	{
    	return typeAnnotationsToStringWithDelimiterSpace() + toJavaStringWithoutTypeAnnotations();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaStringWithoutTypeAnnotations()
	{
		final SeparatedStrBuff boundTypesBuff =
				new SeparatedStrBuff( " & " );

		for ( final AsmType boundType : this.boundTypes )
		{
			boundTypesBuff.add(
					String.valueOf(
							boundType ) );
		}

		return boundTypesBuff.toString();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString()
	{
		return toJavaString();
	}

	@Override
	public AsmIntersectionBound resolveBound(
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap )
	{
		return new AsmIntersectionBound(
				AsmTypeUtil.resolveTypeArgs(
						boundTypes ,
						typeargsNameToBoundMap ,
						new Stack<String>() ) );
	}

	@Override
	public AsmIntersectionBound resolveBound(
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap ,
			final Stack<String> typevarNameStack )
	{
		return new AsmIntersectionBound(
				AsmTypeUtil.resolveTypeArgs(
						boundTypes ,
						typeargsNameToBoundMap ,
						typevarNameStack ) );
	}

	// TODO sollte eigentlich nie aufgerufen werden throw new UnsupportedOperationException
	@Override
	public AsmIntersectionBound cloneAddTypeAnnotations(
			final Map<AsmTypePath, ImmutableArray<AsmAnnotationUse>> typeAnnotationsMap ,
			final AsmTypePath typePath )
	{
		//if ( this.typeAnnotations != null )
		//	// zweimal Typ-Annotationen hinzufgen sollte nicht vorkommen
		//{
		//	throw new IllegalStateException();
		//}
		//throw new RuntimeException( "nicht fertig" );
		// TODO jedem Einzel-Bound einzeln die Type-Annotationen hinzufuegen
		throw new UnsupportedOperationException();
	}

}
