package de.heinerkuecker.asm_improved.type.generics.var;

import java.util.Map;
import java.util.Stack;

import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmTopLevelOrInnerClassTypeOrTypevar;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.AsmTypePath;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;
import de.heinerkuecker.asm_improved.util.JavaIdentifier;

/**
 * Type-Variable.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmTypevar
extends AsmTopLevelOrInnerClassTypeOrTypevar
{
	/**
	 * Name des Typ-Parameters/Variable
	 */
	public final String name;

	/**
	 * Konstruktor.
	 * 
	 * @param name see {@link #name}
	 */
	public AsmTypevar(
			final ImmutableArray<AsmAnnotationUse> typeAnnotations ,
			final String name )
	{
		super( typeAnnotations );
		this.name = name;

		if ( ! JavaIdentifier.is( name ) )
		{
			throw new IllegalArgumentException( name );
		}
	}

	/**
	 * Konstruktor.
	 * 
	 * @param name see {@link #name}
	 */
	public AsmTypevar(
			final String name )
	{
		this(
				null ,
				name );
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaString()
	{
    	return typeAnnotationsToStringWithDelimiterSpace() + toJavaStringWithoutTypeAnnotations();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaStringWithoutTypeAnnotations()
	{
		return this.name;
	}

	/**
	 * {@inheritDoc}
	 */
    @Override
    public String toString()
    {
		return toJavaString();
    }

	/**
	 * {@inheritDoc}
	 */
    @Override
    public boolean isPrimitive()
    {
	    return false;
    }

	/**
	 * {@inheritDoc}
	 */
    @Override
    public boolean isAssignableFrom(
    		final AsmType typeToTest ,
    		final AsmClasspathInfo asmClasspathInfo )
    {
    	throw new RuntimeException( "nicht fertig" );
    }

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((this.name == null) ? 0 : this.name.hashCode());
		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		AsmTypevar other = (AsmTypevar) obj;
		if (this.name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!this.name.equals(other.name)) {
			return false;
		}
		return true;
	}

	@Override
	public AsmType resolveBound(
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap )
	{
		return resolveBound(
				typeargsNameToBoundMap ,
				new Stack<String>() );
	}

	@Override
	public AsmType resolveBound(
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap ,
			final Stack<String> typevarNameStack )
	{
		if ( typeargsNameToBoundMap == null )
		{
			return AsmClassType.JAVA_LANG_OBJECT;
		}
		
		final AsmType bound = typeargsNameToBoundMap.get( name );

		if ( bound == null )
		{
			return AsmClassType.JAVA_LANG_OBJECT;
		}

		if ( bound.equals( this ) )
			// avoid stack over flow
		{
			return AsmClassType.JAVA_LANG_OBJECT;
		}

		if ( typevarNameStack.contains( this.name ) )
			// avoid stack over flow
		{
			return AsmClassType.JAVA_LANG_OBJECT;
		}

		typevarNameStack.push( this.name );
		
		final AsmType resolvedBound =
				bound.resolveBound(
						typeargsNameToBoundMap ,
						typevarNameStack );

		typevarNameStack.pop();

		return resolvedBound;
	}

	@Override
	public AsmTypevar cloneAddTypeAnnotations(
			final Map<AsmTypePath, ImmutableArray<AsmAnnotationUse>> typeAnnotationsMap ,
			final AsmTypePath typePath )
	{
		if ( this.typeAnnotations != null )
			// zweimal Typ-Annotationen hinzufgen sollte nicht vorkommen
		{
			throw new IllegalStateException();
		}

		return new AsmTypevar(
				typeAnnotationsMap.get( typePath ) ,
				this.name );
		//throw new RuntimeException( "nicht fertig" );
	}

}
