package de.heinerkuecker.asm_improved.type.member;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Stack;

import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmTopLevelOrInnerClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.AsmTypePath;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.util.AsmTypeUtil;
import de.heinerkuecker.asm_improved.util.DottedName;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;
import de.heinerkuecker.asm_improved.util.SeparatedStrBuff;

/**
 * Inner class, non static,
 * local or anonymus.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmInnerClassType
extends AsmTopLevelOrInnerClassType
{
	/**
	 * Outer or enclosing type of this type.
	 * 
	 * TODO rename to outerType, weil dies nur nicht statische innere Klassen sein sollten
	 */
	public final AsmTopLevelOrInnerClassType outerOrEnclosingType;

	//public final Fqn fqn;
	//public final String simpleName;

	// TODO eventuell Name ohne punkte typsicher
	//public final DottedName relativName;
	/**
	 * Name der inneren Klasse ohne innere Punkte.
	 * 
	 * Beruht darauf, dass in einer nicht statischen inneren Klasse keine statische innere Klasse mehr deklariert werden darf.
	 */
	//public final JavaIdentifier relativName;
	// wegen anonymen und lokalen Klassen kann hier ein nicht-Java-konformer Bezeichner auftauchen
	public final String relativName;

	public final ImmutableArray<? extends AsmType> typeArgs;

	/**
	 * Konstruktor.
	 */
	public AsmInnerClassType(
			final ImmutableArray<? extends AsmAnnotationUse> typeAnnotations ,
			final AsmTopLevelOrInnerClassType outerOrEnclosingType ,
			//final Fqn fqn ,
			//final String simpleName ,
			//final DottedName relativName ,
			//final JavaIdentifier relativName ,
			final String relativName ,
			final ImmutableArray<? extends AsmType> typeArgs )
	{
		super( typeAnnotations );
		
		if ( outerOrEnclosingType == null )
		{
			throw new IllegalArgumentException();
		}
		
		this.outerOrEnclosingType = outerOrEnclosingType;
		//this.fqn = fqn;
		//this.simpleName = simpleName;

		if ( //relativName == null ||
				relativName.contains( "." ) )
		{
			throw new IllegalArgumentException(
					relativName );
		}
		
		this.relativName = relativName;
		this.typeArgs = typeArgs;

		//if ( ! JavaIdentifier.is( simpleName ) )
		//{
		//	throw new IllegalArgumentException( simpleName );
		//}
	}

	/**
	 * Konstruktor.
	 */
	public AsmInnerClassType(
			final AsmTopLevelOrInnerClassType outerOrEnclosingType ,
			//final Fqn fqn ,
			//final String simpleName ,
			//final DottedName relativName ,
			//final JavaIdentifier relativName ,
			final String relativName ,
			final ImmutableArray<? extends AsmType> typeArgs )
	{
		this(
				null ,
				outerOrEnclosingType ,
				relativName ,
				typeArgs );
	}

	/**
	 * Konstruktor.
	 */
	public AsmInnerClassType(
			final AsmTopLevelOrInnerClassType outerOrEnclosingType ,
			//final DottedName relativName ,
			//final JavaIdentifier relativName ,
			final String relativName ,
			final AsmType... typeArgs )
	{
		this(
				null ,
				outerOrEnclosingType ,
				relativName ,
				( typeArgs == null || typeArgs.length < 1 
				? null 
						: ImmutableArray.create( typeArgs ) ) );
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaString()
	{
    	final String typeAnnotationsToStringWithDelimiterSpace = typeAnnotationsToStringWithDelimiterSpace();

    	return 
    			this.outerOrEnclosingType +
    			"." +
    			( typeAnnotationsToStringWithDelimiterSpace.isEmpty() ? "" : " " ) +
    			typeAnnotationsToStringWithDelimiterSpace +
    			this./*simpleName*/relativName +
    			typeArgsToStr();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaStringWithoutTypeAnnotations()
	{
    	return this.outerOrEnclosingType + "." + this./*simpleName*/relativName + typeArgsToStr();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString()
	{
		return this.toJavaString();
	}

	/**
	 * @return
	 */
	// TODO copy paste code
	private String typeArgsToStr()
	{
		if ( this.typeArgs == null || this.typeArgs.isEmpty() )
		{
			return "";
		}

		final SeparatedStrBuff commaSpaceSepStrBuff =
				new SeparatedStrBuff( ", " );

		for ( final AsmType typeArg : typeArgs )
		{
			commaSpaceSepStrBuff.add(
					typeArg.toString() );
		}

		return "<" + commaSpaceSepStrBuff.toString() + ">";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isPrimitive()
	{
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isAssignableFrom(
			final AsmType typeToTest ,
			final AsmClasspathInfo asmClasspathInfo )
	{
		throw new RuntimeException( "nicht fertig" );
		//if ( ! ( typeToTest instanceof AsmMemberClassOfTypevar ) )
		//{
		//	return false;
		//}
		//
		//final AsmMemberClassOfTypevar classToTest = (AsmMemberClassOfTypevar) typeToTest;
		//
		////ClasspathClasses.ClassInfo classInfoToTest = asmClasspathInfo.getClassInfoMandantory( classToTest.fqn.toFqnStr() );
		//final ClasspathClasses.ClassInfo thisClassInfo = asmClasspathInfo.getClassInfoMandantory( this.fqn.toFqnStr() );
		//
		//if ( thisClassInfo.isAssignableFrom(
		//		classToTest.fqn.toFqnStr() ,
		//		asmClasspathInfo ) )
		//{
		//	return true;
		//}
		//
		//return false;
	}

	//public boolean isMemberOfTypevar()
	//{
	//	if ( outerOrEnclosingType instanceof AsmTypevar )
	//    {
	//        return true;
	//    }
	//
	//	if ( outerOrEnclosingType instanceof AsmMemberClassType &&
	//			( (AsmMemberClassType) outerOrEnclosingType ).isMemberOfTypevar() )
	//    {
	//        return true;
	//    }
	//
	//	return false;
	//}

	/**
	 * {@inheritDoc}
	 */
	public Fqn getEntireFqn()
	{
		final Fqn outerOrEnclosingTypeFqn = this.outerOrEnclosingType.getEntireFqn();

		return new Fqn(
				outerOrEnclosingTypeFqn.packageName ,
				DottedName.valueOfDotSeparatedStr(
						outerOrEnclosingTypeFqn.dottedName.toJavaName() +
						"." +
						//this.relativName.toJavaName()
						this.relativName.toString() ) );
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((this.outerOrEnclosingType == null) ? 0 : this.outerOrEnclosingType.hashCode());
		result = prime * result + ((this.relativName == null) ? 0 : this.relativName.hashCode());
		result = prime * result + ((this.typeArgs == null) ? 0 : this.typeArgs.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AsmInnerClassType other = (AsmInnerClassType) obj;
		if (this.outerOrEnclosingType == null) {
			if (other.outerOrEnclosingType != null)
				return false;
		} else if (!this.outerOrEnclosingType.equals(other.outerOrEnclosingType))
			return false;
		if (this.relativName == null) {
			if (other.relativName != null)
				return false;
		} else if (!this.relativName.equals(other.relativName))
			return false;
		if (this.typeArgs == null) {
			if (other.typeArgs != null)
				return false;
		} else if (!this.typeArgs.equals(other.typeArgs))
			return false;
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ImmutableMap<String, AsmType> getEntireTypeArgs(
			final AsmClasspathInfo asmClasspathInfo )
	{
		final ImmutableMap<String, AsmType> outerOrEnclosingTypeArgs = this.outerOrEnclosingType.getEntireTypeArgs( asmClasspathInfo );

		if ( this.typeArgs == null )
		{
			return outerOrEnclosingTypeArgs;
		}

		final AsmClassInfo thisClassInfo =
				asmClasspathInfo.getClassInfoMandantory(
						this.getEntireFqn() );

		final ImmutableArray<? extends AsmAbstractTypeParam> typeParams = thisClassInfo.typeParams;

		final Map<String, AsmType> result = new HashMap<>();

		if ( outerOrEnclosingTypeArgs != null )
		{
			for ( final Entry<String, ? extends AsmType> outerOrEnclosingTypeArgsEntry : outerOrEnclosingTypeArgs.entrySet() )
				// zuerst die auesseren Typ-Argumente vermerken
			{
				result.put(
						outerOrEnclosingTypeArgsEntry.getKey() ,
						outerOrEnclosingTypeArgsEntry.getValue() );
			}
		}

		for ( int index = 0 ; index < this.typeArgs.size() ; index++ )
		{
			final AsmType typeArg = this.typeArgs.get(index);

			// innere Typ-Argumente verbergen auessere Typ-Argumente mit gleichem Namen
			result.put(
					typeParams.get(index).name ,
					typeArg );
		}

		return ImmutableMap.create( result );

	}

	@Override
	public AsmInnerClassType resolveBound(
			ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap )
	{
		return new AsmInnerClassType(
				(AsmTopLevelOrInnerClassType) outerOrEnclosingType.resolveBound(
						typeargsNameToBoundMap ) ,
				relativName ,
				AsmTypeUtil.resolveTypeArgs(
						typeArgs ,
						typeargsNameToBoundMap ,
						new Stack<String>() ) );
	}

	@Override
	public AsmInnerClassType resolveBound(
			ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap ,
			final Stack<String> typevarNameStack )
	{
		return new AsmInnerClassType(
				(AsmTopLevelOrInnerClassType) outerOrEnclosingType.resolveBound(
						typeargsNameToBoundMap ) ,
				relativName ,
				AsmTypeUtil.resolveTypeArgs(
						typeArgs ,
						typeargsNameToBoundMap ,
						typevarNameStack ) );
	}

	@Override
	public AsmInnerClassType cloneAddTypeAnnotations(
			final Map<AsmTypePath , ImmutableArray<AsmAnnotationUse>> typeAnnotationsMap ,
			final AsmTypePath typePath )
	{
		if ( this.typeAnnotations != null )
			// zweimal Typ-Annotationen hinzufgen sollte nicht vorkommen
		{
			throw new IllegalStateException();
		}

		//System.out.println( new Exception().getStackTrace()[ 0 ] + " " + this + " type path to correct " + typePath );
		//new Exception().printStackTrace( System.out );

		final AsmTypePath thisTypePath =
				typePath.cloneAddInnerClasses(
						innerClassDepth() );

		return new AsmInnerClassType(
				typeAnnotationsMap.get(
						thisTypePath ) ,
				(AsmTopLevelOrInnerClassType) this.outerOrEnclosingType.cloneAddTypeAnnotations(
						typeAnnotationsMap ,
						typePath ) ,
				relativName ,
				AsmTypePath.typeArgsCloneAddTypeAnnotations(
						this.typeArgs ,
						typeAnnotationsMap ,
						thisTypePath ) );
		//throw new RuntimeException( "nicht fertig" );
	}

	public int innerClassDepth()
	{
		if ( this.outerOrEnclosingType instanceof AsmInnerClassType )
		{
			return ( (AsmInnerClassType) this.outerOrEnclosingType ).innerClassDepth() + 1;
		}
		return 1;
	}

}
