package de.heinerkuecker.asm_improved.type.primitive;

import java.util.Map;

import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.AsmTypePath;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * byte-Type
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmByte
extends AsmPrimitive
{
	/**
	 * Singleton-Instanz.
	 */
	public static final AsmByte INSTANCE = new AsmByte();

	/**
	 * Konstruktor.
	 */
	private AsmByte(
    		final ImmutableArray<AsmAnnotationUse> typeAnnotations )
	{
		super( typeAnnotations );
	}

	/**
	 * Konstruktor.
	 */
	private AsmByte()
	{
		super( null );
	}

	@Override
	String primitiveName()
	{
		return "byte";
	}

	/**
	 * {@inheritDoc}
	 */
    @Override
    public boolean isAssignableFrom(
    		final AsmType typeToTest ,
    		final AsmClasspathInfo asmClasspathInfo )
    {
    	/*
    	//TODO Sonderfall Zuweisung char-Wert, wenn Wert in Byte-Range
    	byte b = 'a';
    	b = Short.MAX_VALUE;
    	//TODO Sonderfall Zuweisung int/short-Wert, wenn Wert in Byte-Range
    	b = ( (int) Byte.MAX_VALUE );
    	b = ( (int) Byte.MIN_VALUE );
    	b = Byte.MAX_VALUE + 1;
    	b = Byte.MIN_VALUE - 1;
    	*/

    	return typeToTest instanceof AsmByte;
    }

	@Override
	public AsmByte cloneAddTypeAnnotations(
			final Map<AsmTypePath, ImmutableArray<AsmAnnotationUse>> typeAnnotationsMap ,
			final AsmTypePath typePath )
	{
		if ( this.typeAnnotations != null )
			// zweimal Typ-Annotationen hinzufgen sollte nicht vorkommen
		{
			throw new IllegalStateException();
		}

		return new AsmByte(
				typeAnnotationsMap.get( typePath ) );
		//throw new RuntimeException( "nicht fertig" );
	}

}
