package de.heinerkuecker.asm_improved.type.primitive;

import java.util.Map;

import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.AsmTypePath;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * int-Type
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmInt
extends AsmPrimitive
{
	/**
	 * Singleton-Instanz.
	 */
	public static final AsmInt INSTANCE = new AsmInt();

	/**
	 * Konstruktor.
	 */
	private AsmInt(
			final ImmutableArray<AsmAnnotationUse> typeAnnotations )
	{
		super( typeAnnotations );
	}

	/**
	 * Konstruktor.
	 */
	private AsmInt()
	{
		super( null );
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isAssignableFrom(
			final AsmType typeToTest ,
			final AsmClasspathInfo asmClasspathInfo )
	{
		return typeToTest instanceof AsmChar ||
				typeToTest instanceof AsmInt ||
				typeToTest instanceof AsmShort ||
				typeToTest instanceof AsmByte;
	}

	@Override
	public AsmInt cloneAddTypeAnnotations(
			final Map<AsmTypePath, ImmutableArray<AsmAnnotationUse>> typeAnnotationsMap ,
			final AsmTypePath typePath )
	{
		if ( this.typeAnnotations != null )
			// zweimal Typ-Annotationen hinzufgen sollte nicht vorkommen
		{
			throw new IllegalStateException();
		}

		return new AsmInt(
				typeAnnotationsMap.get( typePath ) );
		//throw new RuntimeException( "nicht fertig" );
	}

	@Override
	String primitiveName()
	{
		return "int";
	}

}
