package de.heinerkuecker.asm_improved.type.primitive;

import java.util.Stack;

import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;

/**
 * Gemeinsame Oberklasse aller primitiven Typen dieser Lib.
 * 
 * @author Heiner K&uuml;cker
 */
public abstract class AsmPrimitive
extends AsmType
{
	/**
	 * Konstruktor.
	 * 
	 * @param typeAnnotations
	 */
	protected AsmPrimitive(
			final ImmutableArray<AsmAnnotationUse> typeAnnotations )
	{
		super( typeAnnotations );
	}

	/**
	 * {@inheritDoc}
	 */
    @Override
    public final boolean isPrimitive()
    {
	    return true;
    }

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode()
	{
		return this.getClass().getSimpleName().hashCode();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(
			final Object obj)
	{
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		return true;
	}

	@Override
	public AsmPrimitive resolveBound(
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap )
	{
		return this;
	}

	@Override
	public AsmPrimitive resolveBound(
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap ,
			final Stack<String> typevarNameStack )
	{
		return this;
	}

	/**
	 * @return primitive name
	 */
	abstract String primitiveName();
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public final String toJavaString() {
    	return typeAnnotationsToStringWithDelimiterSpace() + primitiveName();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toJavaStringWithoutTypeAnnotations()
	{
    	return primitiveName();
	}

	/**
	 * {@inheritDoc}
	 */
    @Override
    public final String toString()
    {
		return toJavaString();
    }
}
