package de.heinerkuecker.asm_improved.type.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodParamInfo;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;

/**
 * TODO
 * 
 * @author Heiner K&uuml;cker
 */
public final class AsmTypeUtil
{
	/**
	 * Konstruktor private, Instanzieren verboten, Util-Klasse.
	 */
	private AsmTypeUtil()
	{
		super();
	}

	public static ImmutableMap<String, AsmType> putTypeargsInMap(
			final ImmutableArray<? extends AsmType> typeArgs ,
			final AsmClassInfo asmClass )
	{
		if ( typeArgs == null || typeArgs.isEmpty() )
		{
			return null;
		}

		if ( asmClass.typeParams == null || asmClass.typeParams.isEmpty() )
		{
			return null;
		}

		final Map<String, AsmType> result = new HashMap<>();

		for( int typeArgIndex = 0 ; typeArgIndex < typeArgs.size() ; typeArgIndex++ )
		{
			result.put(
					asmClass.typeParams.get( typeArgIndex ).name ,
					typeArgs.get( typeArgIndex ) );
		}
		
		return ImmutableMap.create( result );
	}

	public static ImmutableArray<AsmMethodParamInfo> resolveParamBounds(
			final ImmutableArray<? extends AsmMethodParamInfo> params ,
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap )
	{
		if ( params == null || params.isEmpty() )
		{
			return null;
		}

		final List<AsmMethodParamInfo> result = new ArrayList<>();

		for ( final AsmMethodParamInfo param : params )
		{
			result.add(
					param.resolveBounds(
							typeargsNameToBoundMap ) );
		}
		
		return ImmutableArray.create( result );
	}

	public static ImmutableArray<? extends AsmType> resolveTypeArgs(
			final ImmutableArray<? extends AsmType> typeArgs ,
			final ImmutableMap<String, ? extends AsmType> typeargsNameToBoundMap ,
			final Stack<String> typevarNameStack )
	{
		if ( typeArgs == null || typeArgs.isEmpty() )
		{
			return null;
		}

		final List<AsmType> result = new ArrayList<>();

		for ( final AsmType typeArg : typeArgs )
		{
			result.add(
					typeArg.resolveBound(
							typeargsNameToBoundMap ,
							typevarNameStack ) );
		}
		
		return ImmutableArray.create( result );
	}

}
