package de.heinerkuecker.asm_improved.util;

/**
 * {@link java.util.Set}-&auml;hnliche
 * abstrakte Klasse mit typsicherer
 * {@link java.util.Set#contains}-Methode.
 * 
 * @param <E> the type of elements maintained by this set
 * 
 * @author Heiner K&uuml;cker
 * 
 * TODO unter Java8 kann dies ein Interface mit nested class sein
 */
public abstract class ContainsTypesafeSet<E>
{
	/**
	 * @see java.util.Set#contains
	 * 
	 * @param o see {@link java.util.Set#contains}
	 * @return see {@link java.util.Set#contains}
	 */
    abstract public boolean contains(E o);

    /**
     * @see java.util.Set#add
     * 
     * @param e see {@link java.util.Set#add}
     * @return see {@link java.util.Set#add}
     */
    abstract public boolean add(E e);

    /**
	 * Implementierungsklasse fr das Interface
	 * {@link ContainsTypesafeSet}.
	 * 
	 * @param <E> the type of elements maintained by this set
	 */
	public static class Adaptor<E>
	extends ContainsTypesafeSet<E>
	{
		/**
		 * Type safed set.
		 */
		public final java.util.Set<E> innerSet;

		/**
		 * Konstruktor.
		 * 
		 * @param mapToWrap
		 */
        public Adaptor(
        		final java.util.Set<E> setToWrap )
        {
	        this.innerSet = setToWrap;
        }

		/**
		 * {@inheritDoc}
		 */
		@Override
		public boolean contains(
				final E o)
		{
			return this.innerSet.contains( o );
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public boolean add(
				final E e)
		{
			return this.innerSet.add( e );
		}

		/**
		 * {@inheritDoc}
		 */
        @Override
        public String toString()
        {
	        return "Adaptor [innerSet=" + innerSet + "]";
        }

	}

	/**
	 * Factory-Methode.
	 * 
	 * @param <E> the type of elements maintained by this set
	 * @param mapToWrap get type unsafe map
	 * @return Instanz
	 */
	public static <E> ContainsTypesafeSet<E> create(
			final java.util.Set<E> setToWrap )
	{
		return new Adaptor<>( setToWrap );
	}
}
