package de.heinerkuecker.asm_improved.util;


/**
 * Tupel aus {@link Fqn} und
 * {@link SourceFileNameWithoutPath}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class FqnAndSourceFileNameTuple
{
	/**
	 * Package in Punkt-Notation,
	 * Default-Package als Leerstring codiert.
	 */
	public final PackageName packageName;

	/**
	 * Bei inneren oder geschachtelten (nested)
	 * Klassen werden die Namens-Bestandteile
	 * im Quelltext mit Punkten abgetrennt,
	 * aber die Klassen-Datei hat einen
	 * mit Dollar-Zeichen abgetrennten
	 * Namen und so taucht der Name
	 * auch in ASM auf.
	 */
    // darf null sein beim Parsen von top-level-Klassen-Annotationen
	// oder als outer/enclosing class bei top-level-Klassen
	public final DottedName dottedName;

	/**
	 * Vollqualifizierter Name einer
	 * Klasse/Interface/Annotation/Enum.
	 */
    // darf null sein beim Parsen von top-level-Klassen-Annotationen
	// oder als outer/enclosing class bei top-level-Klassen
    public final Fqn fqn;

    /**
     * Dateiname der *.java-Datei ohne Pfad,
     * wie in den *.class-Dateien vermerkt
     * und von der ASM-Lib geliefert.
     */
    public final SourceFileNameWithoutPath sourceFileNameWithoutPath;

    /**
	 * Konstruktor.
	 */
    public FqnAndSourceFileNameTuple(
            //final Fqn fqn ,
    		final PackageName packageName ,
			final DottedName dottedName ,
			final SourceFileNameWithoutPath sourceFileNameWithoutPath )
    {
    	if ( packageName == null )
    	{
    		throw new IllegalArgumentException(
    				"packageName is null" );
    	}

    	// darf null sein beim Parsen von top-level-Klassen-Annotationen
    	//if ( dottedName == null )
    	//{
    	//    throw new IllegalArgumentException(
    	//			"dottedName is null" );
    	//}

    	if ( sourceFileNameWithoutPath == null )
    	{
    		throw new IllegalArgumentException(
    				"sourceFileNameWithoutPath is null" );
    	}

    	this.packageName = packageName;
    	this.dottedName = dottedName;

    	if ( dottedName == null )
    	{
    		this.fqn = null;
    	}
    	else
    	{
    		//new Exception().printStackTrace(System.out);
        	//System.out.println( dottedName.toClassFileName() );

    		this.fqn =
    				new Fqn(
    						packageName ,
    						dottedName );
    	}

    	this.sourceFileNameWithoutPath = sourceFileNameWithoutPath;
    }

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.getClass().getSimpleName() + 
				"[" +
				"packageName=" + packageName + ", " +
				"dottedName=" + dottedName + ", " +
				"fqn=" + fqn + ", " +
				"sourceFileNameWithoutPath=" + sourceFileNameWithoutPath +
				"]";
	}

    
}
