package de.heinerkuecker.asm_improved.util;

import java.util.Map;

/**
 * {@link java.util.Map}-hnliche
 * abstrakte Klasse mit typsicherer
 * {@link java.util.Map#get}-Methode
 * und
 * {@link java.util.Map#containsKey}-Methode.
 * 
 * @param <K> the type of keys maintained by this map
 * @param <V> the type of mapped values
 * 
 * @author Heiner K&uuml;cker
 * 
 * TODO unter Java8 kann dies ein Interface mit nested class sein
 */
public abstract class GetTypesafeMap<K, V>
{
	/**
	 * @see java.util.Map#put
	 * @param key see {@link java.util.Map#put}
	 * @param value see {@link java.util.Map#put}
	 * @return see {@link java.util.Map#put}
	 */
	abstract public V put(K key, V value);

	/**
	 * @see java.util.Map#get
	 * @param key see {@link java.util.Map#get}
	 * @return see {@link java.util.Map#get}
	 */
	abstract public V get(K key);

	/**
	 * @see java.util.Map#containsKey
	 * @param key see {@link java.util.Map#containsKey}
	 * @return see {@link java.util.Map#containsKey}
	 */
	abstract public boolean containsKey(K key);

	/**
	 * @see java.util.Map#values()
	 * @return see {@link java.util.Map#values()k}
	 */
    abstract public Iterable<? extends V> values();

    /**
	 * Implementierungsklasse fr das Interface
	 * {@link GetTypesafeMap}.
	 * 
	 * @param <K> the type of keys maintained by this map
	 * @param <V> the type of mapped values
	 */
	public static class Adaptor<K, V>
	extends GetTypesafeMap<K, V>
	{
		/**
		 * Type safed map.
		 */
		public final java.util.Map<K, V> innerMap;

		/**
		 * Konstruktor.
		 * 
		 * @param mapToWrap
		 */
        public Adaptor(
        		final Map<K, V> mapToWrap )
        {
	        this.innerMap = mapToWrap;
        }

		/**
		 * {@inheritDoc}
		 */
        public V get(
        		final K key )
        {
	        return innerMap.get(key);
        }

		/**
		 * {@inheritDoc}
		 */
        public V put(
        		final K key ,
        		final V value )
        {
	        return innerMap.put(key, value);
        }

		/**
		 * {@inheritDoc}
		 */
        public boolean containsKey(
        		final K key)
        {
	        return innerMap.containsKey(key);
        }

		/**
		 * {@inheritDoc}
		 */
        @Override
        public String toString()
        {
	        return "Adaptor [innerMap=" + innerMap + "]";
        }

		/**
		 * {@inheritDoc}
		 */
		@Override
		public Iterable<? extends V> values()
		{
			return this.innerMap.values();
		}

	}

	/**
	 * Factory-Methode.
	 * 
	 * @param <K> the type of keys maintained by this map
	 * @param <V> the type of mapped values
	 * @param mapToWrap get type unsafe map
	 * @return Instanz
	 */
	public static <K, V> GetTypesafeMap<K, V> create(
			final java.util.Map<K, V> mapToWrap )
	{
		return new Adaptor<>( mapToWrap );
	}
}
