package de.heinerkuecker.asm_improved.util;


/**
 * String mit Java-Identifier,
 * typsicher.
 * 
 * @author Heiner K&uuml;cker
 */
public final class JavaIdentifier
{
    /**
     * Java-Identifier-String
     */
    public final String str;

    /**
     * Konstruktor.
     * 
     * @param str Java-Identifier-String
     */
    public JavaIdentifier(
            final String str )
    {
        check( str );

        this.str = str;
    }

    /**
     * 
     * @param str
     */
    public static void check(
            final String str )
    {
        if ( str.length() < 1 )
        {
            throw new IllegalArgumentException(
                    "java identifier is empty" );
        }

        if ( false == Character.isJavaIdentifierStart(
                str.charAt( 0 ) ) )
        {
            throw new IllegalArgumentException(
                    "wrong java identifier: " +
                            str );
        }

        for ( int pos = 1 ; pos < str.length() ; pos++ )
        {
            if ( false == Character.isJavaIdentifierPart(
                    str.charAt( pos ) ) )
            {
                throw new IllegalArgumentException(
                        "wrong java identifier: " +
                                str );
            }
        }
    }

    /**
     * 
     * @param str
     */
    public static boolean is(
            final String str )
    {
        if ( str == null ||
        		str.isEmpty() )
        {
            return false;
        }

        if ( false == Character.isJavaIdentifierStart(
                str.charAt( 0 ) ) )
        {
            return false;
        }

        for ( int pos = 1 ; pos < str.length() ; pos++ )
        {
            if ( false == Character.isJavaIdentifierPart(
                    str.charAt( pos ) ) )
            {
                return false;
            }
        }
        return true;
    }

    /**
     * TODO diese Methode passt nicht so richtig in diese Klasse
     * 
     * @param str
     */
    public static boolean isJavaIdentifierOrDotSeparatedJavaIdentifier(
            final String str )
    {
        if ( str == null ||
        		str.isEmpty() )
        {
            return false;
        }

        if ( false == Character.isJavaIdentifierStart(
                str.charAt( 0 ) ) )
        {
            return false;
        }

        for ( int pos = 1 ; pos < str.length() ; pos++ )
        {
        	if ( str.charAt( pos ) == '.' )
        	{
        		return isJavaIdentifierOrDotSeparatedJavaIdentifier(
        				str.substring(
        						//beginIndex
        						pos + 1 ) );
        	}
        	else if ( false == Character.isJavaIdentifierPart(
                    str.charAt( pos ) ) )
            {
                return false;
            }
        }
        return true;
    }

    /**
     */
    @Override
    public String toString()
    {
        return this.str;
    }

    /**
     */
    @Override
    public int hashCode()
    {
        final int prime = 31;
        int result = 1;
        result = prime * result + ( ( this.str == null ) ? 0 : this.str.hashCode() );
        return result;
    }

    /**
     */
    @Override
    public boolean equals(
            final Object obj )
    {
        if ( this == obj )
        {
            return true;
        }
        if ( obj == null )
        {
            return false;
        }
        if ( getClass() != obj.getClass() )
        {
            return false;
        }
        final JavaIdentifier other = (JavaIdentifier) obj;
        // TODO Member str darf nicht null sein
        //if ( this.str == null )
        //{
        //    if ( other.str != null )
        //    {
        //        return false;
        //    }
        //}
        //else
        if ( ! this.str.equals( other.str ) )
        {
            return false;
        }
        return true;
    }

}
