package de.heinerkuecker.asm_improved.util;

import java.util.Comparator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

/**
 * Spezielle {@link TreeMap},
 * welche absichert, dass
 * nach einem Aufruf einer
 * {@link #get}-Methode
 * niemals eine {@link #put}-Methode
 * aufgerufen wird, die
 * Map also erst nach vollstndigem Befllen
 * genutzt werden kann.
 * 
 * @author Heiner K&uuml;cker
 */
public class NoSetAfterGetTreeMap<K, V>
extends TreeMap<K, V>
{

	/**
	 * 
	 */
    public NoSetAfterGetTreeMap()
    {
	    super();
	    // TODO Auto-generated constructor stub
    }

	/**
	 * @param comparator
	 */
    public NoSetAfterGetTreeMap(Comparator<? super K> comparator)
    {
	    super(comparator);
	    // TODO Auto-generated constructor stub
    }

	/**
	 * @param m
	 */
    public NoSetAfterGetTreeMap(Map<? extends K, ? extends V> m)
    {
	    super(m);
	    // TODO Auto-generated constructor stub
    }

	/**
	 * @param m
	 */
    public NoSetAfterGetTreeMap(SortedMap<K, ? extends V> m)
    {
	    super(m);
	    // TODO Auto-generated constructor stub
    }

    private boolean isGetCalled;
    
	/* (non-Javadoc)
	 * @see java.util.TreeMap#get(java.lang.Object)
	 */
    @Override
    public V get(Object key)
    {
	    isGetCalled = true;
	    return super.get(key);
    }

	/* (non-Javadoc)
	 * @see java.util.TreeMap#put(java.lang.Object, java.lang.Object)
	 */
    @Override
    public V put(K key, V value)
    {
    	if ( isGetCalled )
        {
	        throw new IllegalStateException( "get already called" );
        }

    	return super.put(key, value);
    }

    

}
