package de.heinerkuecker.asm_improved.util;

import java.util.jar.JarFile;

/**
 * Typsicherer Package-Name.
 * 
 * @author Heiner K&uuml;cker
 */
public final class PackageName
{
	/**
	 * Leerer Package-Name.
	 */
	public static final PackageName DEFAULT_PACKAGE_NAME =
			new PackageName(
					"" );
	
	/**
	 * Package-Name 'java.lang'.
	 */
	public static final PackageName JAVA_LANG_PACKAGE_NAME =
			new PackageName(
					"java.lang" );
	
	/**
	 * Package in Punkt-Notation,
	 * Default-Package als Leerstring codiert.
	 */
	public final String value;

	/**
	 * Konstruktor.
	 * 
	 * @param value
	 */
    public PackageName(
    		final String value )
    {
    	if ( value == null )
    	{
    		throw new IllegalArgumentException(
    				"value is null" );
    	}
    	
    	// TODO Format-Prfung
	    this.value = value;
    }

	/**
	 * Konstruktor.
	 * 
	 * @param packageObj
	 */
    public PackageName(
    		final Package packageObj )
    {
	    this( packageObj.getName() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode()
    {
    	//final int prime = 31;
    	//int result = 1;
    	//result = prime * result + (/*(value == null) ? 0 :*/ value.hashCode());
    	//return result;
	    return value.hashCode();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(
    		final Object obj )
    {
	    if (this == obj)
		    return true;
	    if (obj == null)
		    return false;
	    if (getClass() != obj.getClass())
		    return false;
	    final PackageName other = (PackageName) obj;
	    /*if (value == null)
	    {
		    if (other.value != null)
			    return false;
	    } else*/ if (!value.equals(other.value))
		    return false;
	    return true;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString()
    {
	    return value;
    }

	/**
	 * @return is empty
	 */
    public boolean isEmpty()
    {
	    return /*null nicht erlaubt: this.value == null ||*/ this.value.isEmpty();
    }

	/**
	 * @return Name durch Schrgstriche abgetrennt, wie von {@link JarFile#getEntry}
	 *         oder im Dateisystem(Classpath) erwartet
	 */
    public String slashed()
    {
	    return this.value.replace( '.' , '/' );
    }

}
