package de.heinerkuecker.asm_improved.util;

import java.util.Collection;
import java.util.Map;

/**
 * Map-Adapter zum Hinzuf&uuml;gen von
 * Werten zu {@link Map}s, die
 * wiederum untergeordnete
 * {@link Collection} enthalten,
 * welche beim erstmaligen
 * Hinzuf&uuml;gen erzeugt/initialisiert
 * werden sollen.
 * 
 * @author Heiner K&uuml;cker
 */
public abstract class PutAddCollMap<K, S extends Collection<V>, V>
{
	/**
	 * Adadptierte Map.
	 */
	public final Map<K, S> map;

	/**
	 * Konstruktor.
	 * 
	 * @param map
	 */
	protected PutAddCollMap(
			final Map<K, S> map )
	{
		if ( map == null )
		{
			throw new IllegalArgumentException( "map is null" );
		}

		this.map = map;
	}

	/**
	 * Methode zum zum Hinzuf&uuml;gen von Werten untergeordnete {@link Collection}.
	 * 
	 * @param key Schl&uuml;ssel
	 * @param value Wert
	 */
	public final void putSub(
			final K key, 
			final V value )
	{
		S subColl = map.get( key );

		if ( subColl == null )
		{
			subColl = createSub();
			this.map.put(key, subColl);
		}

		subColl.add( value );
	}

	/**
	 * Zu implementierende Methode zum Erzeugen der untergeordneten {@link Collection}
	 * 
	 * @return erzeugte untergeordnete {@link Collection}
	 */
	protected abstract S createSub();

}
