package de.heinerkuecker.asm_improved;

import java.io.File;

import org.junit.runner.RunWith;
import org.junit.runners.Suite;
import org.junit.runners.Suite.SuiteClasses;

import de.heinerkuecker.asm_improved.test.annotation_def.AllAsmAnnotationDefTests;
import de.heinerkuecker.asm_improved.test.annotation_use.AllAsmAnnotationUseTests;
import de.heinerkuecker.asm_improved.test.inner_class.with_typeparams.field.AllInnerClassTypeparamFieldUsingTests;
import de.heinerkuecker.asm_improved.test.method.exception.MethodExceptionAutocloseableTest;
import de.heinerkuecker.asm_improved.test.method.exception.MethodExceptionCloseableTest;
import de.heinerkuecker.asm_improved.test.type.type_argument.TypeArgumentIntArrayTest;
import de.heinerkuecker.asm_improved.type_param.ClassWithTypeparamTestSuite;
import de.heinerkuecker.asm_improved.type_param.method.static_.exception.StaticMethodWithTypeparamsExceptionTest;
import de.heinerkuecker.asm_improved.type_params.inherit_reinforced_bound.ClassesWithTypeparamsInheritReinforcedBoundedTest;

/**
 * JUnit4-Testsuite fr die ASM-Improved-Lib.
 * 
 * @author Heiner K&uuml;cker
 */
@RunWith(Suite.class)
@SuiteClasses(
{
	//MavenProjectAsmClasspathInfoModeEagerTest.class ,
	TestsuiteUniqueSubTest.class ,
	AllTestsInTestsuiteTest.class ,
	AsmClasspathInfoModeEagerTest.class ,
	AsmClasspathInfoRescanClassWithDollarSepNameTest.class ,
	AllJavaHomeJarsModeEagerTest.class ,
	ClassInfoTest.class ,
	FieldInfoTest.class ,
	MethodInfoTest.class ,
	MultiNestedTest.class ,
	ClassInfoIsAssignableFromTest.class,
	ClassInfoIsSuperClassOrImplementedInterfaceTest.class ,
	AsmDescriptionAndSignatureParserGetSemicolonPosUnderConsiderationGenericsMethodTest.class ,
	AsmDescriptionAndSignatureParserParseMethodTest.class ,
	ClassWithTypeparamTestSuite.class ,
	// TODO: NestedClassOfTypevarWithTypeparamFieldUsingTestSuite.class ,
	ConstructorWithGenericException1Test.class ,
	ConstructorWithGenericException2Test.class ,
	MethodExceptionAutocloseableTest.class ,
	MethodExceptionCloseableTest.class ,
	MethodWithGenericException1Test.class ,
	MethodWithGenericException2Test.class ,
	MethodWithNotGenericNestedExceptionTest.class ,
	MethodWithNotGenericException1Test.class ,
	MethodWithNotGenericException2Test.class ,
	// TODO Kombination generische und nicht-generische Exception
	MethodWithTypeParamAsTypevarOfClassTest.class ,
	MethodElementHandlerTest.class ,
	AllAsmAnnotationDefTests.class ,
	AllAsmAnnotationUseTests.class ,
	// TODO: InnerClassOfTypevarUsingTest.class ,
	// TODO: AllInnerClassOfTypevarWithTypeparamFieldUsingTests.class ,
	// TODO: NestedInterfaceOfTypevarWithTypeparamFieldUsing2Test.class ,
	// TODO: NestedInterfaceOfTypevarWithTypeparamFieldUsing3Test.class ,
	// TODO: InnerClassOfTypevarWithTypeparamOverIndirectionUsingTest.class ,
	// TODO: InnerClassOfTypevarWithTypeparamOverIndirectionUsing2Test.class ,
	// TODO: NestedClassOfTypevarUsingTest.class ,
	// TODO: TypevarAsExceptionTypeTest.class ,
	// TODO: TypevarAsExceptionType2Test.class ,
	// TODO: TypevarSecondLevelUsingTest.class ,
	
	StaticMethodWithTypeparamsExceptionTest.class ,

	AllInnerClassTypeparamFieldUsingTests.class ,
	XmlSchemaGeneratorNamespaceTest.class ,
	TypeArgumentIntArrayTest.class ,
	ClassesWithTypeparamsInheritReinforcedBoundedTest.class ,
	AsmInnerClassTypeCorrectorTest.class ,
	ClassInfoAWTUtilitiesAnonym1Test.class ,
	AsmClassInfoForAnonymusClassTest_1.class

}
)
public class AllAsmTests
{
	/**
	 * Name des System-Jar mit Endung.
	 * <br/>
	 * Es gibt noch einige weitere
	 * zum JDK gehrende Jar-Dateien,
	 * diese werde hier zur Zeit
	 * ignoriert.
	 */
	private static final String RT_JAR_FILE_NAME = "rt.jar";

	/**
	 * Pfad zum Zugriff auf rt.jar
	 * und eventuelle andere
	 * System-Jars.
	 */
	private static final String JRE_LIB_PATH_STR =
			//"C:\\Programme\\Java\\jre1.8.0_40\\lib"
			getJreLibPathStr();

	/**
	 * @return Pfad zu den Sytem-Libraries des JRE
	 */
	private static String getJreLibPathStr()
	{
		//System.out.println( System.getProperty( "java.home" ) );
		//System.out.println( System.getProperty( "java.class.path" ) );
		//System.out.println( System.getProperty( "java.library.path" ) );
		return
				getJavaHomePathStr() +
				File.separator +
				"lib";
	}

	/**
	 * @return JAVA_HOME directory as string
	 */
	public static String getJavaHomePathStr()
	{
		return System.getProperty( "java.home" );
	}

	/**
	 * File-Objekt der Datei rt.jar
	 */
	public static final File RT_JAR_FILE =
		new File(
				JRE_LIB_PATH_STR +
				File.separator +
				RT_JAR_FILE_NAME );

	/**
     * Pfad zum Verzeichnis des Eclipse-Projekts als String.
     */
    private static final String ECLIPSE_PROJECT_DIR_PATH_STR =
    		System.getProperty(
	        "user.dir" );

	/**
     * Pfad zum Verzeichnis <Eclipse-Projekt>/bin,
     * in welchem sich die kompilierten
     * *.class-Dateien der zu parsenden Java-Dateien
     * befinden, als String.
     */
    public static final String BIN_DIR_PATH_STR =
            ECLIPSE_PROJECT_DIR_PATH_STR +
            File.separator +
            "bin";

	/**
     * Pfad zum Verzeichnis <Eclipse-Projekt>/bin,
     * in welchem sich die kompilierten
     * *.class-Dateien der zu parsenden Java-Dateien
     * befinden, als File-Objekt.
     */
	public static final File BIN_DIR_FILE =
		new File(
				BIN_DIR_PATH_STR );

	/**
	 * ASM-Classpath-Info fr Verwendung
	 * in mehreren Tests.
	 */
	public static final AsmClasspathInfo asmClasspathInfo =
			new AsmClasspathInfo(
					AsmClasspathInfo.Mode.LAZY ,
					AllAsmTests.RT_JAR_FILE ,
					AllAsmTests.BIN_DIR_FILE ,
					// JUnit-Jars unter Eclipse-Installation
					new File( JUnitEclipseFileLocations.JUNIT_JAR_PATH_AND_FILENAME_STR ) ,
					new File( JUnitEclipseFileLocations.HAMCREST_JAR_PATH_AND_FILENAME_STR ) );

}
