package de.heinerkuecker.asm_improved;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link AsmClassInfo}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ClassInfoAWTUtilitiesAnonym1Test
{
	/**
	 * Test method for {@link AsmClassInfo}.
	 */
	@Test
	public final void test()
	{
		final AsmClasspathInfo asmClasspathInfo =
				new AsmClasspathInfo(
						AsmClasspathInfo.Mode.LAZY ,
						AllAsmTests.RT_JAR_FILE );
		
		final AsmClassInfo classInfo =
				asmClasspathInfo.getClassInfoMandantory(
						"com.sun.awt.AWTUtilities$1" );

		Assert.assertFalse(
				classInfo.isTopLevel() );

		Assert.assertTrue(
				classInfo.isFinal );

		Assert.assertFalse(
				classInfo.isAbstract );

		Assert.assertTrue(
				classInfo.isStatic );

		Assert.assertFalse(
				classInfo.isInterface );

		Assert.assertFalse(
				classInfo.isEnum );

		Assert.assertFalse(
				classInfo.isAnnotation );

		// --- super class ---

		Assert.assertEquals(
				//expected
				//"java.lang.Object"
				Fqn.JAVA_LANG_OBJECT_FQN ,
				//actual
				classInfo.superType.fqn );

		// --- interfaces ---

		Assert.assertEquals(
				//expected
				3 ,
				//actual
				classInfo.interfaceTypes.size() );

		{
			final AsmClassType interfaceType0 = classInfo.interfaceTypes.get( 0 );

			Assert.assertEquals(
					//expected
					new Fqn( java.io.Serializable.class ) ,
					//actual
					interfaceType0.fqn );

			Assert.assertTrue(
					interfaceType0.typeArgs == null || interfaceType0.typeArgs.isEmpty() );
		}

		{
			final AsmClassType interfaceType1 = classInfo.interfaceTypes.get( 1 );

			Assert.assertEquals(
					//expected
					new Fqn( java.lang.Comparable.class ) ,
					//actual
					interfaceType1.fqn );

			// --- interface type argument ---

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					interfaceType1.typeArgs.size() );

			{
				final AsmType interfacetype1Typearg0 = interfaceType1.typeArgs.get( 0 );

				Assert.assertTrue(
						interfacetype1Typearg0 instanceof AsmClassType );

				{
					final AsmClassType interfacetype1Typearg0ClassType = (AsmClassType) interfacetype1Typearg0;

					Assert.assertEquals(
							//expected
							Fqn.JAVA_LANG_STRING_FQN ,
							//actual
							interfacetype1Typearg0ClassType.fqn );
				}
			}
		}

		{
			final AsmClassType interfaceType2 = classInfo.interfaceTypes.get( 2 );

			Assert.assertEquals(
					//expected
					new Fqn( java.lang.CharSequence.class ) ,
					//actual
					interfaceType2.fqn );

			Assert.assertTrue(
					interfaceType2.typeArgs == null || interfaceType2.typeArgs.isEmpty() );
		}
		
		org.junit.Assert.fail( "not ready" );
	}

}
