package de.heinerkuecker.asm_improved;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmUnboundedTypeParam;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.DottedName;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.PackageName;

/**
 * JUnit4-Testcase fr die Klasse {@link AsmClassInfo}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ClassInfoTest
{
	/**
	 * Test method for {@link AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoJavaLangString()
	{
		final AsmClassInfo stringClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						java.lang.String.class.getCanonicalName() );

		Assert.assertTrue(
				stringClassInfo.isTopLevel() );

		Assert.assertTrue(
				stringClassInfo.isFinal );

		Assert.assertFalse(
				stringClassInfo.isAbstract );

		Assert.assertTrue(
				stringClassInfo.isStatic );

		Assert.assertFalse(
				stringClassInfo.isInterface );

		Assert.assertFalse(
				stringClassInfo.isEnum );

		Assert.assertFalse(
				stringClassInfo.isAnnotation );

		// --- super class ---

		Assert.assertEquals(
				//expected
				//"java.lang.Object"
				Fqn.JAVA_LANG_OBJECT_FQN ,
				//actual
				stringClassInfo.superType.fqn );

		// --- interfaces ---

		Assert.assertEquals(
				//expected
				3 ,
				//actual
				stringClassInfo.interfaceTypes.size() );

		{
			final AsmClassType interfaceType0 = stringClassInfo.interfaceTypes.get( 0 );

			Assert.assertEquals(
					//expected
					new Fqn( java.io.Serializable.class ) ,
					//actual
					interfaceType0.fqn );

			Assert.assertTrue(
					interfaceType0.typeArgs == null || interfaceType0.typeArgs.isEmpty() );
		}

		{
			final AsmClassType interfaceType1 = stringClassInfo.interfaceTypes.get( 1 );

			Assert.assertEquals(
					//expected
					new Fqn( java.lang.Comparable.class ) ,
					//actual
					interfaceType1.fqn );

			// --- interface type argument ---

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					interfaceType1.typeArgs.size() );

			{
				final AsmType interfacetype1Typearg0 = interfaceType1.typeArgs.get( 0 );

				Assert.assertTrue(
						interfacetype1Typearg0 instanceof AsmClassType );

				{
					final AsmClassType interfacetype1Typearg0ClassType = (AsmClassType) interfacetype1Typearg0;

					Assert.assertEquals(
							//expected
							Fqn.JAVA_LANG_STRING_FQN ,
							//actual
							interfacetype1Typearg0ClassType.fqn );
				}
			}
		}

		{
			final AsmClassType interfaceType2 = stringClassInfo.interfaceTypes.get( 2 );

			Assert.assertEquals(
					//expected
					new Fqn( java.lang.CharSequence.class ) ,
					//actual
					interfaceType2.fqn );

			Assert.assertTrue(
					interfaceType2.typeArgs == null || interfaceType2.typeArgs.isEmpty() );
		}
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoJavaUtilArrayList()
	{
		final AsmClassInfo arrayListClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						java.util.ArrayList.class.getCanonicalName() );

		Assert.assertTrue(
				arrayListClassInfo.isTopLevel() );

		Assert.assertFalse(
				arrayListClassInfo.isFinal );

		Assert.assertTrue(
				arrayListClassInfo.isStatic );

		Assert.assertFalse(
				arrayListClassInfo.isAbstract );

		Assert.assertFalse(
				arrayListClassInfo.isInterface );

		Assert.assertFalse(
				arrayListClassInfo.isEnum );

		Assert.assertFalse(
				arrayListClassInfo.isAnnotation );

		// --- super class ---

		Assert.assertEquals(
				//expected
				//"java.util.AbstractList"
				new Fqn( java.util.AbstractList.class ) ,
				//actual
				arrayListClassInfo.superType.fqn );

		// --- super class type argument ---

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				arrayListClassInfo.superType.typeArgs.size() );

		{
			final AsmType supertypeTypearg0 = arrayListClassInfo.superType.typeArgs.get( 0 );

			Assert.assertTrue(
					supertypeTypearg0 instanceof AsmTypevar );

			{
				final AsmTypevar supertypeTypearg0Typevar = (AsmTypevar) supertypeTypearg0;

				Assert.assertEquals(
						//expected
						"E" ,
						//actual
						supertypeTypearg0Typevar.name );
			}
		}

		// --- type params ---

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				arrayListClassInfo.typeParams.size() );

		{
			final AsmAbstractTypeParam typeParam = arrayListClassInfo.typeParams.get( 0 );

			Assert.assertTrue(
					typeParam instanceof AsmUnboundedTypeParam );

			{
				final AsmUnboundedTypeParam unboundedTypeParam = (AsmUnboundedTypeParam) typeParam;

				Assert.assertEquals(
						//expected
						"E" ,
						//actual
						unboundedTypeParam.name );
			}
		}

		// --- interfaces ---

		Assert.assertEquals(
				//expected
				4 ,
				//actual
				arrayListClassInfo.interfaceTypes.size() );

		{
			final AsmClassType interfaceType0 = arrayListClassInfo.interfaceTypes.get( 0 );

			Assert.assertEquals(
					//expected
					new Fqn( java.util.List.class ) ,
					//actual
					interfaceType0.fqn );

			// --- interface type argument ---

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					interfaceType0.typeArgs.size() );

			{
				final AsmType interfacetype0Typearg0 = interfaceType0.typeArgs.get( 0 );

				Assert.assertTrue(
						interfacetype0Typearg0 instanceof AsmTypevar );

				{
					final AsmTypevar interfacetype0TypeargTypevar0 = (AsmTypevar) interfacetype0Typearg0;

					Assert.assertEquals(
							//expected
							"E" ,
							//actual
							interfacetype0TypeargTypevar0.name );
				}
			}
		}

		{
			final AsmClassType interfaceType1 = arrayListClassInfo.interfaceTypes.get( 1 );

			Assert.assertEquals(
					//expected
					new Fqn( java.util.RandomAccess.class ) ,
					//actual
					interfaceType1.fqn );

			Assert.assertTrue(
					interfaceType1.typeArgs == null || interfaceType1.typeArgs.isEmpty() );
		}

		{
			final AsmClassType interfaceType2 = arrayListClassInfo.interfaceTypes.get( 2 );

			Assert.assertEquals(
					//expected
					new Fqn( java.lang.Cloneable.class ) ,
					//actual
					interfaceType2.fqn );

			Assert.assertTrue(
					interfaceType2.typeArgs == null || interfaceType2.typeArgs.isEmpty() );
		}

		{
			final AsmClassType interfaceType3 = arrayListClassInfo.interfaceTypes.get( 3 );

			Assert.assertEquals(
					//expected
					new Fqn( java.io.Serializable.class ) ,
					//actual
					interfaceType3.fqn );

			Assert.assertTrue(
					interfaceType3.typeArgs == null || interfaceType3.typeArgs.isEmpty() );
		}
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoJavaUtilMapEntry()
	{
		final AsmClassInfo entryClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						java.util.Map.Entry.class.getCanonicalName() );

		Assert.assertFalse(
				entryClassInfo.isTopLevel() );

		Assert.assertFalse(
				entryClassInfo.isFinal );

		Assert.assertTrue(
				entryClassInfo.isStatic );

		Assert.assertTrue(
				entryClassInfo.isAbstract );

		Assert.assertTrue(
				entryClassInfo.isInterface );

		Assert.assertFalse(
				entryClassInfo.isEnum );

		Assert.assertFalse(
				entryClassInfo.isAnnotation );

		Assert.assertNull(
				entryClassInfo.superType );

		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoJavaUtilHashMapNode()
	{
		final AsmClassInfo nodeClassInfo;
		if ( System.getProperty("java.version").startsWith("1.8"))
			// Java8
		{
			nodeClassInfo =
					AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
							"java.util.HashMap.Node"
							);
		}
		else
			// Java7
		{
			nodeClassInfo =
					AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
							"java.util.HashMap.Entry"
							);
		}

		Assert.assertFalse(
				nodeClassInfo.isTopLevel() );

		Assert.assertFalse(
				nodeClassInfo.isFinal );

		Assert.assertTrue(
				nodeClassInfo.isStatic );

		Assert.assertFalse(
				nodeClassInfo.isAbstract );

		Assert.assertFalse(
				nodeClassInfo.isInterface );

		Assert.assertFalse(
				nodeClassInfo.isEnum );

		Assert.assertFalse(
				nodeClassInfo.isAnnotation );

		Assert.assertEquals(
				//expected
				Fqn.JAVA_LANG_OBJECT_FQN ,
				//actual
				nodeClassInfo.superType.fqn );

		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoJavaUtilLinkedHashMapEntry()
	{
		final AsmClassInfo entryClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						"java.util.LinkedHashMap.Entry" );

		Assert.assertFalse(
				entryClassInfo.isTopLevel() );

		Assert.assertFalse(
				entryClassInfo.isFinal );

		Assert.assertTrue(
				entryClassInfo.isStatic );

		Assert.assertFalse(
				entryClassInfo.isAbstract );

		Assert.assertFalse(
				entryClassInfo.isInterface );

		Assert.assertFalse(
				entryClassInfo.isEnum );

		Assert.assertFalse(
				entryClassInfo.isAnnotation );

		if ( System.getProperty("java.version").startsWith("1.8"))
			// Java8
		{
			Assert.assertEquals(
					//expected
					//Java8: "java.util.HashMap.Node"
					//Java7: "java.util.HashMap.Entry"
					new Fqn(
							new PackageName( "java.util" ) ,
							DottedName.valueOfDotSeparatedStr( "HashMap.Node" ) ) ,
					//actual
					entryClassInfo.superType.fqn );
		}
		else
			// Java7
        {
	        Assert.assertEquals(
	        		//expected
	        		//Java8: "java.util.HashMap.Node"
	        		//Java7: "java.util.HashMap.Entry"
	        		new Fqn(
	        				new PackageName( "java.util" ) ,
	        				DottedName.valueOfDotSeparatedStr( "HashMap.Entry" ) ) ,
	        		//actual
	        		entryClassInfo.superType.fqn );
        }

		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoJavaxSwingTextHtmlFormSubmitEvent()
	{
		final AsmClassInfo enumClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						javax.swing.text.html.FormSubmitEvent.MethodType.class.getCanonicalName() );

		Assert.assertFalse(
				enumClassInfo.isTopLevel() );

		Assert.assertTrue(
				enumClassInfo.isFinal );

		Assert.assertTrue(
				enumClassInfo.isStatic );

		Assert.assertFalse(
				enumClassInfo.isAbstract );

		Assert.assertFalse(
				enumClassInfo.isInterface );

		Assert.assertTrue(
				enumClassInfo.isEnum );

		Assert.assertFalse(
				enumClassInfo.isAnnotation );

		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoJavaLangSuppressWarnings()
	{
		final AsmClassInfo annotationClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						java.lang.SuppressWarnings.class.getCanonicalName() );

		Assert.assertTrue(
				annotationClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotationClassInfo.isFinal );

		Assert.assertTrue(
				annotationClassInfo.isStatic );

		Assert.assertTrue(
				annotationClassInfo.isAbstract );

		Assert.assertTrue(
				annotationClassInfo.isInterface );

		Assert.assertFalse(
				annotationClassInfo.isEnum );

		Assert.assertTrue(
				annotationClassInfo.isAnnotation );

		//Assert.fail("Not yet implemented");
	}

}
