package de.heinerkuecker.asm_improved;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.test_data.ConstructorWithGenericException1;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr {@link ConstructorWithGenericException1}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ConstructorWithGenericException1Test
{

    /**
     * Test method for {@link ConstructorWithGenericException1#ConstructorWithGenericException1()}.
     */
    @Test
    public void testConstructor()
    {
        final AsmClassInfo constructorWithGenericException1ClassInfo =
                AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
                		ConstructorWithGenericException1.class.getCanonicalName() );

        final ImmutableArray<? extends AsmMethodInfo> constructorInfos =
                constructorWithGenericException1ClassInfo.methods.get(
                		AsmMethodInfo.CONSTRUCTOR_METHOD_NAME );

        Assert.assertEquals( 1 , constructorInfos.size() );

        final AsmMethodInfo constructorInfo = constructorInfos.get( 0 );

        {
            Assert.assertEquals(
                    1 ,
                    constructorInfo.typeParams.size() );

            final AsmAbstractTypeParam typeParam = constructorInfo.typeParams.get( 0 );

            Assert.assertEquals(
                    //expected
                    "T" ,
                    //actual
                    typeParam.name );

            Assert.assertTrue(
                    typeParam instanceof AsmExtendsTypeParam );

            AsmExtendsTypeParam extendsTypeParam = (AsmExtendsTypeParam) typeParam;

            final AsmType extendsTypeParamBoundType = extendsTypeParam.boundType;

            Assert.assertTrue(
                    extendsTypeParamBoundType instanceof AsmClassType );

            final AsmClassType extendsTypeParamBoundClassType = (AsmClassType) extendsTypeParamBoundType;

            Assert.assertEquals(
                    //expected
                    new Fqn( Throwable.class ) ,
                    //actual
                    extendsTypeParamBoundClassType.fqn );

            Assert.assertTrue(
                    extendsTypeParamBoundClassType.typeArgs == null || extendsTypeParamBoundClassType.typeArgs.isEmpty() );
        }

        {
            Assert.assertEquals(
                    1 ,
                    constructorInfo.exceptions.size() );

            //System.out.println( constructorInfo.exceptions.get( 0 ) );

            final AsmType exceptionType = constructorInfo.exceptions.get( 0 );

            Assert.assertTrue(
                    exceptionType instanceof AsmTypevar );

            final AsmTypevar exceptionTypevar = (AsmTypevar) exceptionType;

            Assert.assertEquals(
                    //expected
                    "T" ,
                    //actual
                    exceptionTypevar.name );
        }
        //Assert.fail( "Not yet implemented" );
    }

}
