package de.heinerkuecker.asm_improved;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.test_data.ConstructorWithGenericException2;
import de.heinerkuecker.asm_improved.test_data.MethodWithGenericException2;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr {@link ConstructorWithGenericException2}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ConstructorWithGenericException2Test
{

    /**
     * Test method for {@link ConstructorWithGenericException2#ConstructorWithGenericException2()}.
     */
    @Test
    public void testConstructor()
    {
        final AsmClassInfo constructorWithGenericException2ClassInfo =
                AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
                		ConstructorWithGenericException2.class.getCanonicalName() );

        final ImmutableArray<? extends AsmMethodInfo> constructorInfos =
        		constructorWithGenericException2ClassInfo.methods.get(
        				AsmMethodInfo.CONSTRUCTOR_METHOD_NAME );

        Assert.assertEquals( 1 , constructorInfos.size() );

        final AsmMethodInfo mMethodInfo = constructorInfos.get( 0 );

        {
            Assert.assertEquals(
                    2 ,
                    mMethodInfo.typeParams.size() );

            {
                final AsmAbstractTypeParam typeParam0 = mMethodInfo.typeParams.get( 0 );

                Assert.assertEquals(
                        //expected
                        "T0" ,
                        //actual
                        typeParam0.name );

                Assert.assertTrue(
                        typeParam0 instanceof AsmExtendsTypeParam );

                AsmExtendsTypeParam extendsTypeParam0 = (AsmExtendsTypeParam) typeParam0;

                final AsmType extendsTypeParam0BoundType = extendsTypeParam0.boundType;

                Assert.assertTrue(
                        extendsTypeParam0BoundType instanceof AsmClassType );

                final AsmClassType extendsTypeParam0BoundClassType = (AsmClassType) extendsTypeParam0BoundType;

                Assert.assertEquals(
                        //expected
                        new Fqn( Throwable.class ) ,
                        //actual
                        extendsTypeParam0BoundClassType.fqn );

                Assert.assertTrue(
                        extendsTypeParam0BoundClassType.typeArgs == null || extendsTypeParam0BoundClassType.typeArgs.isEmpty() );
            }

            {
                final AsmAbstractTypeParam typeParam1 = mMethodInfo.typeParams.get( 1 );

                Assert.assertEquals(
                        //expected
                        "T1" ,
                        //actual
                        typeParam1.name );

                Assert.assertTrue(
                        typeParam1 instanceof AsmExtendsTypeParam );

                AsmExtendsTypeParam extendsTypeParam1 = (AsmExtendsTypeParam) typeParam1;

                final AsmType extendsTypeParam1BoundType = extendsTypeParam1.boundType;

                Assert.assertTrue(
                        extendsTypeParam1BoundType instanceof AsmClassType );

                final AsmClassType extendsTypeParam1BoundClassType = (AsmClassType) extendsTypeParam1BoundType;

                Assert.assertEquals(
                        //expected
                        new Fqn( Throwable.class ) ,
                        //actual
                        extendsTypeParam1BoundClassType.fqn );

                Assert.assertTrue(
                        extendsTypeParam1BoundClassType.typeArgs == null || extendsTypeParam1BoundClassType.typeArgs.isEmpty() );
            }
        }

        {
            Assert.assertEquals(
                    2 ,
                    mMethodInfo.exceptions.size() );

            {
                //System.out.println( mMethodInfo.exceptions.get( 0 ) );

                final AsmType exceptionType0 = mMethodInfo.exceptions.get( 0 );

                Assert.assertTrue(
                        exceptionType0 instanceof AsmTypevar );

                final AsmTypevar exceptionTypevar0 = (AsmTypevar) exceptionType0;

                Assert.assertEquals(
                        //expected
                        "T0" ,
                        //actual
                        exceptionTypevar0.name );
            }

            {
                //System.out.println( mMethodInfo.exceptions.get( 1 ) );

                final AsmType exceptionType1 = mMethodInfo.exceptions.get( 1 );

                Assert.assertTrue(
                        exceptionType1 instanceof AsmTypevar );

                final AsmTypevar exceptionTypevar1 = (AsmTypevar) exceptionType1;

                Assert.assertEquals(
                        //expected
                        "T1" ,
                        //actual
                        exceptionTypevar1.name );
            }
        }
        //Assert.fail( "Not yet implemented" );
    }

}
