package de.heinerkuecker.asm_improved;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.type.AsmArray;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.arg.AsmUnboundWildcard;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;

/**
 * JUnit4-Testcase fr Test der Methode {@link MethodElementHandler#getValueObject}.
 * 
 * @author Heiner K&uuml;cker
 */
public class MethodElementHandlerTest
{
    /**
     * Test method for {@link MethodElementHandler#getValueObject}.
     */
    @Test
    public final void testMethodInfoJavaLangString()
    {
        final AsmClassInfo classInfo =
                AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
                        "com.sun.beans.decoder.MethodElementHandler" );

        Assert.assertFalse(
                classInfo.isPublic );

        Assert.assertFalse(
                classInfo.isProtected );

        Assert.assertFalse(
                classInfo.isPrivate );

        Assert.assertFalse(
                classInfo.isAbstract );

        Assert.assertTrue(
                classInfo.isFinal );

        final ImmutableMap<String, ? extends ImmutableArray<? extends AsmMethodInfo>> methods =
                classInfo.methods;

        Assert.assertNotNull(
                methods );

        Assert.assertTrue(
                methods.containsKey( "getValueObject" ) );

        final ImmutableArray<? extends AsmMethodInfo> methodListWithSameName =
                methods.get(
                        "getValueObject" );
        
        Assert.assertEquals(
                //expected
                1 ,
                //actual
                methodListWithSameName.size() );

        final AsmMethodInfo getValueObjectMethod =
                methodListWithSameName.get( 0 );
        
        Assert.assertFalse(
                getValueObjectMethod.isPublic );

        Assert.assertTrue(
                getValueObjectMethod.isProtected );

        Assert.assertFalse(
                getValueObjectMethod.isPrivate );

        Assert.assertFalse(
                getValueObjectMethod.isAbstract );

        Assert.assertFalse(
                getValueObjectMethod.isFinal );

        Assert.assertFalse(
                getValueObjectMethod.isStatic );


        Assert.assertEquals(
                //expected
                1 ,
                //actual
                getValueObjectMethod.exceptions.size() );

        final AsmType exceptionType = getValueObjectMethod.exceptions.get( 0 );
        
        Assert.assertEquals(
                //expected
                new AsmClassType(
                        new Fqn( java.lang.Exception.class ) ) ,
                //actual
                exceptionType );

        Assert.assertEquals(
                //expected
                new AsmClassType(
                        new Fqn(
                        		com.sun.beans.decoder.ValueObject.class ) ) ,
                //actual
                getValueObjectMethod.returnType );

        Assert.assertEquals(
                //expected
                2 ,
                //actual
                getValueObjectMethod.getParamCount() );

        {
            final AsmMethodParamInfo param0 = getValueObjectMethod.params.get( 0 );

            final AsmType param0Type = param0.type;

            Assert.assertTrue(
                    param0Type instanceof AsmClassType );
            
            final AsmClassType param0ClassType = (AsmClassType) param0Type;
            
            Assert.assertEquals(
                    //expected
                    new AsmClassType(
                            Fqn.JAVA_LANG_CLASS_FQN ,
                            AsmUnboundWildcard.INSTANCE ) ,
                    //actual
                    param0ClassType );
        }

        {
            final AsmMethodParamInfo param1 = getValueObjectMethod.params.get( 1 );

            final AsmType param1Type = param1.type;

            Assert.assertTrue(
                    param1Type instanceof AsmArray );
            
            final AsmArray param1ArrayType = (AsmArray) param1Type;

            Assert.assertTrue(
                    param1ArrayType.elementType instanceof AsmClassType );

            final AsmClassType param1ArrayElementClassType = (AsmClassType) param1ArrayType.elementType;
            
            Assert.assertEquals(
                    //expected
                    new AsmClassType(
                            Fqn.JAVA_LANG_OBJECT_FQN ) ,
                    //actual
                    param1ArrayElementClassType );
        }
    }
}
