package de.heinerkuecker.asm_improved;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmUnboundedTypeParam;
import de.heinerkuecker.asm_improved.type.primitive.AsmInt;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;

/**
 * JUnit4-Testcase fr die Klasse {@link AsmMethodInfo}.
 * 
 * @author Heiner K&uuml;cker
 */
public class MethodInfoTest
{
	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmMethodInfo}.
	 */
	@Test
	public final void testMethodInfoJavaLangString()
	{
		final AsmClassInfo stringClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						java.lang.String.class.getCanonicalName() );

		final ImmutableMap<String, ? extends ImmutableArray<? extends AsmMethodInfo>> methods =
				stringClassInfo.methods;

		Assert.assertNotNull(
				methods );

		Assert.assertTrue(
				methods.containsKey( "substring" ) );

		final ImmutableArray<? extends AsmMethodInfo> methodListWithSameName =
				methods.get(
						"substring" );

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				methodListWithSameName.size() );

		final AsmMethodInfo substringMethod0 =
				methodListWithSameName.get( 0 );
		
		final AsmMethodInfo substringMethod1 =
				methodListWithSameName.get( 1 );


		Assert.assertTrue(
				substringMethod0.isPublic );

		Assert.assertFalse(
				substringMethod0.isProtected );

		Assert.assertFalse(
				substringMethod0.isPrivate );

		Assert.assertFalse(
				substringMethod0.isAbstract );

		Assert.assertFalse(
				substringMethod0.isFinal );

		Assert.assertFalse(
				substringMethod0.isStatic );

        Assert.assertTrue(
                substringMethod0.exceptions == null || substringMethod0.exceptions.isEmpty() );

        
		Assert.assertTrue(
				substringMethod1.isPublic );

		Assert.assertFalse(
				substringMethod1.isProtected );

		Assert.assertFalse(
				substringMethod1.isPrivate );

		Assert.assertFalse(
				substringMethod1.isAbstract );

		Assert.assertFalse(
				substringMethod1.isFinal );

		Assert.assertFalse(
				substringMethod1.isStatic );

        Assert.assertTrue(
                substringMethod1.exceptions == null || substringMethod1.exceptions.isEmpty() );


		final AsmMethodInfo substringMethodParamCount1;
		final AsmMethodInfo substringMethodParamCount2;
		if ( substringMethod0.params.size() == 1 )
		{
			substringMethodParamCount1 = substringMethod0;
			substringMethodParamCount2 = substringMethod1;
		}
		else
		{
			substringMethodParamCount1 = substringMethod1;
			substringMethodParamCount2 = substringMethod0;
		}

		Assert.assertEquals(
				//expected
				//"java.lang.String"
				AsmClassType.JAVA_LANG_STRING ,
				//actual
				substringMethodParamCount1.returnType );

		Assert.assertEquals(
				//expected
				//"java.lang.String"
				AsmClassType.JAVA_LANG_STRING ,
				//actual
				substringMethodParamCount2.returnType );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				substringMethodParamCount1.getParamCount() );

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				substringMethodParamCount2.getParamCount() );

		Assert.assertEquals(
				//expected
				AsmInt.INSTANCE ,
				//actual
				substringMethodParamCount1.params.get( 0 ).type );

		Assert.assertTrue(
				substringMethodParamCount1.params.get( 0 ).annotations == null || substringMethodParamCount1.params.get( 0 ).annotations.isEmpty() );

		Assert.assertEquals(
				//expected
				AsmInt.INSTANCE ,
				//actual
				substringMethodParamCount2.params.get( 0 ).type );

		Assert.assertTrue(
				substringMethodParamCount2.params.get( 0 ).annotations == null || substringMethodParamCount2.params.get( 0 ).annotations.isEmpty() );

		Assert.assertEquals(
				//expected
				AsmInt.INSTANCE ,
				//actual
				substringMethodParamCount2.params.get( 1 ).type );

		Assert.assertTrue(
				substringMethodParamCount2.params.get( 1 ).annotations == null || substringMethodParamCount2.params.get( 1 ).annotations.isEmpty() );
		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmMethodInfo}.
	 */
	@Test
	public final void testMethodInfoJavaUtilMapEntry()
	{
		final AsmClassInfo entryClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						java.util.Map.Entry.class.getCanonicalName() );

		final ImmutableMap<String, ? extends ImmutableArray<? extends AsmMethodInfo>> methods =
				entryClassInfo.methods;

		Assert.assertTrue(
				methods.containsKey( "getKey" ) );

		Assert.assertTrue(
				methods.containsKey( "getValue" ) );

		{
			final ImmutableArray<? extends AsmMethodInfo> getKeyMethods =
					methods.get(
							"getKey" );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					getKeyMethods.size() );

			final AsmMethodInfo getKeyMethod = getKeyMethods.get( 0 );

			Assert.assertTrue(
					getKeyMethod.isPublic );

			Assert.assertFalse(
					getKeyMethod.isProtected );

			Assert.assertFalse(
					getKeyMethod.isPrivate );

			Assert.assertTrue(
					getKeyMethod.isAbstract );

			Assert.assertFalse(
					getKeyMethod.isFinal );

			Assert.assertFalse(
					getKeyMethod.isStatic );

	        Assert.assertTrue(
	                getKeyMethod.exceptions == null || getKeyMethod.exceptions.isEmpty() );

	        Assert.assertEquals(
					//expected
					"K" ,
					//actual
					getKeyMethod.returnType.toString() );

			Assert.assertEquals(
					//expected
					0 ,
					//actual
					getKeyMethod.getParamCount() );
		}

		{
			final ImmutableArray<? extends AsmMethodInfo> getValueMethods =
					methods.get(
							"getValue" );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					getValueMethods.size() );

			final AsmMethodInfo getValueMethod = getValueMethods.get( 0 );

			Assert.assertTrue(
					getValueMethod.isPublic );

			Assert.assertFalse(
					getValueMethod.isProtected );

			Assert.assertFalse(
					getValueMethod.isPrivate );

			Assert.assertTrue(
					getValueMethod.isAbstract );

			Assert.assertFalse(
					getValueMethod.isFinal );

			Assert.assertFalse(
					getValueMethod.isStatic );

	        Assert.assertTrue(
	                getValueMethod.exceptions == null || getValueMethod.exceptions.isEmpty() );

	        Assert.assertEquals(
					//expected
					"V" ,
					//actual
					getValueMethod.returnType.toString() );

			Assert.assertEquals(
					//expected
					0 ,
					//actual
					getValueMethod.getParamCount() );
		}
		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmMethodInfo}.
	 */
	@Test
	public final void testMethodInfoJavaUtilArrayList()
	{
		final AsmClassInfo abstractListClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						java.util.AbstractList.class.getCanonicalName() );

		final ImmutableMap<String, ? extends ImmutableArray<? extends AsmMethodInfo>> methods =
				abstractListClassInfo.methods;

		//for ( Entry<String, ImmutableArray<MethodInfo>> methodArrEntry : methods.entrySet() )
		//{
		//	for ( MethodInfo method : methodArrEntry.getValue() )
		//	{
		//		System.out.println( method );
		//	}
		//}
		
		Assert.assertFalse(
				// diese Methode ist in java.util.AbstractCollection deklariert
				methods.containsKey(
						"retainAll" ) );

		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmMethodInfo}.
	 */
	@Test
	public final void testMethodInfoJavaUtilCollections()
	{
		final AsmClassInfo collectionsClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						java.util.Collections.class.getCanonicalName() );

		final ImmutableMap<String, ? extends ImmutableArray<? extends AsmMethodInfo>> methods =
				collectionsClassInfo.methods;

		Assert.assertTrue(
				methods.containsKey(
						"emptyList" ) );

		final ImmutableArray<? extends AsmMethodInfo> methodListWithSameName =
				methods.get(
						"emptyList" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodListWithSameName.size() );

		final AsmMethodInfo method = methodListWithSameName.get( 0 );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				method.typeParams.size() );

		final AsmAbstractTypeParam typeParam = method.typeParams.get( 0 );

		Assert.assertEquals(
				//expected
				"T" ,
				//actual
				typeParam.name );

		Assert.assertTrue(
				typeParam instanceof AsmUnboundedTypeParam );

		Assert.assertEquals(
				//expected
				"java.util.List<T>" ,
				//actual
				method.returnType.toString() );
		//Assert.fail("Not yet implemented");
	}

}
