package de.heinerkuecker.asm_improved;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.test_data.MethodWithGenericException1;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr {@link MethodWithGenericException1}.
 * 
 * @author Heiner K&uuml;cker
 */
public class MethodWithGenericException1Test
{

    /**
     * Test method for {@link de.heinerkuecker.asm_improved.test_data.MethodWithGenericException1#m()}.
     */
    @Test
    public void testM()
    {
        final AsmClassInfo methodWithGenericException1ClassInfo =
                AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
                        MethodWithGenericException1.class.getCanonicalName() );

        final ImmutableArray<? extends AsmMethodInfo> mMethodInfos =
                methodWithGenericException1ClassInfo.methods.get(
                        "m" );

        Assert.assertEquals( 1 , mMethodInfos.size() );

        final AsmMethodInfo mMethodInfo = mMethodInfos.get( 0 );

        {
            Assert.assertEquals(
                    1 ,
                    mMethodInfo.typeParams.size() );

            final AsmAbstractTypeParam typeParam = mMethodInfo.typeParams.get( 0 );

            Assert.assertEquals(
                    //expected
                    "T" ,
                    //actual
                    typeParam.name );

            Assert.assertTrue(
                    typeParam instanceof AsmExtendsTypeParam );

            AsmExtendsTypeParam extendsTypeParam = (AsmExtendsTypeParam) typeParam;

            final AsmType extendsTypeParamBoundType = extendsTypeParam.boundType;

            Assert.assertTrue(
                    extendsTypeParamBoundType instanceof AsmClassType );

            final AsmClassType extendsTypeParamBoundClassType = (AsmClassType) extendsTypeParamBoundType;

            Assert.assertEquals(
                    //expected
                    new Fqn( Throwable.class ) ,
                    //actual
                    extendsTypeParamBoundClassType.fqn );

            Assert.assertTrue(
                    extendsTypeParamBoundClassType.typeArgs == null || extendsTypeParamBoundClassType.typeArgs.isEmpty() );
        }

        {
            Assert.assertEquals(
                    1 ,
                    mMethodInfo.exceptions.size() );

            //System.out.println( mMethodInfo.exceptions.get( 0 ) );

            final AsmType exceptionType = mMethodInfo.exceptions.get( 0 );

            Assert.assertTrue(
                    exceptionType instanceof AsmTypevar );

            final AsmTypevar exceptionTypevar = (AsmTypevar) exceptionType;

            Assert.assertEquals(
                    //expected
                    "T" ,
                    //actual
                    exceptionTypevar.name );
        }
        //Assert.fail( "Not yet implemented" );
    }

}
