package de.heinerkuecker.asm_improved;

import java.io.IOException;
import java.io.NotSerializableException;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.test_data.MethodWithNotGenericException2;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr {@link MethodWithNotGenericException2}.
 * 
 * @author Heiner K&uuml;cker
 */
public class MethodWithNotGenericException2Test
{

    /**
     * Test method for {@link de.heinerkuecker.asm_improved.test_data.MethodWithNotGenericException2#m()}.
     */
    @Test
    public void testM()
    {
        final AsmClassInfo methodWithGenericExceptionClassInfo =
                AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
                        MethodWithNotGenericException2.class.getCanonicalName() );

        final ImmutableArray<? extends AsmMethodInfo> mMethodInfos =
                methodWithGenericExceptionClassInfo.methods.get(
                        "m" );

        Assert.assertEquals( 1 , mMethodInfos.size() );

        final AsmMethodInfo mMethodInfo = mMethodInfos.get( 0 );

        Assert.assertEquals(
                2 ,
                mMethodInfo.exceptions.size() );

        {
            //System.out.println( mMethodInfo.exceptions.get( 0 ) );

            final AsmType exceptionType0 = mMethodInfo.exceptions.get( 0 );

            Assert.assertTrue(
                    exceptionType0 instanceof AsmClassType );

            final AsmClassType exceptionClassType0 = (AsmClassType) exceptionType0;

            Assert.assertEquals(
                    //expected
                    new Fqn( IOException.class ) ,
                    //actual
                    exceptionClassType0.fqn );

            Assert.assertTrue(
                    exceptionClassType0.typeArgs == null || exceptionClassType0.typeArgs.isEmpty() );
        }

        {
            //System.out.println( mMethodInfo.exceptions.get( 1 ) );

            final AsmType exceptionType1 = mMethodInfo.exceptions.get( 1 );

            Assert.assertTrue(
                    exceptionType1 instanceof AsmClassType );

            final AsmClassType exceptionClassType1 = (AsmClassType) exceptionType1;

            Assert.assertEquals(
                    //expected
                    new Fqn( NotSerializableException.class ) ,
                    //actual
                    exceptionClassType1.fqn );

            Assert.assertTrue(
                    exceptionClassType1.typeArgs == null || exceptionClassType1.typeArgs.isEmpty() );
        }
        //Assert.fail( "Not yet implemented" );
    }

}
