package de.heinerkuecker.asm_improved;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.test_data.MethodWithTypeParamAsTypevarOfClass;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.arg.AsmExtendsWildcard;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmUnboundedTypeParam;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr {@link MethodWithTypeParamAsTypevarOfClass}.
 * 
 * @author Heiner K&uuml;cker
 */
public class MethodWithTypeParamAsTypevarOfClassTest
{

    /**
     * Test method for {@link de.heinerkuecker.asm_improved.test_data.MethodWithGenericException1#m()}.
     */
    @Test
    public void testM()
    {
        final AsmClassInfo methodWithGenericExceptionClassInfo =
                AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
                        MethodWithTypeParamAsTypevarOfClass.class.getCanonicalName() );

        final ImmutableArray<? extends AsmMethodInfo> mMethodInfos =
                methodWithGenericExceptionClassInfo.methods.get(
                        "m" );

        Assert.assertEquals( 1 , mMethodInfos.size() );
        
        final AsmMethodInfo mMethodInfo = mMethodInfos.get( 0 );

        //System.out.println( mMethodInfo );
        
        Assert.assertTrue(
                mMethodInfo.exceptions == null || mMethodInfo.exceptions.isEmpty() );
        
        Assert.assertEquals(
                1 ,
                mMethodInfo.typeParams.size() );
        
        final AsmAbstractTypeParam typeParam = mMethodInfo.typeParams.get( 0 );
        
        //System.out.println( typeParam );

        Assert.assertEquals(
                "T" ,
                typeParam.name );

        Assert.assertTrue(
                typeParam instanceof AsmUnboundedTypeParam );

        Assert.assertTrue(
                mMethodInfo.returnType instanceof AsmClassType );
        
        final AsmClassType returnType = (AsmClassType) mMethodInfo.returnType;

        //System.out.println( returnType );

        Assert.assertEquals(
                Fqn.JAVA_LANG_CLASS_FQN ,
                returnType.fqn );

        Assert.assertEquals(
                1 ,
                returnType.typeArgs.size() );

        final AsmType returnTypeArgType = returnType.typeArgs.get( 0 );
        
        Assert.assertTrue(
                returnTypeArgType instanceof AsmExtendsWildcard );

        final AsmExtendsWildcard returnTypeArgExtendsWildcard = (AsmExtendsWildcard) returnTypeArgType;
        
        Assert.assertTrue(
                returnTypeArgExtendsWildcard.boundType instanceof AsmTypevar );
        
        final AsmTypevar returnTypevarTypeArgBoundTypevar = (AsmTypevar) returnTypeArgExtendsWildcard.boundType;
        
        Assert.assertEquals(
                "T" ,
                returnTypevarTypeArgBoundTypevar.name );

        //Assert.fail( "Not yet implemented" );
    }

}
