package de.heinerkuecker.asm_improved;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.test_data.MultiEnclosingClassExtendsMultiNestedClassImplementsMultiNestedInterface;
import de.heinerkuecker.asm_improved.test_data.MultiNestedClass;
import de.heinerkuecker.asm_improved.test_data.MultiNestedClassAsTypeArgument;
import de.heinerkuecker.asm_improved.test_data.MultiNestedClassAsTypeBound;
import de.heinerkuecker.asm_improved.test_data.MultiNestedClassExtendsMultiNestedClass;
import de.heinerkuecker.asm_improved.test_data.MultiNestedInterface;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.arg.AsmExtendsWildcard;
import de.heinerkuecker.asm_improved.type.generics.arg.AsmSuperWildcard;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link AsmClassInfo}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class MultiNestedTest
{
	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoMultiNestedClass()
	{
		final AsmClassInfo multiNestedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class.getCanonicalName() );

		Assert.assertFalse(
				multiNestedClassInfo.isTopLevel() );

		Assert.assertFalse(
				multiNestedClassInfo.isFinal );

		Assert.assertTrue(
				multiNestedClassInfo.isAbstract );

		Assert.assertTrue(
				multiNestedClassInfo.isStatic );

		Assert.assertFalse(
				multiNestedClassInfo.isInterface );

		Assert.assertFalse(
				multiNestedClassInfo.isEnum );

		Assert.assertFalse(
				multiNestedClassInfo.isAnnotation );

		Assert.assertEquals(
				//expected
				//"java.lang.Object"
				Fqn.JAVA_LANG_OBJECT_FQN ,
				//actual
				multiNestedClassInfo.superType.fqn );

		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoMultiNestedClassExtendsMultiNestedClass()
	{
		final AsmClassInfo multiNestedClassExtendsMultiEnclosingClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						MultiNestedClassExtendsMultiNestedClass.NestedClass0.NestedClass1.EnclosingClass2.NestedClass.class.getCanonicalName() );

		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassInfo.isTopLevel() );

		Assert.assertTrue(
				multiNestedClassExtendsMultiEnclosingClassInfo.isFinal );

		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassInfo.isAbstract );

		Assert.assertTrue(
				multiNestedClassExtendsMultiEnclosingClassInfo.isStatic );

		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassInfo.isInterface );

		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassInfo.isEnum );

		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassInfo.isAnnotation );

		Assert.assertEquals(
				//expected
				new Fqn( MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class ) ,
				//actual
				multiNestedClassExtendsMultiEnclosingClassInfo.superType.fqn );

		final AsmClassInfo multiNestedClassExtendsMultiEnclosingClassSuperClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						multiNestedClassExtendsMultiEnclosingClassInfo.superType.fqn );
				
		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassSuperClassInfo.isTopLevel() );

		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassSuperClassInfo.isFinal );

		Assert.assertTrue(
				multiNestedClassExtendsMultiEnclosingClassSuperClassInfo.isAbstract );

		Assert.assertTrue(
				multiNestedClassExtendsMultiEnclosingClassSuperClassInfo.isStatic );

		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassSuperClassInfo.isInterface );

		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassSuperClassInfo.isEnum );

		Assert.assertFalse(
				multiNestedClassExtendsMultiEnclosingClassSuperClassInfo.isAnnotation );

		Assert.assertEquals(
				//expected
				//"java.lang.Object"
				Fqn.JAVA_LANG_OBJECT_FQN ,
				//actual
				multiNestedClassExtendsMultiEnclosingClassSuperClassInfo.superType.fqn );

		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoMultiNestedClassExtendsMultiNestedClassImplementsMultiNestedInterface()
	{
		final AsmClassInfo multiNestedClassExtendsMultiEnclosingClassImplementsMultiNestedInterfaceInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						MultiEnclosingClassExtendsMultiNestedClassImplementsMultiNestedInterface.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class.getCanonicalName() );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				multiNestedClassExtendsMultiEnclosingClassImplementsMultiNestedInterfaceInfo.interfaceTypes.size() );

		final AsmClassType interfaceType =
				multiNestedClassExtendsMultiEnclosingClassImplementsMultiNestedInterfaceInfo.interfaceTypes.get( 0 );

		Assert.assertEquals(
				//expected
				new Fqn( MultiNestedInterface.NestedInterface0.NestedInterface1.NestedInterface2.NestedInterface3.class ) ,
				//actual
				interfaceType.fqn );

		final AsmClassInfo interfaceClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						interfaceType.fqn );

		Assert.assertNotNull(
				interfaceClassInfo );

		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoMultiNestedClassAsTypeArgument()
	{
		final AsmClassInfo multiNestedClassAsTypeArgumentInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						MultiNestedClassAsTypeArgument.class.getCanonicalName() );

		{
			Assert.assertNotNull(
					multiNestedClassAsTypeArgumentInfo.superType );

			final AsmClassType superClassType = multiNestedClassAsTypeArgumentInfo.superType;

			Assert.assertEquals(
					//expected
					new Fqn( java.util.ArrayList.class ) ,
					//actual
					superClassType.fqn );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					superClassType.typeArgs.size() );

			final AsmType superClassTypeArg = superClassType.typeArgs.get( 0 );

			Assert.assertTrue(
					superClassTypeArg instanceof AsmClassType );

			final AsmClassType superClassTypeArgClass = (AsmClassType) superClassTypeArg;

			Assert.assertEquals(
					//expected
					new Fqn( MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class ) ,
					//actual
					superClassTypeArgClass.fqn );
		}

		{
			Assert.assertNotNull(
					multiNestedClassAsTypeArgumentInfo.fields );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					multiNestedClassAsTypeArgumentInfo.fields.size() );

			Assert.assertTrue(
					multiNestedClassAsTypeArgumentInfo.fields.containsKey( "listField" ) );

			final AsmFieldInfo listField = multiNestedClassAsTypeArgumentInfo.fields.get( "listField" );

			Assert.assertEquals(
					//expected
					"java.util.List<" + MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class.getCanonicalName() + ">" ,
					//actual
					listField.type.toString() );

			Assert.assertTrue(
					listField.type instanceof AsmClassType );

			final AsmClassType listFieldType = (AsmClassType) listField.type;
			
			Assert.assertEquals(
					//expected
					1 ,
					//actual
					listFieldType.typeArgs.size() );

			final AsmType typeArg = listFieldType.typeArgs.get( 0 );

			Assert.assertTrue(
					typeArg instanceof AsmClassType );

			final AsmClassType typeArgType = (AsmClassType) typeArg;
			
			Assert.assertEquals(
					//expected
					new Fqn( MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class ) ,
					//actual
					typeArgType.fqn );
		}

		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link de.heinerkuecker.asm_improved.AsmClassInfo}.
	 */
	@Test
	public final void testClassInfoMultiNestedClassAsTypeBound()
	{
		final AsmClassInfo multiNestedClassAsTypeBoundInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						MultiNestedClassAsTypeBound.class.getCanonicalName() );

		{
			Assert.assertEquals(
					//expected
					1 ,
					//actual
					multiNestedClassAsTypeBoundInfo.typeParams.size() );

			final AsmAbstractTypeParam typeParam = multiNestedClassAsTypeBoundInfo.typeParams.get( 0 );

			Assert.assertTrue(
					typeParam instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam = (AsmExtendsTypeParam) typeParam;

			//final ImmutableArray<? extends AsmType> boundTypes = extendsTypeParam.boundTypes;
			//Assert.assertEquals(
			//		//expected
			//		1 ,
			//		//actual
			//		boundTypes.size() );
			//final AsmType boundType = boundTypes.get( 0 );
			final AsmType boundType = extendsTypeParam.boundType;

			Assert.assertTrue(
					boundType instanceof AsmClassType );

			final AsmClassType boundClassType = (AsmClassType) boundType;

			Assert.assertEquals(
					//expected
					new Fqn( MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class ) ,
					//actual
					boundClassType.fqn );
		}

		Assert.assertEquals(
				//expected
				3 ,
				//actual
				multiNestedClassAsTypeBoundInfo.fields.size() );

		{
			final AsmFieldInfo typevarArgListField =
					multiNestedClassAsTypeBoundInfo.fields.get(
							"typevarArgListField" );

			final AsmType typevarArgListFieldType = typevarArgListField.type;

			Assert.assertTrue(
					typevarArgListFieldType instanceof AsmClassType );

			final AsmClassType typevarArgListFieldClassType = (AsmClassType) typevarArgListFieldType;

			Assert.assertEquals(
					//expected
					"java.util.List<T>" ,
					//actual
					typevarArgListFieldClassType.toString() );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					typevarArgListFieldClassType.typeArgs.size() );

			final AsmType typevarArgListFieldClassTypeTypeArg = typevarArgListFieldClassType.typeArgs.get( 0 );

			Assert.assertTrue(
					typevarArgListFieldClassTypeTypeArg instanceof AsmTypevar );

			final AsmTypevar typevarArgListFieldClassTypeTypeArgTypevar = (AsmTypevar) typevarArgListFieldClassTypeTypeArg;

			Assert.assertEquals(
					//expected
					"T" ,
					//actual
					typevarArgListFieldClassTypeTypeArgTypevar.name );
		}

		{
			final AsmFieldInfo extendsBoundListField =
					multiNestedClassAsTypeBoundInfo.fields.get(
							"extendsBoundListField" );

			final AsmType extendsBoundListFieldType = extendsBoundListField.type;

			Assert.assertTrue(
					extendsBoundListFieldType instanceof AsmClassType );

			final AsmClassType extendsBoundListFieldClassType = (AsmClassType) extendsBoundListFieldType;

			Assert.assertEquals(
					//expected
					"java.util.List<? extends " + new Fqn( MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class ) + ">" ,
					//actual
					extendsBoundListFieldClassType.toString() );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					extendsBoundListFieldClassType.typeArgs.size() );

			final AsmType extendsBoundListFieldClassTypeTypeArg = extendsBoundListFieldClassType.typeArgs.get( 0 );

			Assert.assertTrue(
					extendsBoundListFieldClassTypeTypeArg instanceof AsmExtendsWildcard );

			final AsmExtendsWildcard extendsBoundListFieldClassTypeExtendsWildcardTypeArg = (AsmExtendsWildcard) extendsBoundListFieldClassTypeTypeArg;

			Assert.assertTrue(
					extendsBoundListFieldClassTypeExtendsWildcardTypeArg.boundType instanceof AsmClassType );

			final AsmClassType extendsBoundListFieldClassTypeExtendsWildcardTypeArgBoundType = (AsmClassType) extendsBoundListFieldClassTypeExtendsWildcardTypeArg.boundType;

			Assert.assertEquals(
					//expected
					new Fqn( MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class ) ,
					//actual
					extendsBoundListFieldClassTypeExtendsWildcardTypeArgBoundType.fqn );
		}

		{
			final AsmFieldInfo superBoundListField =
					multiNestedClassAsTypeBoundInfo.fields.get(
							"superBoundListField" );

			final AsmType superBoundListFieldType = superBoundListField.type;

			Assert.assertTrue(
					superBoundListFieldType instanceof AsmClassType );

			final AsmClassType superBoundListFieldClassType = (AsmClassType) superBoundListFieldType;

			Assert.assertEquals(
					//expected
					"java.util.List<? super " + new Fqn( MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class ) + ">" ,
					//actual
					superBoundListFieldClassType.toString() );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					superBoundListFieldClassType.typeArgs.size() );

			final AsmType superBoundListFieldClassTypeTypeArg = superBoundListFieldClassType.typeArgs.get( 0 );

			Assert.assertTrue(
					superBoundListFieldClassTypeTypeArg instanceof AsmSuperWildcard );

			final AsmSuperWildcard superBoundListFieldClassTypeSuperWildcardTypeArg = (AsmSuperWildcard) superBoundListFieldClassTypeTypeArg;

			Assert.assertTrue(
					superBoundListFieldClassTypeSuperWildcardTypeArg.boundType instanceof AsmClassType );

			final AsmClassType superBoundListFieldClassTypeSuperWildcardTypeArgBoundType = (AsmClassType) superBoundListFieldClassTypeSuperWildcardTypeArg.boundType;

			Assert.assertEquals(
					//expected
					new Fqn( MultiNestedClass.NestedClass0.NestedClass1.NestedClass2.NestedClass3.class ) ,
					//actual
					superBoundListFieldClassTypeSuperWildcardTypeArgBoundType.fqn );
		}

		//Assert.fail("Not yet implemented");
	}

}
