package de.heinerkuecker.asm_improved;

import java.util.HashSet;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.util.ContainsTypesafeSet;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link AsmClassInfo}.<br/>
 * Pr&uuml;fen, ob ein untergeordneter Test mehrfach in einer Test-Suite notiert ist.
 * 
 * @author Heiner K&uuml;cker
 */
public class TestsuiteUniqueSubTest
{
	/**
	 * Test method for {@link AsmClassInfo}.
	 */
	@Test
	public final void test()
	{
		/*
		 * Suchen nach Test-Suites
		 */
		final AsmClassType testSuiteAnnotationType = new AsmClassType( new Fqn( org.junit.runners.Suite.SuiteClasses.class ) );

		final AsmClasspathInfo asmClasspathInfo =
				new AsmClasspathInfo(
						AsmClasspathInfo.Mode.EAGER ,
						AllAsmTests.RT_JAR_FILE ,
						AllAsmTests.BIN_DIR_FILE );

		for ( final AsmClassInfo classInfo : asmClasspathInfo.getAllClassInfos() )
		{
			if ( classInfo.annotations != null )
			{
				for ( final AsmAnnotationUse annotation : classInfo.annotations )
				{
					if ( annotation.annotationType.equals( testSuiteAnnotationType ) )
					{
						innerTestOneSuiteAnnotationUniqueSub(
								classInfo ,
								annotation );
					}
				}
			}
		}
	}

	private static void innerTestOneSuiteAnnotationUniqueSub(
			final AsmClassInfo testSuiteClassInfo ,
			final AsmAnnotationUse testSuiteAnnotation )
	{
		//System.out.println( testSuiteClassInfo.fqn.toFqnStr() );

		final Object[] testAsmClassTypeObjArr = (Object[]) testSuiteAnnotation.values .get( "value" );

		final ContainsTypesafeSet<Fqn> subTestClassFqns = ContainsTypesafeSet.create( new HashSet<>() );

		for ( final Object testAsmClassTypeObj : testAsmClassTypeObjArr )
		{
			final AsmClassType testAsmClassType = (AsmClassType) testAsmClassTypeObj;

			final Fqn testClassFqn = testAsmClassType.fqn;

			//System.out.println( "\t" + fqn.toFqnStr() );

			if ( subTestClassFqns.contains( testClassFqn ) )
			{
				Assert.fail(
						"not unique test " +
								testAsmClassType.fqn.toFqnStr() +
								" in suite " +
								testSuiteClassInfo.fqn.toFqnStr() );
			}

			subTestClassFqns.add( testClassFqn );
		}
	}

}
