package de.heinerkuecker.asm_improved;

import java.util.Map.Entry;
import java.util.Set;

import org.junit.Assert;
import org.junit.Test;

import com.sun.xml.internal.bind.v2.schemagen.XmlSchemaGenerator;

import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.type.member.AsmInnerClassType;
import de.heinerkuecker.asm_improved.type.primitive.AsmBoolean;
import de.heinerkuecker.asm_improved.util.DottedName;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.PackageName;

/**
 * JUnit4-Testcase fr {@link com.sun.xml.internal.bind.v2.schemagen.XmlSchemaGenerator.Namespace}.
 * 
 * @author Heiner K&uuml;cker
 */
public class XmlSchemaGeneratorNamespaceTest
{
	/**
	 * Test fr {@link XmlSchemaGenerator.Namespace}.
	 */
	@Test
	public void test()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						new Fqn(
								// com.sun.xml.internal.bind.v2.schemagen.XmlSchemaGenerator.Namespace
								new PackageName( "com.sun.xml.internal.bind.v2.schemagen" ) ,
								DottedName.valueOfDotSeparatedStr( "XmlSchemaGenerator.Namespace" ) ) );

		//System.out.println( classInfo.fields.size() );
		//for ( final Entry<String, ? extends AsmFieldInfo> fieldEntry : classInfo.fields.entrySet() )
		//{
		//	System.out.println( fieldEntry );
		//}
		
		Assert.assertEquals(
				//expected
				15 ,
				//actual
				classInfo.fields.size() );

		{
			final AsmFieldInfo uriField = classInfo.fields.get( "uri" );

			Assert.assertFalse( uriField.isPrivate );

			Assert.assertFalse( uriField.isSynthetic );
			
			Assert.assertEquals(
					//expected
					"uri" ,
					//actual
					uriField.name );

			Assert.assertEquals(
					//expected
					AsmClassType.JAVA_LANG_STRING ,
					//actual
					uriField.type );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					uriField.annotations.size() );

			final AsmAnnotationUse annotation = uriField.annotations.get( 0 );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn(
									new PackageName( "com.sun.istack.internal" ) ,
									DottedName.valueOfNoDotSeparatedStr( "NotNull" ) ) ) ,
					//actual
					annotation.annotationType );

			Assert.assertTrue(
					annotation.values == null || annotation.values.isEmpty() );
		}

		{
			final AsmFieldInfo dependsField = classInfo.fields.get( "depends" );

			Assert.assertTrue( dependsField.isPrivate );

			Assert.assertFalse( dependsField.isSynthetic );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn( Set.class ) ,
							// typeargs
							new AsmInnerClassType(
									//outerOrEnclosingType
									new AsmClassType(
											new Fqn(
													new PackageName( "com.sun.xml.internal.bind.v2.schemagen" ) ,
													DottedName.valueOfNoDotSeparatedStr( "XmlSchemaGenerator" ) ) ,
											//typeArgs
											ImmutableArray.create(
													new AsmTypevar( "T" ) ,
													new AsmTypevar( "C" ) ,
													new AsmTypevar( "F" ) ,
													new AsmTypevar( "M" ) ) ) ,
									//relativName
									//DottedName.valueOfNoDotSeparatedStr(
									//new JavaIdentifier(
											"Namespace" ) ) ,
					//actual
					dependsField.type );

			Assert.assertTrue(
					dependsField.annotations == null || dependsField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo selfReferenceField = classInfo.fields.get( "selfReference" );

			Assert.assertTrue( selfReferenceField.isPrivate );

			Assert.assertFalse( selfReferenceField.isSynthetic );

			Assert.assertEquals(
					//expected
					AsmBoolean.INSTANCE ,
					//actual
					selfReferenceField.type );

			Assert.assertTrue(
					selfReferenceField.annotations == null || selfReferenceField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo classesField = classInfo.fields.get( "classes" );

			Assert.assertTrue( classesField.isPrivate );

			Assert.assertFalse( classesField.isSynthetic );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn( Set.class ) ,
							// typeargs
							new AsmClassType(
									new Fqn(
											new PackageName( "com.sun.xml.internal.bind.v2.model.core" ) ,
											DottedName.valueOfNoDotSeparatedStr( "ClassInfo" ) ) ,
									//typeArgs
									ImmutableArray.create(
											new AsmTypevar( "T" ) ,
											new AsmTypevar( "C" ) ) ) ) ,
					//actual
					classesField.type );

			Assert.assertTrue(
					classesField.annotations == null || classesField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo enumsField = classInfo.fields.get( "enums" );

			Assert.assertTrue( enumsField.isPrivate );

			Assert.assertFalse( enumsField.isSynthetic );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn( Set.class ) ,
							// typeargs
							new AsmClassType(
									new Fqn(
											new PackageName( "com.sun.xml.internal.bind.v2.model.core" ) ,
											DottedName.valueOfNoDotSeparatedStr( "EnumLeafInfo" ) ) ,
									//typeArgs
									ImmutableArray.create(
											new AsmTypevar( "T" ) ,
											new AsmTypevar( "C" ) ) ) ) ,
					//actual
					enumsField.type );

			Assert.assertTrue(
					enumsField.annotations == null || enumsField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo arraysField = classInfo.fields.get( "arrays" );

			Assert.assertTrue( arraysField.isPrivate );

			Assert.assertFalse( arraysField.isSynthetic );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn( Set.class ) ,
							// typeargs
							new AsmClassType(
									new Fqn(
											new PackageName( "com.sun.xml.internal.bind.v2.model.core" ) ,
											DottedName.valueOfNoDotSeparatedStr( "ArrayInfo" ) ) ,
									//typeArgs
									ImmutableArray.create(
											new AsmTypevar( "T" ) ,
											new AsmTypevar( "C" ) ) ) ) ,
					//actual
					arraysField.type );

			Assert.assertTrue(
					arraysField.annotations == null || arraysField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo attributeDeclsField = classInfo.fields.get( "attributeDecls" );

			Assert.assertTrue( attributeDeclsField.isPrivate );

			Assert.assertFalse( attributeDeclsField.isSynthetic );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn(
									new PackageName( "com.sun.xml.internal.bind.v2.schemagen" ) ,
									DottedName.valueOfNoDotSeparatedStr( "MultiMap" ) ) ,
							// typeargs
							AsmClassType.JAVA_LANG_STRING ,
							new AsmClassType(
									new Fqn(
											new PackageName( "com.sun.xml.internal.bind.v2.model.core" ) ,
											DottedName.valueOfNoDotSeparatedStr( "AttributePropertyInfo" ) ) ,
									//typeArgs
									ImmutableArray.create(
											new AsmTypevar( "T" ) ,
											new AsmTypevar( "C" ) ) ) ) ,
					//actual
					attributeDeclsField.type );

			Assert.assertTrue(
					attributeDeclsField.annotations == null || attributeDeclsField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo elementDeclsField = classInfo.fields.get( "elementDecls" );

			Assert.assertTrue( elementDeclsField.isPrivate );

			Assert.assertFalse( elementDeclsField.isSynthetic );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn(
									// com.sun.xml.internal.bind.v2.schemagen.XmlSchemaGenerator.Namespace
									new PackageName( "com.sun.xml.internal.bind.v2.schemagen" ) ,
									DottedName.valueOfNoDotSeparatedStr( "MultiMap" ) ) ,
							// typeargs
							AsmClassType.JAVA_LANG_STRING ,
							new AsmInnerClassType(
									//outerOrEnclosingType
									new AsmInnerClassType(
											//outerOrEnclosingType
											new AsmClassType(
													new Fqn(
															new PackageName( "com.sun.xml.internal.bind.v2.schemagen" ) ,
															DottedName.valueOfNoDotSeparatedStr( "XmlSchemaGenerator" ) ) ,
													//typeArgs
													ImmutableArray.create(
															new AsmTypevar( "T" ) ,
															new AsmTypevar( "C" ) ,
															new AsmTypevar( "F" ) ,
															new AsmTypevar( "M" ) ) ) ,
											//relativName
											//DottedName.valueOfNoDotSeparatedStr(
											//new JavaIdentifier(
													"Namespace" ) ,
									//relativName
									//DottedName.valueOfNoDotSeparatedStr(
									//new JavaIdentifier(
											"ElementDeclaration" ) ) ,
					//actual
					elementDeclsField.type );

			Assert.assertTrue(
					elementDeclsField.annotations == null || elementDeclsField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo attributeFormDefaultField = classInfo.fields.get( "attributeFormDefault" );

			Assert.assertTrue( attributeFormDefaultField.isPrivate );

			Assert.assertFalse( attributeFormDefaultField.isSynthetic );
			
			Assert.assertEquals(
					//expected
					"attributeFormDefault" ,
					//actual
					attributeFormDefaultField.name );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn(
									new PackageName( "com.sun.xml.internal.bind.v2.schemagen" ) ,
									DottedName.valueOfNoDotSeparatedStr( "Form" ) ) ) ,
					//actual
					attributeFormDefaultField.type );

			Assert.assertTrue(
					attributeFormDefaultField.annotations == null || attributeFormDefaultField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo elementFormDefaultField = classInfo.fields.get( "elementFormDefault" );

			Assert.assertTrue( elementFormDefaultField.isPrivate );

			Assert.assertFalse( elementFormDefaultField.isSynthetic );
			
			Assert.assertEquals(
					//expected
					"elementFormDefault" ,
					//actual
					elementFormDefaultField.name );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn(
									new PackageName( "com.sun.xml.internal.bind.v2.schemagen" ) ,
									DottedName.valueOfNoDotSeparatedStr( "Form" ) ) ) ,
					//actual
					elementFormDefaultField.type );

			Assert.assertTrue(
					elementFormDefaultField.annotations == null || elementFormDefaultField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo useSwaRefField = classInfo.fields.get( "useSwaRef" );

			Assert.assertFalse( useSwaRefField.isPublic );

			Assert.assertTrue( useSwaRefField.isPrivate );

			Assert.assertFalse( useSwaRefField.isFinal );

			Assert.assertFalse( useSwaRefField.isStatic );

			Assert.assertFalse( useSwaRefField.isSynthetic );
			
			Assert.assertEquals(
					//expected
					"useSwaRef" ,
					//actual
					useSwaRefField.name );

			Assert.assertEquals(
					//expected
					AsmBoolean.INSTANCE ,
					//actual
					useSwaRefField.type );

			Assert.assertTrue(
					useSwaRefField.annotations == null || useSwaRefField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo useMimeNsField = classInfo.fields.get( "useMimeNs" );

			Assert.assertFalse( useMimeNsField.isPublic );

			Assert.assertTrue( useMimeNsField.isPrivate );

			Assert.assertFalse( useMimeNsField.isFinal );

			Assert.assertFalse( useMimeNsField.isStatic );

			Assert.assertFalse( useMimeNsField.isSynthetic );
			
			Assert.assertEquals(
					//expected
					"useMimeNs" ,
					//actual
					useMimeNsField.name );

			Assert.assertEquals(
					//expected
					AsmBoolean.INSTANCE ,
					//actual
					useMimeNsField.type );

			Assert.assertTrue(
					useMimeNsField.annotations == null || useMimeNsField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo $assertionsDisabledField = classInfo.fields.get( "$assertionsDisabled" );

			Assert.assertFalse( $assertionsDisabledField.isPublic );

			Assert.assertFalse( $assertionsDisabledField.isPrivate );

			Assert.assertTrue( $assertionsDisabledField.isFinal );

			Assert.assertTrue( $assertionsDisabledField.isStatic );

			Assert.assertTrue( $assertionsDisabledField.isSynthetic );
			
			Assert.assertEquals(
					//expected
					"$assertionsDisabled" ,
					//actual
					$assertionsDisabledField.name );

			Assert.assertEquals(
					//expected
					AsmBoolean.INSTANCE ,
					//actual
					$assertionsDisabledField.type );

			Assert.assertTrue(
					$assertionsDisabledField.annotations == null || $assertionsDisabledField.annotations.isEmpty() );
		}

		{
			final AsmFieldInfo this$0Field = classInfo.fields.get( "this$0" );

			Assert.assertFalse( this$0Field.isPrivate );

			Assert.assertTrue( this$0Field.isSynthetic );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn(
									new PackageName( "com.sun.xml.internal.bind.v2.schemagen" ) ,
									DottedName.valueOfNoDotSeparatedStr( "XmlSchemaGenerator" ) ) ) ,
					//actual
					this$0Field.type );

			Assert.assertTrue(
					this$0Field.annotations == null || this$0Field.annotations.isEmpty() );
		}

		{
			// Dieses Feld scheint erst mit einer neueren Version von Java8 drin zu sein oder nur unter Windows
			final AsmFieldInfo writtenField = classInfo.fields.get( "written" );

			Assert.assertTrue( writtenField.isPrivate );

			Assert.assertFalse( writtenField.isSynthetic );

			Assert.assertTrue( writtenField.isFinal );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn(
									java.util.Set.class ) ,
							new AsmClassType(
									new Fqn(
											new PackageName( "com.sun.xml.internal.bind.v2.model.core" ) ,
											DottedName.valueOfNoDotSeparatedStr( "ClassInfo" ) ) ) ) ,
					//actual
					writtenField.type );

			Assert.assertTrue(
					writtenField.annotations == null || writtenField.annotations.isEmpty() );
		}

		//Assert.fail( "Not yet implemented" );
	}

}
