package de.heinerkuecker.asm_improved.test.annotation_def;

import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotationDefDeepNestedWithDefault;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link TestAnnotationDefDeepNestedWithDefault}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AnnotationDefDeepNestedWithDefaultTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotationClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestAnnotationDefDeepNestedWithDefault.class.getCanonicalName() );

		Assert.assertTrue(
				annotationClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotationClassInfo.isFinal );

		Assert.assertTrue(
				annotationClassInfo.isAbstract );

		Assert.assertTrue(
				annotationClassInfo.isStatic );

		Assert.assertTrue(
				annotationClassInfo.isInterface );

		Assert.assertFalse(
				annotationClassInfo.isEnum );

		Assert.assertTrue(
				annotationClassInfo.isAnnotation );

		// --- super class ---

		Assert.assertNull(
				annotationClassInfo.superType );

		// --- member ---

		Assert.assertTrue(
				annotationClassInfo.fields == null || annotationClassInfo.fields.isEmpty() );

		// --- methods ---

		Assert.assertNotNull(
				annotationClassInfo.methods );

		Assert.assertFalse(
				annotationClassInfo.methods.isEmpty() );

		Assert.assertEquals(
				1 ,
				annotationClassInfo.methods.size() );

		final ImmutableArray<? extends AsmMethodInfo> methodInfos = annotationClassInfo.methods.get( "value" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodInfos.size() );

		final AsmMethodInfo methodInfo = methodInfos.get( 0 );

		Assert.assertEquals(
				//expected
				new AsmClassType( new Fqn( TestAnnotationDefDeepNestedWithDefault.DeepNestedAnnotation1.class ) ) ,
				//actual
				methodInfo.returnType );

		Assert.assertTrue(
				methodInfo.params == null || methodInfo.params.isEmpty() );

		Assert.assertTrue(
				methodInfo.annotationDefaultValue instanceof AsmAnnotationUse );

		final AsmAnnotationUse annotation = (AsmAnnotationUse) methodInfo.annotationDefaultValue;

		assertRecursiv(
				Arrays.asList(
						TestAnnotationDefDeepNestedWithDefault.DeepNestedAnnotation1.class ,
						TestAnnotationDefDeepNestedWithDefault.DeepNestedAnnotation2.class ,
						TestAnnotationDefDeepNestedWithDefault.DeepNestedAnnotation3.class ,
						TestAnnotationDefDeepNestedWithDefault.DeepNestedAnnotation4.class ,
						TestAnnotationDefDeepNestedWithDefault.DeepNestedAnnotation5.class ) ,
				annotation );
	}

	/**
	 * Rekursives Durchlaufen der geschachtelten Annotationen mit
	 * assert der Annotations-Klasse.
	 * 
	 * @param classList Liste der Annotations-Klassen
	 * @param annotation aktuelle Annotation
	 */
	private static void assertRecursiv(
			final List<? extends Class<?>> classList ,
			final AsmAnnotationUse annotation )
	{
		Assert.assertEquals(
				new AsmClassType( new Fqn( classList.get( 0 ) ) ) ,
				annotation.annotationType );

		final List<? extends Class<?>> tailList = classList.subList( 1 , classList.size() );

		if ( tailList.isEmpty() )
		{
			Assert.assertTrue(
					annotation.values == null || annotation.values.isEmpty() );
		}
		else
		{
			Assert.assertFalse(
					annotation.values.isEmpty() );

			Assert.assertEquals(
					1 ,
					annotation.values.size() );

			final AsmAnnotationUse nextAnnotation = (AsmAnnotationUse) annotation.values.get( "value" );

			assertRecursiv(
					tailList ,
					nextAnnotation );
		}
	}

}
