package de.heinerkuecker.asm_improved.test.annotation_def;

import java.io.Serializable;
import java.util.ArrayList;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotationWithClassBoundedArrayMethodWithDefault;
import de.heinerkuecker.asm_improved.type.AsmArray;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.generics.arg.AsmExtendsWildcard;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link TestAnnotationWithClassBoundedArrayMethodWithDefault}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AnnotationWithClassBoundedArrayMethodWithDefaultTest
{
	/**
	 * Test method for {@link TestAnnotationWithClassBoundedArrayMethodWithDefault}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotationClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestAnnotationWithClassBoundedArrayMethodWithDefault.class.getCanonicalName() );

		Assert.assertTrue(
				annotationClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotationClassInfo.isFinal );

		Assert.assertTrue(
				annotationClassInfo.isAbstract );

		Assert.assertTrue(
				annotationClassInfo.isStatic );

		Assert.assertTrue(
				annotationClassInfo.isInterface );

		Assert.assertFalse(
				annotationClassInfo.isEnum );

		Assert.assertTrue(
				annotationClassInfo.isAnnotation );

		// --- super class ---

		Assert.assertNull(
				annotationClassInfo.superType );

		// --- member ---

		Assert.assertTrue(
				annotationClassInfo.fields == null || annotationClassInfo.fields.isEmpty() );

		// --- methods ---

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				annotationClassInfo.methods.size() );

		final ImmutableArray<? extends AsmMethodInfo> methodInfos = annotationClassInfo.methods.get( "clazzArray" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodInfos.size() );

		final AsmMethodInfo methodInfo = methodInfos.get( 0 );

		Assert.assertEquals(
				//expected
				new AsmArray( new AsmClassType( new Fqn( Class.class ) , new AsmExtendsWildcard( new AsmClassType( new Fqn( Serializable.class ) ) ) ) ) ,
				//actual
				methodInfo.returnType );

		Assert.assertTrue(
				methodInfo.params == null || methodInfo.params.isEmpty() );

		Assert.assertArrayEquals(
				//expected
				new Object[]{ new AsmClassType( new Fqn( String.class ) ) , new AsmClassType( new Fqn( ArrayList.class ) ) } ,
				//actual
				(Object[]) methodInfo.annotationDefaultValue );
	}

}
