package de.heinerkuecker.asm_improved.test.annotation_def;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmEnumValueUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotationWithEnumArrayMethodWithDefault;
import de.heinerkuecker.asm_improved.type.AsmArray;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link TestAnnotationWithEnumArrayMethodWithDefault}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AnnotationWithEnumArrayMethodWithDefaultTest
{
	/**
	 * Test method for {@link TestAnnotationWithEnumArrayMethodWithDefault}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotationClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestAnnotationWithEnumArrayMethodWithDefault.class.getCanonicalName() );

		Assert.assertTrue(
				annotationClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotationClassInfo.isFinal );

		Assert.assertTrue(
				annotationClassInfo.isAbstract );

		Assert.assertTrue(
				annotationClassInfo.isStatic );

		Assert.assertTrue(
				annotationClassInfo.isInterface );

		Assert.assertFalse(
				annotationClassInfo.isEnum );

		Assert.assertTrue(
				annotationClassInfo.isAnnotation );

		// --- super class ---

		Assert.assertNull(
				annotationClassInfo.superType );

		// --- member ---

		Assert.assertTrue(
				annotationClassInfo.fields == null || annotationClassInfo.fields.isEmpty() );

		// --- methods ---

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				annotationClassInfo.methods.size() );

		final ImmutableArray<? extends AsmMethodInfo> methodInfos = annotationClassInfo.methods.get( "enumArray" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodInfos.size() );

		final AsmMethodInfo methodInfo = methodInfos.get( 0 );

		Assert.assertEquals(
				//expected
				new AsmArray( new AsmClassType( new Fqn( de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotationWithEnumArrayMethodWithDefault.AnnotationEnum.class ) ) ) ,
				//actual
				methodInfo.returnType );

		Assert.assertTrue(
				methodInfo.params == null || methodInfo.params.isEmpty() );

		Assert.assertArrayEquals(
				//expected
				new Object[] {
						new AsmEnumValueUse( new AsmClassType( new Fqn( de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotationWithEnumArrayMethodWithDefault.AnnotationEnum.class ) ) , "A" ) ,
						new AsmEnumValueUse( new AsmClassType( new Fqn( de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotationWithEnumArrayMethodWithDefault.AnnotationEnum.class ) ) , "B" ) ,
						new AsmEnumValueUse( new AsmClassType( new Fqn( de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotationWithEnumArrayMethodWithDefault.AnnotationEnum.class ) ) , "C" ) } ,
				//actual
				(Object[]) methodInfo.annotationDefaultValue );
	}

}
