package de.heinerkuecker.asm_improved.test.annotation_use;

import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.annotation_use.AsmEnumValueUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestAnnotationAnnotatedTarget;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link TestAnnotationAnnotatedTarget}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AnnotationAnnotatedTargetTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestAnnotationAnnotatedTarget.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertTrue(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertTrue(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertTrue(
				annotatedClassInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertFalse(
				annotatedClassInfo.annotations.isEmpty() );

		Assert.assertEquals(
				1 ,
				annotatedClassInfo.annotations.size() );

		final AsmAnnotationUse annotation = annotatedClassInfo.annotations.get( 0 );

		Assert.assertEquals(
				new AsmClassType(
						new Fqn( Target.class ) ,
						// typeArgs
						(ImmutableArray<? extends AsmType>) null ) ,
				annotation.annotationType );

		Assert.assertFalse(
				annotation.values.isEmpty() );

		Assert.assertEquals(
				1 ,
				annotation.values.size() );

		final Object annotationValue = annotation.values.get( "value" );

		Assert.assertArrayEquals(
				new Object[]{
						new AsmEnumValueUse(
								new AsmClassType(
										new Fqn( ElementType.class ) ,
										// typeArgs
										(ImmutableArray<? extends AsmType>) null ) ,
								"TYPE" ) ,
						new AsmEnumValueUse(
								new AsmClassType(
										new Fqn( ElementType.class ) ,
										// typeArgs
										(ImmutableArray<? extends AsmType>) null ) ,
								"METHOD" ) ,
						new AsmEnumValueUse(
								new AsmClassType(
										new Fqn( ElementType.class ) ,
										// typeArgs
										(ImmutableArray<? extends AsmType>) null ) ,
								"CONSTRUCTOR" ) ,
						new AsmEnumValueUse(
								new AsmClassType(
										new Fqn( ElementType.class ) ,
										// typeArgs
										(ImmutableArray<? extends AsmType>) null ) ,
								"ANNOTATION_TYPE" ) } ,
				(Object[]) annotationValue );
	}

}
