package de.heinerkuecker.asm_improved.test.annotation_use;

import java.io.File;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmFieldInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.annotation_use.AsmEnumValueUse;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.generics.arg.AsmExtendsWildcard;
import de.heinerkuecker.asm_improved.type.primitive.AsmLong;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link org.junit.Test}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class AnnotationOrgJunitTestTest
{
	/**
	 * Test method for {@link org.junit.Test}.
	 */
	@Test
	public final void testSelf()
	{
		final AsmClassInfo junitClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						this.getClass().getCanonicalName() );

		Assert.assertTrue(
				junitClassInfo.isTopLevel() );

		Assert.assertTrue(
				junitClassInfo.isFinal );

		Assert.assertFalse(
				junitClassInfo.isAbstract );

		Assert.assertTrue(
				junitClassInfo.isStatic );

		Assert.assertFalse(
				junitClassInfo.isInterface );

		Assert.assertFalse(
				junitClassInfo.isEnum );

		Assert.assertFalse(
				junitClassInfo.isAnnotation );

		// --- methods ---

		final ImmutableArray<? extends AsmMethodInfo> methodsWithNameTestSelf = junitClassInfo.methods.get( "testSelf" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodsWithNameTestSelf.size() );

		final AsmMethodInfo methodWithNameTestSelf = methodsWithNameTestSelf.get( 0 );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodWithNameTestSelf.annotations.size() );

		final AsmAnnotationUse annotation = methodWithNameTestSelf.annotations.get( 0 );

		Assert.assertEquals(
				//expected
				new Fqn( org.junit.Test.class ) ,
				//actual
				annotation.annotationType.fqn );

		Assert.assertTrue(
				annotation.values == null || annotation.values.isEmpty() );
	}

	@Ignore
	@Test( expected = UnsupportedOperationException.class )
	public void testMethodWithExceptionAndIgnore()
	{
		//Kein Code erforderlich, diese Methode soll nie durchlaufen werden
	}

	/**
	 * Test method for {@link org.junit.Test}.
	 */
	@Test
	public final void test_testMethodWithExceptionAndIgnore()
	{
		final AsmClassInfo junitClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						this.getClass().getCanonicalName() );

		Assert.assertTrue(
				junitClassInfo.isTopLevel() );

		Assert.assertTrue(
				junitClassInfo.isFinal );

		Assert.assertFalse(
				junitClassInfo.isAbstract );

		Assert.assertTrue(
				junitClassInfo.isStatic );

		Assert.assertFalse(
				junitClassInfo.isInterface );

		Assert.assertFalse(
				junitClassInfo.isEnum );

		Assert.assertFalse(
				junitClassInfo.isAnnotation );

		// --- methods ---

		final ImmutableArray<? extends AsmMethodInfo> testMethodsWithExceptionAndIgnore = junitClassInfo.methods.get( "testMethodWithExceptionAndIgnore" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				testMethodsWithExceptionAndIgnore.size() );

		final AsmMethodInfo testMethodWithExceptionAndIgnore = testMethodsWithExceptionAndIgnore.get( 0 );

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				testMethodWithExceptionAndIgnore.annotations.size() );

		{
			final AsmAnnotationUse annotation0 = testMethodWithExceptionAndIgnore.annotations.get( 0 );

			Assert.assertEquals(
					//expected
					new Fqn( org.junit.Ignore.class ) ,
					//actual
					annotation0.annotationType.fqn );

			Assert.assertTrue(
					annotation0.values == null || annotation0.values.isEmpty() );
		}

		{
			final AsmAnnotationUse annotation1 = testMethodWithExceptionAndIgnore.annotations.get( 1 );

			Assert.assertEquals(
					//expected
					new Fqn( org.junit.Test.class ) ,
					//actual
					annotation1.annotationType.fqn );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					annotation1.values.size() );

			final Object annotation1Value = annotation1.values.get( "expected" );

			Assert.assertEquals(
					//expected
					new Fqn( UnsupportedOperationException.class ) ,
					//actual
					( (AsmClassType) annotation1Value ).fqn );
		}
	}

	/**
	 * Test method for {@link org.junit.Test}.
	 */
	@Test
	public final void testAnnotationOrgJunitTest()
	{
		final AsmClassInfo annotationClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						org.junit.Test.class.getCanonicalName() );

		Assert.assertTrue(
				annotationClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotationClassInfo.isFinal );

		Assert.assertTrue(
				annotationClassInfo.isAbstract );

		Assert.assertTrue(
				annotationClassInfo.isStatic );

		Assert.assertTrue(
				annotationClassInfo.isInterface );

		Assert.assertFalse(
				annotationClassInfo.isEnum );

		Assert.assertTrue(
				annotationClassInfo.isAnnotation );

		// --- super class ---

		Assert.assertNull(
				annotationClassInfo.superType );

		// --- meta annotations ---

		Assert.assertFalse(
				annotationClassInfo.annotations == null || annotationClassInfo.annotations.isEmpty() );

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				annotationClassInfo.annotations.size() );

		{
			final AsmAnnotationUse metaAnnotation0 = annotationClassInfo.annotations.get( 0 );

			Assert.assertEquals(
					//expected
					new AsmClassType( new Fqn( Retention.class ) ) ,
					//actual
					metaAnnotation0.annotationType );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					metaAnnotation0.values.size() );

			final Object metaAnnotation0Value = metaAnnotation0.values.get( "value" );

			Assert.assertEquals(
					//expected
					new AsmEnumValueUse( new AsmClassType( new Fqn( RetentionPolicy.class ) ) , RetentionPolicy.RUNTIME.name() ) ,
					//actual
					metaAnnotation0Value );
		}

		{
			final AsmAnnotationUse metaAnnotation1 = annotationClassInfo.annotations.get( 1 );

			Assert.assertEquals(
					//expected
					new AsmClassType( new Fqn( Target.class ) ) ,
					//actual
					metaAnnotation1.annotationType );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					metaAnnotation1.values.size() );

			final Object metaAnnotation0Value = metaAnnotation1.values.get( "value" );

			Assert.assertArrayEquals(
					//expected
					new Object[]{ new AsmEnumValueUse( new AsmClassType( new Fqn( ElementType.class ) ) , ElementType.METHOD.name() ) } ,
					//actual
					(Object[]) metaAnnotation0Value );
		}


		// --- fields ---

		Assert.assertTrue(
				annotationClassInfo.fields == null || annotationClassInfo.fields.isEmpty() );

		// --- methods ---

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				annotationClassInfo.methods.size() );

		{
			final ImmutableArray<? extends AsmMethodInfo> methodsWithNameExpected = annotationClassInfo.methods.get( "expected" );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					methodsWithNameExpected.size() );

			final AsmMethodInfo methodWithNameExpected = methodsWithNameExpected.get( 0 );

			Assert.assertTrue(
					methodWithNameExpected.params == null || methodWithNameExpected.params.isEmpty() );

			// TODO check keine Type-Parameter
			// TODO check keine Annotationen

			Assert.assertTrue(
					methodWithNameExpected.isAbstract );

			Assert.assertTrue(
					methodWithNameExpected.isPublic );

			Assert.assertEquals(
					//expected
					new AsmClassType(
							new Fqn( Class.class ) ,
							new AsmExtendsWildcard(
									new AsmClassType( new Fqn( Throwable.class ) ) ) ) ,
					//actual
					methodWithNameExpected.returnType );

			Assert.assertEquals(
					//expected
					new AsmClassType( new Fqn( org.junit.Test.None.class ) ) ,
					//actual
					methodWithNameExpected.annotationDefaultValue );
		}

		{
			final ImmutableArray<? extends AsmMethodInfo> methodsWithNameTimeout = annotationClassInfo.methods.get( "timeout" );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					methodsWithNameTimeout.size() );

			final AsmMethodInfo methodWithNameTimeout = methodsWithNameTimeout.get( 0 );

			Assert.assertTrue(
					methodWithNameTimeout.params == null || methodWithNameTimeout.params.isEmpty() );

			// TODO check keine Type-Parameter
			// TODO check keine Annotationen

			Assert.assertTrue(
					methodWithNameTimeout.isAbstract );

			Assert.assertTrue(
					methodWithNameTimeout.isPublic );

			Assert.assertEquals(
					//expected
					AsmLong.INSTANCE ,
					//actual
					methodWithNameTimeout.returnType );

			Assert.assertEquals(
					//expected
					0L ,
					//actual
					( (Long) methodWithNameTimeout.annotationDefaultValue ).longValue() );
		}
	}

/* Sourcecode from org.junit.Test
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.METHOD})
public @interface Test {

    static class None extends Throwable {
        private static final long serialVersionUID = 1L;

        private None() {
        }
    }

    Class<? extends Throwable> expected() default None.class;

    long timeout() default 0L;
}
*/

	/**
	 * Test method for {@link org.junit.Test.None}.
	 */
	@Test
	@SuppressWarnings( "static-method" )
	public final void testAnnotationInnerClassOrgJunitTestNone()
	{
		final AsmClassInfo noneClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						org.junit.Test.class.getCanonicalName() + ".None" );

		Assert.assertFalse(
				noneClassInfo.isTopLevel() );

		Assert.assertFalse(
				noneClassInfo.isFinal );

		Assert.assertFalse(
				noneClassInfo.isAbstract );

		Assert.assertTrue(
				noneClassInfo.isStatic );

		Assert.assertFalse(
				noneClassInfo.isInterface );

		Assert.assertFalse(
				noneClassInfo.isEnum );

		Assert.assertFalse(
				noneClassInfo.isAnnotation );

		// --- super class ---

		Assert.assertEquals(
				//expected
				new AsmClassType(
						new Fqn( Throwable.class ) ) ,
				//actual
				noneClassInfo.superType );

		// --- fields ---

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				noneClassInfo.fields.size() );

		final AsmFieldInfo field = noneClassInfo.fields.get( "serialVersionUID" );

		Assert.assertEquals(
				//expected
				"serialVersionUID" ,
				//actual
				field.name );

		Assert.assertEquals(
				//expected
				Long.valueOf( 1 ) ,
				//actual
				field.initialValue );

		// --- methods ---

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				noneClassInfo.methods.size() );

		final ImmutableArray<? extends AsmMethodInfo> constructors = noneClassInfo.methods.get( AsmMethodInfo.CONSTRUCTOR_METHOD_NAME );
		
		Assert.assertEquals(
				//expected
				1 ,
				//actual
				constructors.size() );

		final AsmMethodInfo constructor = constructors.get( 0 );
		
		
		Assert.assertTrue(
				constructor.params == null || constructor.params.isEmpty() );

		// TODO check keine Annotationen
		// TODO check return type
	}

}
