package de.heinerkuecker.asm_improved.test.annotation_use;

import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.annotation_use.AsmEnumValueUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestClassAnnotationUseDeepNestedFinallyEnum;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestClassAnnotationUseValues1Enum;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link TestClassAnnotationUseDeepNestedFinallyEnum}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class ClassAnnotationUseDeepNestedFinallyEnumTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestClassAnnotationUseDeepNestedFinallyEnum.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertTrue(
				annotatedClassInfo.isFinal );

		Assert.assertFalse(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertFalse(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertFalse(
				annotatedClassInfo.annotations.isEmpty() );

		Assert.assertEquals(
				1 ,
				annotatedClassInfo.annotations.size() );

		final AsmAnnotationUse annotation = annotatedClassInfo.annotations.get( 0 );

		assertRecursiv(
				Arrays.asList(
						TestClassAnnotationUseDeepNestedFinallyEnum.DeepNestedAnnotation0.class ,
						TestClassAnnotationUseDeepNestedFinallyEnum.DeepNestedAnnotation1.class ,
						TestClassAnnotationUseDeepNestedFinallyEnum.DeepNestedAnnotation2.class ,
						TestClassAnnotationUseDeepNestedFinallyEnum.DeepNestedAnnotation3.class ,
						TestClassAnnotationUseDeepNestedFinallyEnum.DeepNestedAnnotation4.class ,
						TestClassAnnotationUseDeepNestedFinallyEnum.DeepNestedAnnotation5.class ) ,
				annotation );
	}

	/**
	 * Rekursives Durchlaufen der geschachtelten Annotationen mit
	 * assert der Annotations-Klasse.
	 * 
	 * @param classList Liste der Annotations-Klassen
	 * @param annotation aktuelle Annotation
	 */
	private static void assertRecursiv(
			final List<? extends Class<?>> classList ,
			final AsmAnnotationUse annotation )
	{
		Assert.assertEquals(
				new AsmClassType( new Fqn( classList.get( 0 ) ) ) ,
				annotation.annotationType );

		final List<? extends Class<?>> tailList = classList.subList( 1 , classList.size() );

		if ( tailList.isEmpty() )
		{
			Assert.assertFalse(
					annotation.values == null || annotation.values.isEmpty() );

			Assert.assertEquals(
					1 ,
					annotation.values.size() );

			final AsmEnumValueUse enumValue = (AsmEnumValueUse) annotation.values.get( "value" );
			
			Assert.assertNotNull(
					enumValue );

			Assert.assertEquals(
					new AsmClassType( new Fqn( TestClassAnnotationUseValues1Enum.AnnotationEnum.class ) ) ,
					enumValue.enumType );

			Assert.assertEquals(
					"A" ,
					enumValue.enumValueName );
		}
		else
		{
			Assert.assertFalse(
					annotation.values.isEmpty() );

			Assert.assertEquals(
					1 ,
					annotation.values.size() );

			final AsmAnnotationUse nextAnnotation = (AsmAnnotationUse) annotation.values.get( "value" );

			assertRecursiv(
					tailList ,
					nextAnnotation );
		}
	}

}
