package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestClassAnnotationUseValues1BooleanDefault;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.primitive.AsmBoolean;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link TestClassAnnotationUseValues1BooleanDefault}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class ClassAnnotationUseValues1BooleanDefaultTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestClassAnnotationUseValues1BooleanDefault.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertFalse(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertFalse(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertFalse(
				annotatedClassInfo.annotations.isEmpty() );

		Assert.assertEquals(
				1 ,
				annotatedClassInfo.annotations.size() );

		final AsmAnnotationUse annotation = annotatedClassInfo.annotations.get( 0 );

		Assert.assertEquals(
				new AsmClassType( new Fqn( TestClassAnnotationUseValues1BooleanDefault.AnnotationValues1Boolean.class ) ) ,
				annotation.annotationType );

		// verwendeter Default-Value nicht vorhanden
		Assert.assertTrue(
				annotation.values.isEmpty() );

		final AsmClassInfo annotationClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						annotation.annotationType.fqn );

		Assert.assertFalse(
				annotationClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotationClassInfo.isFinal );

		Assert.assertTrue(
				annotationClassInfo.isAbstract );

		Assert.assertTrue(
				annotationClassInfo.isStatic );

		Assert.assertTrue(
				annotationClassInfo.isInterface );

		Assert.assertFalse(
				annotationClassInfo.isEnum );

		Assert.assertTrue(
				annotationClassInfo.isAnnotation );

		// --- super class ---

		Assert.assertNull(
				annotationClassInfo.superType );

		// --- member ---

		Assert.assertTrue(
				annotationClassInfo.fields == null || annotationClassInfo.fields.isEmpty() );

		// --- methods ---

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				annotationClassInfo.methods.size() );

		final ImmutableArray<? extends AsmMethodInfo> methodInfos = annotationClassInfo.methods.get( "value" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodInfos.size() );

		final AsmMethodInfo methodInfo = methodInfos.get( 0 );

		Assert.assertEquals(
				//expected
				AsmBoolean.INSTANCE ,
				//actual
				methodInfo.returnType );

		Assert.assertTrue(
				methodInfo.params == null || methodInfo.params.isEmpty() );

		Assert.assertEquals(
				//expected
				false ,
				//actual
				( (Boolean) methodInfo.annotationDefaultValue ).booleanValue() );
		
		// Bequemlichkeits-Methode
		Assert.assertEquals(
				//expected
				false ,
				//actual
				( (Boolean) annotation.getValueUnderConsiderationDefault(
						AllAsmTests.asmClasspathInfo ,
						"value" ) ).booleanValue() );
	}

}
