package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.AsmMethodParamInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestMethodParamAnnotation;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link TestMethodParamAnnotation}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class MethodParamAnnotationUseTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestMethodParamAnnotation.class.getCanonicalName() );

		final ImmutableArray<? extends AsmMethodInfo> methods = annotatedClassInfo.methods.get( "method" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methods.size() );

		final AsmMethodInfo method = methods.get( 0 );

		Assert.assertEquals(
				//expected
				1 , 
				//actual
				method.params.size() );
		
		final AsmMethodParamInfo methodParam = method.params.get( 0 );

		Assert.assertTrue(
				methodParam.annotations == null || methodParam.annotations.isEmpty() );

		//Assert.assertEquals(
		//		//expected
		//		1 , 
		//		//actual
		//		methodParam.annotations.size() );
		//
		//final AsmAnnotationUse annotation = methodParam.annotations.get( 0 );
		//
		//Assert.assertEquals(
		//		//expected
		//		new Fqn( NotNull.class ) , 
		//		//actual
		//		annotation.annotationType.fqn );
		//
		//Assert.assertTrue(
		//		annotation.annotationType.typeArgs == null || annotation.annotationType.typeArgs.size() == 0 );
		//
		//Assert.assertTrue(
		//		annotation.values == null || annotation.values.size() == 0 );

		{
			final AsmType methodParamType = methodParam.type;

			Assert.assertEquals(
					//expected
					1 , 
					//actual
					methodParamType.typeAnnotations.size() );

			final AsmAnnotationUse typeAnnotation = methodParamType.typeAnnotations.get( 0 );

			Assert.assertEquals(
					//expected
					new Fqn( NotNull.class ) , 
					//actual
					typeAnnotation.annotationType.fqn );

			Assert.assertTrue(
					typeAnnotation.annotationType.typeArgs == null || typeAnnotation.annotationType.typeArgs.size() == 0 );

			Assert.assertTrue(
					typeAnnotation.values == null || typeAnnotation.values.size() == 0 );
		}


		//Assert.fail( "nicht fertig" );
	}

}
