package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.AsmMethodParamInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestMethodParamAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestMethodReturnAndParamAnnotation;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link TestMethodParamAnnotation}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class MethodReturnAndParamAnnotationUseTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestMethodReturnAndParamAnnotation.class.getCanonicalName() );

		final ImmutableArray<? extends AsmMethodInfo> methods = annotatedClassInfo.methods.get( "method" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methods.size() );

		final AsmMethodInfo method = methods.get( 0 );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				method.annotations.size() );

		{
			final AsmAnnotationUse annotation = method.annotations.get( 0 );

			Assert.assertEquals(
					//expected
					new AsmClassType( new Fqn( de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
					//actual
					annotation.annotationType );
		}

		{

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					method.params.size() );

			final AsmMethodParamInfo param = method.params.get( 0 );

			Assert.assertEquals(
					//expected
					1 ,
					//actual
					param.annotations.size() );
			{
				final AsmAnnotationUse paramAnnotation = param.annotations.get( 0 );

				Assert.assertEquals(
						//expected
						new AsmClassType( new Fqn( de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
						//actual
						paramAnnotation.annotationType );
			}
		}

		//Assert.fail( "nicht fertig" );
	}

}
