package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmFieldInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestTypeAnnotationOnFieldArray0;
import de.heinerkuecker.asm_improved.type.AsmArray;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link TestTypeAnnotationOnFieldArray0}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class TypeAnnotationOnFieldArray0Test
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestTypeAnnotationOnFieldArray0.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertFalse(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertFalse(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertTrue(
				annotatedClassInfo.annotations != null || annotatedClassInfo.annotations.isEmpty() );

		Assert.assertEquals(
				// expected
				2 ,
				//actual
				annotatedClassInfo.fields.size() );

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArr0" );

			final AsmType fieldType = field.type;

			Assert.assertTrue(
					fieldType.typeAnnotations == null || fieldType.typeAnnotations.isEmpty() );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

			Assert.assertEquals(
					// expected
					Fqn.JAVA_LANG_STRING_FQN ,
					//actual
					( (AsmClassType) arrElementType ).fqn );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					arrElementType.typeAnnotations.size() );

			final AsmAnnotationUse arrElementTypeTypeAnnotation = arrElementType.typeAnnotations.get( 0 );

			Assert.assertEquals(
					// expected
					new AsmClassType(
							new Fqn(
									de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
					//actual
					arrElementTypeTypeAnnotation.annotationType );
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArr1" );

			final AsmType fieldType = field.type;

			Assert.assertTrue(
					fieldType.typeAnnotations == null || fieldType.typeAnnotations.isEmpty() );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

			Assert.assertEquals(
					// expected
					Fqn.JAVA_LANG_STRING_FQN ,
					//actual
					( (AsmClassType) arrElementType ).fqn );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					arrElementType.typeAnnotations.size() );

			final AsmAnnotationUse arrElementTypeTypeAnnotation = arrElementType.typeAnnotations.get( 0 );

			Assert.assertEquals(
					// expected
					new AsmClassType(
							new Fqn(
									de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
					//actual
					arrElementTypeTypeAnnotation.annotationType );
		}
	}

}
