package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmFieldInfo;
import de.heinerkuecker.asm_improved.AsmTestUtil;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestTypeAnnotationOnFieldArray1;
import de.heinerkuecker.asm_improved.type.AsmArray;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link TestTypeAnnotationOnFieldArray1}.
 * 
 * Ziemliche seltsame Spezifikation:
 * <a href="https://docs.oracle.com/javase/specs/jls/se8/html/jls-9.html#jls-9.7.4">Java Language Specification - 9.7.4. Where Annotations May Appear </a>
 * in "@C int @A [] @B [] f; @A applies to the array type
 * int[][], @B applies to its component type int[], and @C
 * applies to the element type int."
 * 
 * @author Heiner K&uuml;cker
 */
public final class TypeAnnotationOnFieldArray1Test
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestTypeAnnotationOnFieldArray1.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertFalse(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertFalse(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArr0" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertTrue(
					fieldType.typeAnnotations == null || fieldType.typeAnnotations.isEmpty() );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					arrElementType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					arrElementType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class );

			Assert.assertEquals(
					// expected
					Fqn.JAVA_LANG_STRING_FQN ,
					//actual
					( (AsmClassType) arrElementType ).fqn );
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArr1" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

			Assert.assertEquals(
					// expected
					Fqn.JAVA_LANG_STRING_FQN ,
					//actual
					( (AsmClassType) arrElementType ).fqn );

			Assert.assertTrue(
					arrElementType.typeAnnotations == null || arrElementType.typeAnnotations.isEmpty() );
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArr2" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

			Assert.assertEquals(
					// expected
					Fqn.JAVA_LANG_STRING_FQN ,
					//actual
					( (AsmClassType) arrElementType ).fqn );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					arrElementType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					arrElementType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class ) ;
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArrArr" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			{
				final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

				Assert.assertTrue(
						arrElementType instanceof AsmArray );

				Assert.assertEquals(
						// expected
						1 ,
						//actual
						arrElementType.typeAnnotations.size() );

				AsmTestUtil.assertAnnotationClassFqn(
						arrElementType.typeAnnotations.get( 0 ) ,
						de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno2.class );

				{
					final AsmType arrElementElementType = ( (AsmArray) arrElementType ).elementType;

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							arrElementElementType.typeAnnotations.size() );

					final AsmAnnotationUse arrElementElementTypeTypeAnnotation = arrElementElementType.typeAnnotations.get( 0 );

					Assert.assertEquals(
							// expected
							new AsmClassType(
									new Fqn(
											de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class ) ) ,
							//actual
							arrElementElementTypeTypeAnnotation.annotationType );

					Assert.assertEquals(
							// expected
							Fqn.JAVA_LANG_STRING_FQN ,
							//actual
							( (AsmClassType) arrElementElementType ).fqn );
				}
			}
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArrArrArr" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			{
				final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

				Assert.assertTrue(
						arrElementType instanceof AsmArray );

				Assert.assertEquals(
						// expected
						1 ,
						//actual
						arrElementType.typeAnnotations.size() );

				AsmTestUtil.assertAnnotationClassFqn(
						arrElementType.typeAnnotations.get( 0 ) ,
						de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno2.class );

				{
					final AsmType arrElementElementType = ( (AsmArray) arrElementType ).elementType;

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							arrElementElementType.typeAnnotations.size() );

					AsmTestUtil.assertAnnotationClassFqn(
							arrElementElementType.typeAnnotations.get( 0 ) ,
							de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno3.class );

					{
						final AsmType arrElementElementElementType = ( (AsmArray) arrElementElementType ).elementType;

						Assert.assertEquals(
								// expected
								1 ,
								//actual
								arrElementElementElementType.typeAnnotations.size() );

						AsmTestUtil.assertAnnotationClassFqn(
								arrElementElementElementType.typeAnnotations.get( 0 ) ,
								de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class );

						Assert.assertEquals(
								// expected
								Fqn.JAVA_LANG_STRING_FQN ,
								//actual
								( (AsmClassType) arrElementElementElementType ).fqn );
					}
				}
			}
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArrArrArrList" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class );

			Assert.assertTrue(
					fieldType instanceof AsmClassType );

			AsmClassType fieldClassType = (AsmClassType) fieldType;

			Assert.assertEquals(
					// expected
					new Fqn( java.util.List.class ) ,
					//actual
					fieldClassType.fqn );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldClassType.typeArgs.size() );

			{
				final AsmType fieldTypeArg = fieldClassType.typeArgs.get( 0 );

				Assert.assertTrue(
						fieldTypeArg instanceof AsmArray );

				{
					final AsmArray fieldTypeArgArray = (AsmArray) fieldTypeArg;

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							fieldTypeArgArray.typeAnnotations.size() );

					AsmTestUtil.assertAnnotationClassFqn(
							fieldTypeArgArray.typeAnnotations.get( 0 ) ,
							de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno2.class );

					Assert.assertTrue(
							fieldTypeArgArray.elementType instanceof AsmArray );

					{
						final AsmArray fieldTypeArgSubArray = (AsmArray) fieldTypeArgArray.elementType;

						Assert.assertEquals(
								// expected
								1 ,
								//actual
								fieldTypeArgSubArray.typeAnnotations.size() );

						AsmTestUtil.assertAnnotationClassFqn(
								fieldTypeArgSubArray.typeAnnotations.get( 0 ) ,
								de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno3.class );

						Assert.assertTrue(
								fieldTypeArgSubArray.elementType instanceof AsmArray );

						{
							final AsmArray fieldTypeArgSubSubArray = (AsmArray) fieldTypeArgSubArray.elementType;

							Assert.assertEquals(
									// expected
									1 ,
									//actual
									fieldTypeArgSubSubArray.typeAnnotations.size() );

							AsmTestUtil.assertAnnotationClassFqn(
									fieldTypeArgSubSubArray.typeAnnotations.get( 0 ) ,
									de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno4.class );

							Assert.assertEquals(
									// expected
									Fqn.JAVA_LANG_STRING_FQN ,
									//actual
									( (AsmClassType) fieldTypeArgSubSubArray.elementType ).fqn );

							{
								final AsmType arrElementFieldTypeArgSubSubArrayType = ( (AsmArray) fieldTypeArgSubSubArray ).elementType;

								Assert.assertEquals(
										// expected
										1 ,
										//actual
										arrElementFieldTypeArgSubSubArrayType.typeAnnotations.size() );

								AsmTestUtil.assertAnnotationClassFqn(
										arrElementFieldTypeArgSubSubArrayType.typeAnnotations.get( 0 ) ,
										de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

								Assert.assertEquals(
										// expected
										Fqn.JAVA_LANG_STRING_FQN ,
										//actual
										( (AsmClassType) arrElementFieldTypeArgSubSubArrayType ).fqn );
							}
						}
					}
				}
			}
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArrArrArrMap" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class );

			Assert.assertTrue(
					fieldType instanceof AsmClassType );

			AsmClassType fieldClassType = (AsmClassType) fieldType;

			Assert.assertEquals(
					// expected
					new Fqn( java.util.Map.class ) ,
					//actual
					fieldClassType.fqn );

			Assert.assertEquals(
					// expected
					2 ,
					//actual
					fieldClassType.typeArgs.size() );

			{
				final AsmType fieldTypeArg = fieldClassType.typeArgs.get( 0 );

				Assert.assertTrue(
						fieldTypeArg instanceof AsmArray );

				{
					final AsmArray fieldTypeArgArray = (AsmArray) fieldTypeArg;

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							fieldTypeArgArray.typeAnnotations.size() );

					AsmTestUtil.assertAnnotationClassFqn(
							fieldTypeArgArray.typeAnnotations.get( 0 ) ,
							de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno2.class );

					Assert.assertTrue(
							fieldTypeArgArray.elementType instanceof AsmArray );

					{
						final AsmArray fieldTypeArgSubArray = (AsmArray) fieldTypeArgArray.elementType;

						Assert.assertEquals(
								// expected
								1 ,
								//actual
								fieldTypeArgSubArray.typeAnnotations.size() );

						AsmTestUtil.assertAnnotationClassFqn(
								fieldTypeArgSubArray.typeAnnotations.get( 0 ) ,
								de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno3.class );

						Assert.assertTrue(
								fieldTypeArgSubArray.elementType instanceof AsmArray );

						{
							final AsmArray fieldTypeArgSubSubArray = (AsmArray) fieldTypeArgSubArray.elementType;

							Assert.assertEquals(
									// expected
									1 ,
									//actual
									fieldTypeArgSubSubArray.typeAnnotations.size() );

							AsmTestUtil.assertAnnotationClassFqn(
									fieldTypeArgSubSubArray.typeAnnotations.get( 0 ) ,
									de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno4.class );

							Assert.assertEquals(
									// expected
									Fqn.JAVA_LANG_STRING_FQN ,
									//actual
									( (AsmClassType) fieldTypeArgSubSubArray.elementType ).fqn );

							{
								final AsmType arrElementFieldTypeArgSubSubArrayType = ( (AsmArray) fieldTypeArgSubSubArray ).elementType;

								Assert.assertEquals(
										// expected
										1 ,
										//actual
										arrElementFieldTypeArgSubSubArrayType.typeAnnotations.size() );

								AsmTestUtil.assertAnnotationClassFqn(
										arrElementFieldTypeArgSubSubArrayType.typeAnnotations.get( 0 ) ,
										de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

								Assert.assertEquals(
										// expected
										Fqn.JAVA_LANG_STRING_FQN ,
										//actual
										( (AsmClassType) arrElementFieldTypeArgSubSubArrayType ).fqn );
							}
						}
					}
				}
			}

			{
				final AsmType fieldTypeArg = fieldClassType.typeArgs.get( 1 );

				Assert.assertTrue(
						fieldTypeArg instanceof AsmArray );

				{
					final AsmArray fieldTypeArgArray = (AsmArray) fieldTypeArg;

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							fieldTypeArgArray.typeAnnotations.size() );

					AsmTestUtil.assertAnnotationClassFqn(
							fieldTypeArgArray.typeAnnotations.get( 0 ) ,
							de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno2.class );

					Assert.assertTrue(
							fieldTypeArgArray.elementType instanceof AsmArray );

					{
						final AsmArray fieldTypeArgSubArray = (AsmArray) fieldTypeArgArray.elementType;

						Assert.assertEquals(
								// expected
								1 ,
								//actual
								fieldTypeArgSubArray.typeAnnotations.size() );

						AsmTestUtil.assertAnnotationClassFqn(
								fieldTypeArgSubArray.typeAnnotations.get( 0 ) ,
								de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno3.class );

						Assert.assertTrue(
								fieldTypeArgSubArray.elementType instanceof AsmArray );

						{
							final AsmArray fieldTypeArgSubSubArray = (AsmArray) fieldTypeArgSubArray.elementType;

							Assert.assertEquals(
									// expected
									1 ,
									//actual
									fieldTypeArgSubSubArray.typeAnnotations.size() );

							AsmTestUtil.assertAnnotationClassFqn(
									fieldTypeArgSubSubArray.typeAnnotations.get( 0 ) ,
									de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno4.class );

							Assert.assertEquals(
									// expected
									Fqn.JAVA_LANG_STRING_FQN ,
									//actual
									( (AsmClassType) fieldTypeArgSubSubArray.elementType ).fqn );

							{
								final AsmType arrElementFieldTypeArgSubSubArrayType = ( (AsmArray) fieldTypeArgSubSubArray ).elementType;

								Assert.assertEquals(
										// expected
										1 ,
										//actual
										arrElementFieldTypeArgSubSubArrayType.typeAnnotations.size() );

								AsmTestUtil.assertAnnotationClassFqn(
										arrElementFieldTypeArgSubSubArrayType.typeAnnotations.get( 0 ) ,
										de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

								Assert.assertEquals(
										// expected
										Fqn.JAVA_LANG_STRING_FQN ,
										//actual
										( (AsmClassType) arrElementFieldTypeArgSubSubArrayType ).fqn );
							}
						}
					}
				}
			}
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArrArrArrListArr" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

			Assert.assertEquals(
					// expected
					new Fqn( java.util.List.class ) ,
					//actual
					( (AsmClassType) arrElementType ).fqn );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					( (AsmClassType) arrElementType ).typeArgs.size() );

			{
				final AsmType fieldTypeArg = ( (AsmClassType) arrElementType ).typeArgs.get( 0 );

				assertArrayString_TypeAnno_2_to_5( fieldTypeArg );
			}
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArrArrArrListArrArr" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			{
				final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

				Assert.assertTrue(
						arrElementType instanceof AsmArray );

				Assert.assertEquals(
						// expected
						1 ,
						//actual
						arrElementType.typeAnnotations.size() );

				AsmTestUtil.assertAnnotationClassFqn(
						arrElementType.typeAnnotations.get( 0 ) ,
						de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno2.class );

				{
					final AsmType arrArrElementType = ( (AsmArray) arrElementType ).elementType;

					Assert.assertEquals(
							// expected
							new Fqn( java.util.List.class ) ,
							//actual
							( (AsmClassType) arrArrElementType ).fqn );

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							arrArrElementType.typeAnnotations.size() );

					AsmTestUtil.assertAnnotationClassFqn(
							arrArrElementType.typeAnnotations.get( 0 ) ,
							de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class );

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							( (AsmClassType) arrArrElementType ).typeArgs.size() );

					final AsmType typeArg = ( (AsmClassType) arrArrElementType ).typeArgs.get( 0 );

					assertArrayString_TypeAnno_3_to_6( typeArg );
				}
			}
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArrArrArrMapArr" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			{
				final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

				Assert.assertEquals(
						// expected
						new Fqn( java.util.Map.class ) ,
						//actual
						( (AsmClassType) arrElementType ).fqn );

				Assert.assertEquals(
						// expected
						2 ,
						//actual
						( (AsmClassType) arrElementType ).typeArgs.size() );

				{
					final AsmType fieldTypeArg = ( (AsmClassType) arrElementType ).typeArgs.get( 0 );

					assertArrayString_TypeAnno_2_to_5( fieldTypeArg );
				}

				{
					final AsmType fieldTypeArg = ( (AsmClassType) arrElementType ).typeArgs.get( 1 );

					assertArrayString_TypeAnno_2_to_5( fieldTypeArg );
				}
			}
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strArrArrArrMapArrArr" );

			System.out.println( field );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldType.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class );

			Assert.assertTrue(
					fieldType instanceof AsmArray );

			{
				final AsmType arrElementType = ( (AsmArray) fieldType ).elementType;

				Assert.assertTrue(
						arrElementType instanceof AsmArray );

				Assert.assertEquals(
						// expected
						1 ,
						//actual
						arrElementType.typeAnnotations.size() );

				AsmTestUtil.assertAnnotationClassFqn(
						arrElementType.typeAnnotations.get( 0 ) ,
						de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno2.class );

				{
					final AsmType arrArrElementType = ( (AsmArray) arrElementType ).elementType;

					Assert.assertEquals(
							// expected
							new Fqn( java.util.Map.class ) ,
							//actual
							( (AsmClassType) arrArrElementType ).fqn );

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							arrArrElementType.typeAnnotations.size() );

					AsmTestUtil.assertAnnotationClassFqn(
							arrArrElementType.typeAnnotations.get( 0 ) ,
							de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class );

					Assert.assertEquals(
							// expected
							2 ,
							//actual
							( (AsmClassType) arrArrElementType ).typeArgs.size() );

					{
						final AsmType typeArg = ( (AsmClassType) arrArrElementType ).typeArgs.get( 0 );

						assertArrayString_TypeAnno_3_to_6( typeArg );
					}

					{
						final AsmType typeArg = ( (AsmClassType) arrArrElementType ).typeArgs.get( 1 );

						assertArrayString_TypeAnno_3_to_6( typeArg );
					}
				}
			}
		}

		Assert.assertEquals(
				// expected
				11 ,
				//actual
				annotatedClassInfo.fields.size() );
		//Assert.fail( "nicht fertig" );
	}
	/**
	 * @param arrayType type to assert
	 */
	private void assertArrayString_TypeAnno_2_to_5(
			final AsmType arrayType )
	{
		Assert.assertTrue(
				arrayType instanceof AsmArray );

		{
			final AsmArray fieldTypeArgArray = (AsmArray) arrayType;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldTypeArgArray.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					fieldTypeArgArray.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno3.class );

			Assert.assertTrue(
					fieldTypeArgArray.elementType instanceof AsmArray );

			{
				final AsmArray fieldTypeArgSubArray = (AsmArray) fieldTypeArgArray.elementType;

				Assert.assertEquals(
						// expected
						1 ,
						//actual
						fieldTypeArgSubArray.typeAnnotations.size() );

				AsmTestUtil.assertAnnotationClassFqn(
						fieldTypeArgSubArray.typeAnnotations.get( 0 ) ,
						de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno4.class );

				Assert.assertTrue(
						fieldTypeArgSubArray.elementType instanceof AsmArray );

				{
					final AsmArray fieldTypeArgSubSubArray = (AsmArray) fieldTypeArgSubArray.elementType;

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							fieldTypeArgSubSubArray.typeAnnotations.size() );

					AsmTestUtil.assertAnnotationClassFqn(
							fieldTypeArgSubSubArray.typeAnnotations.get( 0 ) ,
							de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno5.class );

					Assert.assertEquals(
							// expected
							Fqn.JAVA_LANG_STRING_FQN ,
							//actual
							( (AsmClassType) fieldTypeArgSubSubArray.elementType ).fqn );

					{
						final AsmType arrElementFieldTypeArgSubSubArrayType = ( (AsmArray) fieldTypeArgSubSubArray ).elementType;

						Assert.assertEquals(
								// expected
								1 ,
								//actual
								arrElementFieldTypeArgSubSubArrayType.typeAnnotations.size() );

						AsmTestUtil.assertAnnotationClassFqn(
								arrElementFieldTypeArgSubSubArrayType.typeAnnotations.get( 0 ) ,
								de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno2.class );

						Assert.assertEquals(
								// expected
								Fqn.JAVA_LANG_STRING_FQN ,
								//actual
								( (AsmClassType) arrElementFieldTypeArgSubSubArrayType ).fqn );
					}
				}
			}
		}
	}

	/**
	 * @param arrayType type to assert
	 */
	private void assertArrayString_TypeAnno_3_to_6(
			final AsmType arrayType )
	{
		{
			Assert.assertTrue(
					arrayType instanceof AsmArray );

			final AsmArray TypeArgArray = (AsmArray) arrayType;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					TypeArgArray.typeAnnotations.size() );

			AsmTestUtil.assertAnnotationClassFqn(
					TypeArgArray.typeAnnotations.get( 0 ) ,
					de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno4.class );

			Assert.assertTrue(
					TypeArgArray.elementType instanceof AsmArray );

			{
				final AsmArray TypeArgSubArray = (AsmArray) TypeArgArray.elementType;

				Assert.assertEquals(
						// expected
						1 ,
						//actual
						TypeArgSubArray.typeAnnotations.size() );

				AsmTestUtil.assertAnnotationClassFqn(
						TypeArgSubArray.typeAnnotations.get( 0 ) ,
						de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno5.class );

				Assert.assertTrue(
						TypeArgSubArray.elementType instanceof AsmArray );

				{
					final AsmArray TypeArgSubSubArray = (AsmArray) TypeArgSubArray.elementType;

					Assert.assertEquals(
							// expected
							1 ,
							//actual
							TypeArgSubSubArray.typeAnnotations.size() );

					AsmTestUtil.assertAnnotationClassFqn(
							TypeArgSubSubArray.typeAnnotations.get( 0 ) ,
							de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno6.class );

					Assert.assertEquals(
							// expected
							Fqn.JAVA_LANG_STRING_FQN ,
							//actual
							( (AsmClassType) TypeArgSubSubArray.elementType ).fqn );

					{
						final AsmType arrElementTypeArgSubSubArrayType = ( (AsmArray) TypeArgSubSubArray ).elementType;

						Assert.assertEquals(
								// expected
								1 ,
								//actual
								arrElementTypeArgSubSubArrayType.typeAnnotations.size() );

						AsmTestUtil.assertAnnotationClassFqn(
								arrElementTypeArgSubSubArrayType.typeAnnotations.get( 0 ) ,
								de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno3.class );

						Assert.assertEquals(
								// expected
								Fqn.JAVA_LANG_STRING_FQN ,
								//actual
								( (AsmClassType) arrElementTypeArgSubSubArrayType ).fqn );
					}
				}
			}
		}
	}

}
