package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmFieldInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestTypeAnnotationOnFieldInnerType;
import de.heinerkuecker.asm_improved.type.AsmTopLevelOrInnerClassType;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.member.AsmInnerClassType;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link TestTypeAnnotationOnFieldInnerType}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class TypeAnnotationOnFieldInnerTypeTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestTypeAnnotationOnFieldInnerType.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertFalse(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertFalse(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertTrue(
				annotatedClassInfo.annotations != null || annotatedClassInfo.annotations.isEmpty() );

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "fld3" );

			System.out.println( field );
			
			final AsmType fieldType = field.type;

			{
				Assert.assertEquals(
						// expected
						1 ,
						//actual
						fieldType.typeAnnotations.size() );

				final AsmAnnotationUse fieldTypeAnnotation = fieldType.typeAnnotations.get( 0 );

				Assert.assertEquals(
						// expected
						new AsmClassType(
								new Fqn(
										de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno3.class ) ) ,
						//actual
						fieldTypeAnnotation.annotationType );

				Assert.assertTrue(
						fieldType instanceof AsmInnerClassType );

				final AsmTopLevelOrInnerClassType innerClassFieldType = ( (AsmInnerClassType) fieldType ).outerOrEnclosingType;

				{
					Assert.assertEquals(
							// expected
							1 ,
							//actual
							innerClassFieldType.typeAnnotations.size() );

					final AsmAnnotationUse innerClassTypeAnnotation = innerClassFieldType.typeAnnotations.get( 0 );

					Assert.assertEquals(
							// expected
							new AsmClassType(
									new Fqn(
											de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno2.class ) ) ,
							//actual
							innerClassTypeAnnotation.annotationType );

					Assert.assertTrue(
							innerClassFieldType instanceof AsmInnerClassType );

					AsmInnerClassType innerInnerClassFieldType = (AsmInnerClassType) innerClassFieldType;

					final AsmTopLevelOrInnerClassType innerInnerInnerClassFieldType = ( (AsmInnerClassType) innerInnerClassFieldType ).outerOrEnclosingType;

					{
						Assert.assertEquals(
								// expected
								1 ,
								//actual
								innerInnerInnerClassFieldType.typeAnnotations.size() );

						final AsmAnnotationUse innerInnerInnerClassTypeAnnotation = innerInnerInnerClassFieldType.typeAnnotations.get( 0 );

						Assert.assertEquals(
								// expected
								new AsmClassType(
										new Fqn(
												de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno1.class ) ) ,
								//actual
								innerInnerInnerClassTypeAnnotation.annotationType );

						Assert.assertTrue(
								innerInnerInnerClassFieldType instanceof AsmInnerClassType );

						final AsmTopLevelOrInnerClassType innerInnerInnerInnerClassFieldType = ( (AsmInnerClassType) innerInnerInnerClassFieldType ).outerOrEnclosingType;

						Assert.assertTrue(
								innerInnerInnerInnerClassFieldType instanceof AsmClassType );
					}
				}
			}
		}

		Assert.assertEquals(
				// expected
				1 ,
				//actual
				annotatedClassInfo.fields.size() );

		Assert.fail( "nicht fertig" );
	}

}
