package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmFieldInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestTypeAnnotationOnFieldTypeArgument;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link TestTypeAnnotationOnFieldTypeArgument}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class TypeAnnotationOnFieldTypeargumentTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestTypeAnnotationOnFieldTypeArgument.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertFalse(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertFalse(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertTrue(
				annotatedClassInfo.annotations != null || annotatedClassInfo.annotations.isEmpty() );

		Assert.assertEquals(
				// expected
				2 ,
				//actual
				annotatedClassInfo.fields.size() );

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strList" );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			final AsmAnnotationUse fieldTypeAnnotation = fieldType.typeAnnotations.get( 0 );

			Assert.assertEquals(
					// expected
					new AsmClassType(
							new Fqn(
									de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
					//actual
					fieldTypeAnnotation.annotationType );

			AsmClassType fieldClassType = (AsmClassType) fieldType;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldClassType.typeArgs.size() );

			final AsmType fieldTypeArg = fieldClassType.typeArgs.get( 0 );

			Assert.assertEquals(
					// expected
					2 ,
					//actual
					fieldTypeArg.typeAnnotations.size() );

			{
				final AsmAnnotationUse fieldTypeArgTypeAnnotation0 = fieldTypeArg.typeAnnotations.get( 0 );

				Assert.assertEquals(
						// expected
						new AsmClassType(
								new Fqn(
										de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
						//actual
						fieldTypeArgTypeAnnotation0.annotationType );
			}

			{
				final AsmAnnotationUse fieldTypeArgTypeAnnotation1 = fieldTypeArg.typeAnnotations.get( 1 );

				Assert.assertEquals(
						// expected
						new AsmClassType(
								new Fqn(
										de.heinerkuecker.asm_improved.test_data.annotation_def.NotEmpty.class ) ) ,
						//actual
						fieldTypeArgTypeAnnotation1.annotationType );
			}
		}

		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "strListList" );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					new Fqn(
							java.util.List.class ) ,
					//actual
					( (AsmClassType) fieldType ).fqn );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			final AsmAnnotationUse fieldTypeAnnotation = fieldType.typeAnnotations.get( 0 );

			Assert.assertEquals(
					// expected
					new AsmClassType(
							new Fqn(
									de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
					//actual
					fieldTypeAnnotation.annotationType );

			final AsmClassType fieldClassType = (AsmClassType) fieldType;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldClassType.typeArgs.size() );

			final AsmType fieldTypeArg = fieldClassType.typeArgs.get( 0 );

			Assert.assertEquals(
					// expected
					new Fqn(
							java.util.List.class ) ,
					//actual
					( (AsmClassType) fieldTypeArg ).fqn );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldTypeArg.typeAnnotations.size() );

			{
				final AsmAnnotationUse fieldTypeArgTypeAnnotation0 = fieldTypeArg.typeAnnotations.get( 0 );

				Assert.assertEquals(
						// expected
						new AsmClassType(
								new Fqn(
										de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
						//actual
						fieldTypeArgTypeAnnotation0.annotationType );

				final AsmType fieldTypeArgTypeArg = ( (AsmClassType) fieldTypeArg ).typeArgs.get( 0 );

				Assert.assertEquals(
						// expected
						2 ,
						//actual
						fieldTypeArgTypeArg.typeAnnotations.size() );

				{
					final AsmAnnotationUse fieldTypeArgTypeArgTypeAnnotation0 = fieldTypeArgTypeArg.typeAnnotations.get( 0 );

					Assert.assertEquals(
							// expected
							new AsmClassType(
									new Fqn(
											de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
							//actual
							fieldTypeArgTypeArgTypeAnnotation0.annotationType );
				}

				{
					final AsmAnnotationUse fieldTypeArgTypeArgTypeAnnotation1 = fieldTypeArgTypeArg.typeAnnotations.get( 1 );

					Assert.assertEquals(
							// expected
							new AsmClassType(
									new Fqn(
											de.heinerkuecker.asm_improved.test_data.annotation_def.NotEmpty.class ) ) ,
							//actual
							fieldTypeArgTypeArgTypeAnnotation1.annotationType );
				}

				Assert.assertEquals(
						// expected
						Fqn.JAVA_LANG_STRING_FQN ,
						//actual
						( (AsmClassType) fieldTypeArgTypeArg ).fqn );
			}
		}
	}

}
