package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestTypeAnnotationOnMethodParamArray1;
import de.heinerkuecker.asm_improved.type.AsmArray;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link TestTypeAnnotationOnMethodParamArray1}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class TypeAnnotationOnMethodParamArray1Test
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestTypeAnnotationOnMethodParamArray1.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertFalse(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertFalse(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertTrue(
				annotatedClassInfo.annotations != null || annotatedClassInfo.annotations.isEmpty() );

		Assert.assertEquals(
				// expected
				1 ,
				//actual
				annotatedClassInfo.methods.size() );

		{
			final ImmutableArray<? extends AsmMethodInfo> methods = annotatedClassInfo.methods.get( "m" );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					methods.size() );

			final AsmMethodInfo method = methods.get( 0 );

			final AsmType methodReturnType = method.returnType;

			Assert.assertTrue(
					methodReturnType.typeAnnotations == null || methodReturnType.typeAnnotations.isEmpty() );

			Assert.assertTrue(
					methodReturnType instanceof AsmArray );

			final AsmType arrElementType = ( (AsmArray) methodReturnType ).elementType;

			Assert.assertEquals(
					// expected
					Fqn.JAVA_LANG_STRING_FQN ,
					//actual
					( (AsmClassType) arrElementType ).fqn );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					arrElementType.typeAnnotations.size() );

			final AsmAnnotationUse arrElementTypeTypeAnnotation = arrElementType.typeAnnotations.get( 0 );

			Assert.assertEquals(
					// expected
					new AsmClassType(
							new Fqn(
									de.heinerkuecker.asm_improved.test_data.annotation_def.NotNull.class ) ) ,
					//actual
					arrElementTypeTypeAnnotation.annotationType );
		}
		
		Assert.fail( "nicht fertig" );
	}

}
