package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmFieldInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestTypeAnnotationOnSuperClassAndOnImplementsTypeargs;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmIntersectionBound;
import de.heinerkuecker.asm_improved.type.generics.param.AsmUnboundedTypeParam;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link TestTypeAnnotationOnSuperClassAndOnImplementsTypeargs}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class TypeAnnotationOnSuperClassAndOnImplementsTypeargsTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestTypeAnnotationOnSuperClassAndOnImplementsTypeargs.class.getCanonicalName() );

		System.out.println( classInfo );
		
		Assert.assertTrue(
				classInfo.isTopLevel() );

		Assert.assertFalse(
				classInfo.isFinal );

		Assert.assertTrue(
				classInfo.isAbstract );

		Assert.assertTrue(
				classInfo.isStatic );

		Assert.assertFalse(
				classInfo.isInterface );

		Assert.assertFalse(
				classInfo.isEnum );

		Assert.assertFalse(
				classInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertTrue(
				classInfo.annotations == null || classInfo.annotations.isEmpty() );

		Assert.assertTrue(
				classInfo.typeParams == null || classInfo.typeParams.isEmpty() );

		{
			final AsmClassType superType = classInfo.superType;
			
			System.out.println( superType );

			Assert.assertEquals(
					//expected
					new Fqn( java.util.HashMap.class ) ,
					//actual
					superType.fqn );

			Assert.assertTrue(
					superType.typeAnnotations == null || superType.typeAnnotations.isEmpty() );

			Assert.assertEquals(
					//expected
					2 ,
					//actual
					superType.typeArgs.size() );

			
			
			Assert.fail( "nicht fertig" );
		}

		{
			final AsmAbstractTypeParam typeParam = classInfo.typeParams.get( 1 );

			System.out.println( typeParam );
			
			Assert.assertTrue(
					typeParam instanceof AsmExtendsTypeParam );

			
			
		}

		{
			final AsmAbstractTypeParam typeParam = classInfo.typeParams.get( 2 );

			System.out.println( typeParam );
			
			Assert.assertTrue(
					typeParam instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam = (AsmExtendsTypeParam) typeParam;

			Assert.assertTrue(
					extendsTypeParam.boundType instanceof AsmIntersectionBound );

			AsmIntersectionBound intersectionBound = (AsmIntersectionBound) extendsTypeParam.boundType;

			System.out.println( intersectionBound );
		}
		
		{
			final AsmFieldInfo field = classInfo.fields.get( "intField" );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			final AsmAnnotationUse fieldTypeAnnotation = fieldType.typeAnnotations.get( 0 );

			Assert.assertEquals(
					// expected
					new AsmClassType(
							new Fqn(
									de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class ) ) ,
					//actual
					fieldTypeAnnotation.annotationType );
		}

		Assert.fail( "nicht fertig" );
	}

}
