package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmFieldInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestTypeAnnotationOnSuperClassTypeargs;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmIntersectionBound;
import de.heinerkuecker.asm_improved.type.generics.param.AsmUnboundedTypeParam;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link TestTypeAnnotationOnSuperClassTypeargs}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class TypeAnnotationOnSuperClassTypeargsTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestTypeAnnotationOnSuperClassTypeargs.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertFalse(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertFalse(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertTrue(
				annotatedClassInfo.annotations != null || annotatedClassInfo.annotations.isEmpty() );

		Assert.assertEquals(
				// expected
				3 ,
				//actual
				annotatedClassInfo.typeParams.size() );

		{
			final AsmAbstractTypeParam typeParam = annotatedClassInfo.typeParams.get( 0 );

			System.out.println( typeParam );
			
			Assert.assertTrue(
					typeParam instanceof AsmUnboundedTypeParam );

			
			
		}

		{
			final AsmAbstractTypeParam typeParam = annotatedClassInfo.typeParams.get( 1 );

			System.out.println( typeParam );
			
			Assert.assertTrue(
					typeParam instanceof AsmExtendsTypeParam );

			
			
		}

		{
			final AsmAbstractTypeParam typeParam = annotatedClassInfo.typeParams.get( 2 );

			System.out.println( typeParam );
			
			Assert.assertTrue(
					typeParam instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam = (AsmExtendsTypeParam) typeParam;

			Assert.assertTrue(
					extendsTypeParam.boundType instanceof AsmIntersectionBound );

			AsmIntersectionBound intersectionBound = (AsmIntersectionBound) extendsTypeParam.boundType;

			System.out.println( intersectionBound );
		}
		
		{
			final AsmFieldInfo field = annotatedClassInfo.fields.get( "intField" );

			final AsmType fieldType = field.type;

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					fieldType.typeAnnotations.size() );

			final AsmAnnotationUse fieldTypeAnnotation = fieldType.typeAnnotations.get( 0 );

			Assert.assertEquals(
					// expected
					new AsmClassType(
							new Fqn(
									de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class ) ) ,
					//actual
					fieldTypeAnnotation.annotationType );
		}

		Assert.fail( "nicht fertig" );
	}

}
