package de.heinerkuecker.asm_improved.test.annotation_use;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.test_data.annotation_use.TestTypeAnnotationOnThrows;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;

/**
 * JUnit4-Testcase fr die Klasse {@link TestTypeAnnotationOnThrows}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class TypeAnnotationOnThrowsTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestTypeAnnotationOnThrows.class.getCanonicalName() );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertFalse(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertFalse(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		// Klassen-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertTrue(
				annotatedClassInfo.annotations != null || annotatedClassInfo.annotations.isEmpty() );

		{
			final ImmutableArray<? extends AsmMethodInfo> methods = annotatedClassInfo.methods.get( "m00" );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					methods.size() );

			final AsmMethodInfo method = methods.get( 0 );

			System.out.println( method );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					method.exceptions.size() );

			final AsmType exceptionType = method.exceptions.get( 0 );

			Assert.assertEquals(
					// expected
					new Fqn( Exception.class ) ,
					//actual
					( (AsmClassType) exceptionType ).fqn );

			Assert.assertEquals(
					// expected
					1 ,
					//actual
					exceptionType.typeAnnotations.size() );

			final AsmAnnotationUse exceptionTypeAnnotation = exceptionType.typeAnnotations.get( 0 );

			Assert.assertEquals(
					// expected
					new AsmClassType(
							new Fqn(
									de.heinerkuecker.asm_improved.test_data.annotation_def.TypeAnno0.class ) ) ,
					//actual
					exceptionTypeAnnotation.annotationType );

			Assert.fail( "nicht fertig" );
		}

		Assert.fail( "nicht fertig" );
	}

}
