package de.heinerkuecker.asm_improved.test.method.exception;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.util.Fqn;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;

/**
 * JUnit4-Testcase fr die Klasse {@link AutoCloseable}.
 * 
 * @author Heiner K&uuml;cker
 */
public class MethodExceptionAutocloseableTest
{
	/**
	 * Test method for {@link AutoCloseable}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						java.lang.AutoCloseable.class.getCanonicalName() );

		final ImmutableMap<String, ? extends ImmutableArray<? extends AsmMethodInfo>> methods =
				classInfo.methods;

		Assert.assertNotNull(
				methods );

		Assert.assertTrue(
				methods.containsKey( "close" ) );

		final ImmutableArray<? extends AsmMethodInfo> methodsWithSameName =
				methods.get( "close" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodsWithSameName.size() );

		final AsmMethodInfo method = methodsWithSameName.get( 0 );

		Assert.assertTrue(
				method.isPublic );

		Assert.assertFalse(
				method.isProtected );

		Assert.assertFalse(
				method.isPrivate );

		Assert.assertTrue(
				method.isAbstract );

		Assert.assertFalse(
				method.isFinal );

		Assert.assertFalse(
				method.isStatic );

		Assert.assertTrue(
				method.annotations == null || method.annotations.isEmpty() );

		Assert.assertFalse(
				method.exceptions == null || method.exceptions.isEmpty() );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				method.exceptions.size() );

		final AsmType exceptionType = method.exceptions.get( 0 );

		Assert.assertEquals(
				//expected
				new AsmClassType( new Fqn( Exception.class ) ) ,
				//actual
				exceptionType );

		Assert.assertTrue(
				method.returnType == null );

		Assert.assertEquals(
				//expected
				0 ,
				//actual
				method.getParamCount() );

		//Assert.fail("Not yet implemented");
	}

}
