package de.heinerkuecker.asm_improved.test_data.annotation_use.package_annotation_xmlschemas;

import javax.xml.bind.annotation.XmlNs;
import javax.xml.bind.annotation.XmlNsForm;
import javax.xml.bind.annotation.XmlSchema;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.annotation_use.AsmAnnotationUse;
import de.heinerkuecker.asm_improved.annotation_use.AsmEnumValueUse;
import de.heinerkuecker.asm_improved.test_data.annotation_def.TestAnnotation;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr die Klasse {@link package-info}.
 * 
 * @author Heiner K&uuml;cker
 */
public final class PackageAnnotationXmlschemasTest
{
	/**
	 * Test method for {@link TestAnnotation}.
	 */
	@Test
	public final void test()
	{
		final AsmClassInfo annotatedClassInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						PackageAnnotationXmlschemasTest.class.getPackage().getName() + ".package-info" );

		Assert.assertTrue(
				annotatedClassInfo.isTopLevel() );

		Assert.assertFalse(
				annotatedClassInfo.isFinal );

		Assert.assertTrue(
				annotatedClassInfo.isAbstract );

		Assert.assertTrue(
				annotatedClassInfo.isStatic );

		Assert.assertTrue(
				annotatedClassInfo.isInterface );

		Assert.assertFalse(
				annotatedClassInfo.isEnum );

		Assert.assertFalse(
				annotatedClassInfo.isAnnotation );

		// Package-Annotation
		Assert.assertNotNull(
				annotatedClassInfo.annotations );

		Assert.assertFalse(
				annotatedClassInfo.annotations.isEmpty() );

		Assert.assertEquals(
				1 ,
				annotatedClassInfo.annotations.size() );

		final AsmAnnotationUse annotation = annotatedClassInfo.annotations.get( 0 );

		Assert.assertEquals(
				new AsmClassType( new Fqn( XmlSchema.class ) ) ,
				annotation.annotationType );

		Assert.assertFalse(
				annotation.values == null || annotation.values.isEmpty() );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				annotation.values.size() );
		
		final Object annotationValue = annotation.values.get( "xmlns" );

		final Object[] annotationValueArr = (Object[]) annotationValue;
		
		Assert.assertEquals(
				//expected
				2 ,
				//actual
				annotationValueArr.length );

		{
			final AsmAnnotationUse annotationValue0 = (AsmAnnotationUse) annotationValueArr[ 0 ];

			Assert.assertEquals(
					new AsmClassType( new Fqn( XmlNs.class ) ) ,
					annotationValue0.annotationType );

			Assert.assertEquals(
					//expected
					2 ,
					//actual
					annotationValue0.values.size() );

			Assert.assertEquals(
					//expected
					"de.heinerkuecker.asm_improved.test_data.annotation_use.package_annotation_xmlschemas" ,
					//actual
					annotationValue0.values.get( "prefix" ) );

			Assert.assertEquals(
					//expected
					"http://de.heinerkuecker.asm_improved.test_data.annotation_use.package_annotation_xmlschemas" ,
					//actual
					annotationValue0.values.get( "namespaceURI" ) );
		}
		
		{		
			final AsmAnnotationUse annotationValue1 = (AsmAnnotationUse) annotationValueArr[ 1 ];

			Assert.assertEquals(
					new AsmClassType( new Fqn( XmlNs.class ) ) ,
					annotationValue1.annotationType );

			Assert.assertEquals(
					//expected
					2 ,
					//actual
					annotationValue1.values.size() );

			Assert.assertEquals(
					//expected
					"de.heinerkuecker.asm_improved.test_data.annotation_use.package_annotation_xmlschemas2" ,
					//actual
					annotationValue1.values.get( "prefix" ) );

			Assert.assertEquals(
					//expected
					"http://de.heinerkuecker.asm_improved.test_data.annotation_use.package_annotation_xmlschemas2" ,
					//actual
					annotationValue1.values.get( "namespaceURI" ) );
		}

		/*
		 * values under consideration defaults
		 */
		Assert.assertArrayEquals(
				//expected
				new Object[ 0 ] ,
				//actual
				// Bequemlichkeits-Methode
				(Object[]) annotation.getDefault(
						AllAsmTests.asmClasspathInfo ,
						"xmlns" ) );

		Assert.assertEquals(
				//expected
				"" ,
				//actual
				// Bequemlichkeits-Methode
				annotation.getValueUnderConsiderationDefault(
						AllAsmTests.asmClasspathInfo ,
						"namespace" ) );

		Assert.assertEquals(
				//expected
				XmlNsForm.UNSET.name() ,
				//actual
				// Bequemlichkeits-Methode
				( (AsmEnumValueUse) annotation.getValueUnderConsiderationDefault(
						AllAsmTests.asmClasspathInfo ,
						"elementFormDefault" ) ).enumValueName );

		Assert.assertEquals(
				//expected
				XmlNsForm.UNSET.name() ,
				//actual
				// Bequemlichkeits-Methode
				( (AsmEnumValueUse) annotation.getValueUnderConsiderationDefault(
						AllAsmTests.asmClasspathInfo ,
						"attributeFormDefault" ) ).enumValueName );

		Assert.assertEquals(
				//expected
				XmlSchema.NO_LOCATION ,
				//actual
				// Bequemlichkeits-Methode
				annotation.getValueUnderConsiderationDefault(
						AllAsmTests.asmClasspathInfo ,
						"location" ) );
	}

}
