package de.heinerkuecker.asm_improved.type_param;

import java.io.Serializable;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmIntersectionBound;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr {@link AsmClasspathInfo}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ClassWithTypeparamIntersectionBoundedTest
{

	@SuppressWarnings( "javadoc" )
    @Test
	public final void test()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						ClassWithTypeparamIntersectionBounded.class.getCanonicalName() );

		//System.out.println( classInfo.typeParams );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				classInfo.typeParams.size() );

		final AsmAbstractTypeParam typeParam0 =
				classInfo.typeParams.get( 0 );

		//System.out.println( typeParam0 );

		Assert.assertEquals(
				//expected
				"T" ,
				//actual
				typeParam0.name );

		Assert.assertTrue(
				typeParam0 instanceof AsmExtendsTypeParam );

		final AsmExtendsTypeParam extendsTypeParam0 = (AsmExtendsTypeParam) typeParam0;

		Assert.assertTrue(
				extendsTypeParam0.boundType instanceof AsmIntersectionBound );

		final AsmIntersectionBound intersectionBoundType = (AsmIntersectionBound) extendsTypeParam0.boundType;

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				intersectionBoundType.boundTypes.size() );

		{
			final AsmType boundType0 = intersectionBoundType.boundTypes.get( 0 );

			Assert.assertTrue(
					boundType0 instanceof AsmClassType );

			final AsmClassType boundClassType0 = (AsmClassType) boundType0;

			Assert.assertEquals(
					//expected
					new Fqn( Serializable.class ) ,
					//actual
					boundClassType0.fqn );
		}


		{
			final AsmType boundType1 = intersectionBoundType.boundTypes.get( 1 );

			Assert.assertTrue(
					boundType1 instanceof AsmClassType );

			final AsmClassType boundClassType1 = (AsmClassType) boundType1;

			Assert.assertEquals(
					//expected
					new Fqn( Cloneable.class ) ,
					//actual
					boundClassType1.fqn );
		}

		//Assert.fail( "Not yet implemented" );
	}

}
