package de.heinerkuecker.asm_improved.type_param.method.static_.exception;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.AsmMethodInfo;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.ImmutableArray;
import de.heinerkuecker.asm_improved.util.ImmutableMap;

/**
 * JUnit4-Testcase fr die Klasse {@link TestStaticMethodWithTypeparamsException}.
 * 
 * @author Heiner K&uuml;cker
 */
public class StaticMethodWithTypeparamsExceptionTest
{
	/**
	 * Test method for {@link TestStaticMethodWithTypeparamsException}.
	 */
	@Test
	public final void testM1()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestStaticMethodWithTypeparamsException.class.getCanonicalName() );

		final ImmutableMap<String, ? extends ImmutableArray<? extends AsmMethodInfo>> methods =
				classInfo.methods;

		Assert.assertNotNull(
				methods );

		Assert.assertTrue(
				methods.containsKey( "m1" ) );

		final ImmutableArray<? extends AsmMethodInfo> methodsWithSameName =
				methods.get( "m1" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodsWithSameName.size() );

		final AsmMethodInfo method = methodsWithSameName.get( 0 );

		Assert.assertTrue(
				method.isPublic );

		Assert.assertFalse(
				method.isProtected );

		Assert.assertFalse(
				method.isPrivate );

		Assert.assertFalse(
				method.isAbstract );

		Assert.assertFalse(
				method.isFinal );

		Assert.assertTrue(
				method.isStatic );

		Assert.assertTrue(
				method.annotations == null || method.annotations.isEmpty() );

		Assert.assertTrue(
				method.returnType == null );

		Assert.assertEquals(
				//expected
				0 ,
				//actual
				method.getParamCount() );

		Assert.assertFalse(
				method.exceptions == null || method.exceptions.isEmpty() );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				method.exceptions.size() );

		final AsmType exceptionType = method.exceptions.get( 0 );

		final AsmTypevar exceptionTypevar = (AsmTypevar) exceptionType;

		Assert.assertEquals(
				//expected
				"T" ,
				//actual
				exceptionTypevar.name );

		// TODO check extends bound of type var
		
		//Assert.fail("Not yet implemented");
	}

	/**
	 * Test method for {@link TestStaticMethodWithTypeparamsException}.
	 */
	@Test
	public final void testM2()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						TestStaticMethodWithTypeparamsException.class.getCanonicalName() );

		final ImmutableMap<String, ? extends ImmutableArray<? extends AsmMethodInfo>> methods =
				classInfo.methods;

		Assert.assertNotNull(
				methods );

		Assert.assertTrue(
				methods.containsKey( "m2" ) );

		final ImmutableArray<? extends AsmMethodInfo> methodsWithSameName =
				methods.get( "m2" );

		Assert.assertEquals(
				//expected
				1 ,
				//actual
				methodsWithSameName.size() );

		final AsmMethodInfo method = methodsWithSameName.get( 0 );

		Assert.assertTrue(
				method.isPublic );

		Assert.assertFalse(
				method.isProtected );

		Assert.assertFalse(
				method.isPrivate );

		Assert.assertFalse(
				method.isAbstract );

		Assert.assertFalse(
				method.isFinal );

		Assert.assertTrue(
				method.isStatic );

		Assert.assertTrue(
				method.annotations == null || method.annotations.isEmpty() );

		Assert.assertTrue(
				method.returnType == null );

		Assert.assertEquals(
				//expected
				0 ,
				//actual
				method.getParamCount() );

		Assert.assertFalse(
				method.exceptions == null || method.exceptions.isEmpty() );

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				method.exceptions.size() );

		{
			final AsmType exceptionType0 = method.exceptions.get( 0 );

			final AsmTypevar exceptionTypevar0 = (AsmTypevar) exceptionType0;

			Assert.assertEquals(
					//expected
					"T1" ,
					//actual
					exceptionTypevar0.name );

			// TODO check extends bound of type var
		}

		{
			final AsmType exceptionType1 = method.exceptions.get( 1 );

			final AsmTypevar exceptionTypevar1 = (AsmTypevar) exceptionType1;

			Assert.assertEquals(
					//expected
					"T2" ,
					//actual
					exceptionTypevar1.name );

			// TODO check extends bound of type var
		}

		//Assert.fail("Not yet implemented");
	}

}
