package de.heinerkuecker.asm_improved.type_params;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmIntersectionBound;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr {@link AsmClasspathInfo}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ClassWithTypeparamsBoundedTest
{

	@Test
	public final void test()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						ClassWithTypeparamsBounded.class.getCanonicalName() );

		//System.out.println( classInfo.typeParams );

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				classInfo.typeParams.size() );

		{
			final AsmAbstractTypeParam typeParam0 =
					classInfo.typeParams.get( 0 );

			//System.out.println( typeParam0 );

			Assert.assertEquals(
					//expected
					"T0" ,
					//actual
					typeParam0.name );

			Assert.assertTrue(
					typeParam0 instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam0 = (AsmExtendsTypeParam) typeParam0;

			final AsmType extendsTypeParam0BoundType = extendsTypeParam0.boundType;

			Assert.assertTrue(
					extendsTypeParam0BoundType instanceof AsmClassType );

			final AsmClassType extendsTypeParam0ClassBoundType = (AsmClassType) extendsTypeParam0BoundType;

			Assert.assertEquals(
					//expected
					"java.io.Serializable" ,
					//actual
					extendsTypeParam0ClassBoundType.toString() );

			Assert.assertEquals(
					//expected
					new Fqn( java.io.Serializable.class ) ,
					//actual
					extendsTypeParam0ClassBoundType.fqn );

			Assert.assertTrue(
					extendsTypeParam0ClassBoundType.typeArgs == null ||
							extendsTypeParam0ClassBoundType.typeArgs.size() == 0 );
		}

		{
			final AsmAbstractTypeParam typeParam1 =
					classInfo.typeParams.get( 1 );

			//System.out.println( typeParam1 );

			Assert.assertEquals(
					//expected
					"T1" ,
					//actual
					typeParam1.name );

			Assert.assertTrue(
					typeParam1 instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam1 = (AsmExtendsTypeParam) typeParam1;

			final AsmType extendsTypeParam1BoundType = extendsTypeParam1.boundType;

			Assert.assertTrue(
					extendsTypeParam1BoundType instanceof AsmClassType );

			final AsmClassType extendsTypeParam1ClassBoundType = (AsmClassType) extendsTypeParam1BoundType;

			Assert.assertEquals(
					//expected
					"java.lang.Cloneable" ,
					//actual
					extendsTypeParam1ClassBoundType.toString() );

			Assert.assertEquals(
					//expected
					new Fqn( Cloneable.class ) ,
					//actual
					extendsTypeParam1ClassBoundType.fqn );

			Assert.assertTrue(
					extendsTypeParam1ClassBoundType.typeArgs == null ||
							extendsTypeParam1ClassBoundType.typeArgs.size() == 0 );
		}

		//Assert.fail( "Not yet implemented" );
	}

}
