package de.heinerkuecker.asm_improved.type_params;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr {@link AsmClasspathInfo}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ClassWithTypeparamsBoundedToTypevarTest
{

	@Test
	public final void test()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						ClassWithTypeparamsBoundedToTypevar.class.getCanonicalName() );

		//System.out.println( classInfo.typeParams );

		Assert.assertEquals(
				//expected
				3 ,
				//actual
				classInfo.typeParams.size() );

		{
			final AsmAbstractTypeParam typeParam0 =
					classInfo.typeParams.get( 0 );

			//System.out.println( typeParam0 );

			Assert.assertEquals(
					//expected
					"T0" ,
					//actual
					typeParam0.name );

			Assert.assertTrue(
					typeParam0 instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam0 = (AsmExtendsTypeParam) typeParam0;

			final AsmType extendsTypeParam0BoundType = extendsTypeParam0.boundType;

			Assert.assertTrue(
					extendsTypeParam0BoundType instanceof AsmTypevar );

			final AsmTypevar extendsTypeParam0TypevarBoundType = (AsmTypevar) extendsTypeParam0BoundType;

			Assert.assertEquals(
					//expected
					"T1" ,
					//actual
					extendsTypeParam0TypevarBoundType.name );
		}

		{
			final AsmAbstractTypeParam typeParam1 =
					classInfo.typeParams.get( 1 );

			//System.out.println( typeParam1 );

			Assert.assertEquals(
					//expected
					"T1" ,
					//actual
					typeParam1.name );

			Assert.assertTrue(
					typeParam1 instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam1 = (AsmExtendsTypeParam) typeParam1;

			final AsmType extendsTypeParam1BoundType = extendsTypeParam1.boundType;

			Assert.assertTrue(
					extendsTypeParam1BoundType instanceof AsmClassType );

			final AsmClassType extendsTypeParam0ClassBoundType = (AsmClassType) extendsTypeParam1BoundType;

			Assert.assertEquals(
					//expected
					"java.io.Serializable" ,
					//actual
					extendsTypeParam0ClassBoundType.toString() );

			Assert.assertEquals(
					//expected
					new Fqn( java.io.Serializable.class ) ,
					//actual
					extendsTypeParam0ClassBoundType.fqn );

			Assert.assertTrue(
					extendsTypeParam0ClassBoundType.typeArgs == null ||
							extendsTypeParam0ClassBoundType.typeArgs.size() == 0 );
		}


		{
			final AsmAbstractTypeParam typeParam2 =
					classInfo.typeParams.get( 2 );

			//System.out.println( typeParam2 );

			Assert.assertEquals(
					//expected
					"T2" ,
					//actual
					typeParam2.name );

			Assert.assertTrue(
					typeParam2 instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam2 = (AsmExtendsTypeParam) typeParam2;

			final AsmType extendsTypeParam2BoundType = extendsTypeParam2.boundType;

			Assert.assertTrue(
					extendsTypeParam2BoundType instanceof AsmClassType );

			final AsmClassType extendsTypeParam2ClassBoundType = (AsmClassType) extendsTypeParam2BoundType;

			Assert.assertEquals(
					//expected
					"java.lang.Cloneable" ,
					//actual
					extendsTypeParam2ClassBoundType.toString() );

			Assert.assertEquals(
					//expected
					new Fqn( Cloneable.class ) ,
					//actual
					extendsTypeParam2ClassBoundType.fqn );

			Assert.assertTrue(
					extendsTypeParam2ClassBoundType.typeArgs == null ||
							extendsTypeParam2ClassBoundType.typeArgs.size() == 0 );
		}

		//Assert.fail( "Not yet implemented" );
	}

}
