package de.heinerkuecker.asm_improved.type_params;

import java.io.Serializable;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmIntersectionBound;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr {@link AsmClasspathInfo}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ClassWithTypeparamsIntersectionBoundedTest
{

	@Test
	public final void test()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						ClassWithTypeparamsIntersectionBounded.class.getCanonicalName() );

		//System.out.println( classInfo.typeParams );

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				classInfo.typeParams.size() );

		{
			final AsmAbstractTypeParam typeParam0 =
					classInfo.typeParams.get( 0 );

			//System.out.println( typeParam0 );

			Assert.assertEquals(
					//expected
					"T0" ,
					//actual
					typeParam0.name );

			Assert.assertTrue(
					typeParam0 instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam0 = (AsmExtendsTypeParam) typeParam0;

			final AsmType extendsTypeParam0BoundType0 = extendsTypeParam0.boundType;

			Assert.assertTrue(
					extendsTypeParam0BoundType0 instanceof AsmIntersectionBound );

			final AsmIntersectionBound extendsTypeParam0IntersectionBoundType0 = (AsmIntersectionBound) extendsTypeParam0BoundType0;

			Assert.assertEquals(
					//expected
					2 ,
					//actual
					extendsTypeParam0IntersectionBoundType0.boundTypes.size() );

			{
				final AsmType extendsTypeParam0IntersectionBoundType0BoundType0 = extendsTypeParam0IntersectionBoundType0.boundTypes.get( 0 );

				Assert.assertEquals(
						//expected
						"java.lang.Comparable<T0>" ,
						//actual
						extendsTypeParam0IntersectionBoundType0BoundType0.toString() );

				Assert.assertTrue(
						extendsTypeParam0IntersectionBoundType0BoundType0 instanceof AsmClassType );
				
				final AsmClassType extendsTypeParam0IntersectionBoundType0BoundClassType0 = (AsmClassType) extendsTypeParam0IntersectionBoundType0BoundType0;
				
				Assert.assertEquals(
						//expected
						new Fqn( Comparable.class ) ,
						//actual
						extendsTypeParam0IntersectionBoundType0BoundClassType0.fqn );

				Assert.assertEquals(
						//expected
						1 ,
						//actual
						extendsTypeParam0IntersectionBoundType0BoundClassType0.typeArgs.size() );

				final AsmType extendsTypeParam0IntersectionBoundType0BoundClassType0TypeArg0 = extendsTypeParam0IntersectionBoundType0BoundClassType0.typeArgs.get( 0 );
				
				Assert.assertTrue(
						extendsTypeParam0IntersectionBoundType0BoundClassType0TypeArg0 instanceof AsmTypevar );

				final AsmTypevar extendsTypeParam0IntersectionBoundType0BoundClassType0TypeArg0Typevar = (AsmTypevar) extendsTypeParam0IntersectionBoundType0BoundClassType0TypeArg0;

				Assert.assertEquals(
						//expected
						"T0" ,
						//actual
						extendsTypeParam0IntersectionBoundType0BoundClassType0TypeArg0Typevar.name );
			}

			{
				final AsmType extendsTypeParam0IntersectionBoundType0BoundType1 = extendsTypeParam0IntersectionBoundType0.boundTypes.get( 1 );

				Assert.assertEquals(
						//expected
						"java.io.Serializable" ,
						//actual
						extendsTypeParam0IntersectionBoundType0BoundType1.toString() );

				Assert.assertTrue(
						extendsTypeParam0IntersectionBoundType0BoundType1 instanceof AsmClassType );
				
				final AsmClassType extendsTypeParam0IntersectionBoundType0BoundClassType1 = (AsmClassType) extendsTypeParam0IntersectionBoundType0BoundType1;
				
				Assert.assertEquals(
						//expected
						new Fqn( Serializable.class ) ,
						//actual
						extendsTypeParam0IntersectionBoundType0BoundClassType1.fqn );

				Assert.assertTrue(
						extendsTypeParam0IntersectionBoundType0BoundClassType1.typeArgs == null ||
								extendsTypeParam0IntersectionBoundType0BoundClassType1.typeArgs.size() == 0 );
			}

		}

		{
			final AsmAbstractTypeParam typeParam1 =
					classInfo.typeParams.get( 1 );

			//System.out.println( typeParam1 );

			Assert.assertEquals(
					//expected
					"T1" ,
					//actual
					typeParam1.name );

			Assert.assertTrue(
					typeParam1 instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam1 = (AsmExtendsTypeParam) typeParam1;

			final AsmType extendsTypeParam1BoundType0 = extendsTypeParam1.boundType;

			Assert.assertTrue(
					extendsTypeParam1BoundType0 instanceof AsmIntersectionBound );

			final AsmIntersectionBound extendsTypeParam1IntersectionBoundType0 = (AsmIntersectionBound) extendsTypeParam1BoundType0;

			Assert.assertEquals(
					//expected
					2 ,
					//actual
					extendsTypeParam1IntersectionBoundType0.boundTypes.size() );

			{
				final AsmType extendsTypeParam1IntersectionBoundType0BoundType0 = extendsTypeParam1IntersectionBoundType0.boundTypes.get( 0 );

				Assert.assertEquals(
						//expected
						"java.lang.Comparable<T1>" ,
						//actual
						extendsTypeParam1IntersectionBoundType0BoundType0.toString() );

				Assert.assertTrue(
						extendsTypeParam1IntersectionBoundType0BoundType0 instanceof AsmClassType );
				
				final AsmClassType extendsTypeParam1IntersectionBoundType0BoundClassType0 = (AsmClassType) extendsTypeParam1IntersectionBoundType0BoundType0;
				
				Assert.assertEquals(
						//expected
						new Fqn( Comparable.class ) ,
						//actual
						extendsTypeParam1IntersectionBoundType0BoundClassType0.fqn );

				Assert.assertEquals(
						//expected
						1 ,
						//actual
						extendsTypeParam1IntersectionBoundType0BoundClassType0.typeArgs.size() );

				final AsmType extendsTypeParam1IntersectionBoundType0BoundClassType0TypeArg0 = extendsTypeParam1IntersectionBoundType0BoundClassType0.typeArgs.get( 0 );
				
				Assert.assertTrue(
						extendsTypeParam1IntersectionBoundType0BoundClassType0TypeArg0 instanceof AsmTypevar );

				final AsmTypevar extendsTypeParam1IntersectionBoundType0BoundClassType0TypeArg0Typevar = (AsmTypevar) extendsTypeParam1IntersectionBoundType0BoundClassType0TypeArg0;

				Assert.assertEquals(
						//expected
						"T1" ,
						//actual
						extendsTypeParam1IntersectionBoundType0BoundClassType0TypeArg0Typevar.name );
			}

			{
				final AsmType extendsTypeParam1IntersectionBoundType0BoundType1 = extendsTypeParam1IntersectionBoundType0.boundTypes.get( 1 );

				Assert.assertEquals(
						//expected
						"java.io.Serializable" ,
						//actual
						extendsTypeParam1IntersectionBoundType0BoundType1.toString() );

				Assert.assertTrue(
						extendsTypeParam1IntersectionBoundType0BoundType1 instanceof AsmClassType );
				
				final AsmClassType extendsTypeParam1IntersectionBoundType0BoundClassType1 = (AsmClassType) extendsTypeParam1IntersectionBoundType0BoundType1;
				
				Assert.assertEquals(
						//expected
						new Fqn( Serializable.class ) ,
						//actual
						extendsTypeParam1IntersectionBoundType0BoundClassType1.fqn );

				Assert.assertTrue(
						extendsTypeParam1IntersectionBoundType0BoundClassType1.typeArgs == null ||
								extendsTypeParam1IntersectionBoundType0BoundClassType1.typeArgs.size() == 0 );
			}

		}

		//Assert.fail( "Not yet implemented" );
	}

}
