package de.heinerkuecker.asm_improved.type_params;

import java.util.EnumMap;

import org.junit.Assert;
import org.junit.Test;

import de.heinerkuecker.asm_improved.AllAsmTests;
import de.heinerkuecker.asm_improved.AsmClasspathInfo;
import de.heinerkuecker.asm_improved.AsmClassInfo;
import de.heinerkuecker.asm_improved.type.AsmClassType;
import de.heinerkuecker.asm_improved.type.AsmType;
import de.heinerkuecker.asm_improved.type.generics.param.AsmExtendsTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmAbstractTypeParam;
import de.heinerkuecker.asm_improved.type.generics.param.AsmUnboundedTypeParam;
import de.heinerkuecker.asm_improved.type.generics.var.AsmTypevar;
import de.heinerkuecker.asm_improved.util.Fqn;

/**
 * JUnit4-Testcase fr {@link AsmClasspathInfo}.
 * 
 * @author Heiner K&uuml;cker
 */
public class EnumMapTest
{

	@Test
	public final void test()
	{
		final AsmClassInfo classInfo =
				AllAsmTests.asmClasspathInfo.getClassInfoMandantory(
						EnumMap.class.getCanonicalName() );

		//System.out.println( classInfo.typeParams );

		Assert.assertEquals(
				//expected
				"java.util.EnumMap<K extends java.lang.Enum<K>, V>" ,
				//actual
				classInfo.toString() );

		Assert.assertEquals(
				//expected
				2 ,
				//actual
				classInfo.typeParams.size() );

		{
			final AsmAbstractTypeParam typeParam0 =
					classInfo.typeParams.get( 0 );

			//System.out.println( typeParam0 );

			Assert.assertEquals(
					//expected
					"K" ,
					//actual
					typeParam0.name );

			Assert.assertTrue(
					typeParam0 instanceof AsmExtendsTypeParam );

			final AsmExtendsTypeParam extendsTypeParam0 = (AsmExtendsTypeParam) typeParam0;

			final AsmType extendsTypeParam0BoundType = extendsTypeParam0.boundType;

			Assert.assertTrue(
					extendsTypeParam0BoundType instanceof AsmClassType );

			final AsmClassType extendsTypeParam0ClassBoundType = (AsmClassType) extendsTypeParam0BoundType;

			Assert.assertEquals(
					//expected
					"java.lang.Enum<K>" ,
					//actual
					extendsTypeParam0ClassBoundType.toString() );

			Assert.assertEquals(
					//expected
					new Fqn( Enum.class ) ,
					//actual
					extendsTypeParam0ClassBoundType.fqn );

			Assert.assertTrue(
					extendsTypeParam0ClassBoundType.typeArgs != null &&
							extendsTypeParam0ClassBoundType.typeArgs.size() == 1 );
			
			final AsmType extendsTypeParam0ClassBoundTypeTypeArg = extendsTypeParam0ClassBoundType.typeArgs.get( 0 );

			Assert.assertTrue(
					extendsTypeParam0ClassBoundTypeTypeArg instanceof AsmTypevar );

			final AsmTypevar extendsTypeParam0ClassBoundTypeTypeArgTypevar = (AsmTypevar) extendsTypeParam0ClassBoundTypeTypeArg;

			Assert.assertEquals(
					//expected
					"K" ,
					//actual
					extendsTypeParam0ClassBoundTypeTypeArgTypevar.name );
		}

		{
			final AsmAbstractTypeParam typeParam1 =
					classInfo.typeParams.get( 1 );

			//System.out.println( typeParam1 );

			Assert.assertEquals(
					//expected
					"V" ,
					//actual
					typeParam1.name );

			Assert.assertTrue(
					typeParam1 instanceof AsmUnboundedTypeParam );
		}

		//Assert.fail( "Not yet implemented" );
	}

}
